# Generated for IBM Rational Tau 4.3.0.0.15182 at Wed Sep 30 04:38:17 MEST 2009
# Licensed Materials - Property of IBM 
# (c) Copyright IBM Corporation 2002,2009.  All Rights Reserved.
# 
# U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#########################################################################
# Utility procedures

# Checks if an identifier is empty or not
proc NotEmpty { id } {
  if {$id == 0} {
    return 0
  }
  if {$id == ""} {
    return 0
  }
  return 1
}

# Displays a message dialog with the provided string
proc MessageBox { str } {
	MessageDialog -name "Information" -message $str -style ok
}

#########################################################################
# Very simplified name resolution algorithm
# Does not handle e.g. qualifiers, use dependencies or inheritance
proc LocallyDefined { id scope } {
	if [catch {set signatures [GetA $scope Signature]}] {
		set signatures ""
	}
	foreach sig $signatures {
		set name [GetA $sig Name]
		if {$name == $id} {
			return 1
		}
		if [IsKindOf $sig Interface] {
			set res [LocallyDefined $id $sig]
			if {$res == 1} {
				return 1
			}
		}
	}
	return 0
}

proc Defined { id scope } {
	if [LocallyDefined $id $scope] {
		return 1
	}
	set context [GetA $scope CompositionOwner]
	if [NotEmpty $context] {
		set res [Defined $id $context]
		if {$res == 1} {
			return 1
		}
	}
	return 0
}

proc NotDefined { id scope } {
	return [expr ! [Defined $id $scope]]
}

proc ExtractType { attributeSymbol } {
	set symbolText [GetA $attributeSymbol Text]
	set typeName ""
	
	set attribute [GetA $attributeSymbol Definition]
	set attributeName [GetA $attribute Name]
	regsub ^\ *'?${attributeName}'?\ * $symbolText "" typeName
	set typeName [string trimleft $typeName " :"]
	set typeName [string trim $typeName " '"]
	return $typeName
}

proc UpdateAttribute { symbol } {
	global U2
	set context [GetA [GetA $symbol Diagram] Owner]
	set attrType 0
	set attrName ""
	set attr [GetA $symbol Definition]
        if {$attr != 0} {
		set attrType [GetA $attr Type]
		set attrName [GetA $attr Name]
        } else {
		set attr [CreateIn $context Attribute]
		SetA $symbol Definition $attr
		if [IsKindOf $symbol ActorSymbol] {
			set newStereotype [CreateIn $attr InstanceExpr]
			SetA $newStereotype InstanceOf  [FindByGuid [GetSession $symbol] "@Predefined@actor"]
			AMReport $attr "Create" "Created actor '[GetA $attr Name]' in [Class $context] '[GetA $context Name]'"
		}
	}

	if {$attrType == 0} {
		set attrType [CreateIn $context Class]
		set attrTypeName [ExtractType $symbol]
		if { $attrTypeName != ""} {
			SetA $attrType Name $attrTypeName
		} else {
			if {$attrName != ""} {
				SetA $attrType Name ${attrName}Class
			}
		}
		set attrName [GetA $attrType Name]
		AMReport $attrType "Create" "Created class '$attrName' in [Class $context] '[GetA $context Name]'"
		
		SetA $attr Type $attrName
	}
	return $attr
}

# Returns the full qualifer of an entity. The qualifier does not include 
# the entity name itself. 
# This procedure is supposed to to the same thing as GetFullQualifier in 
# U2UtilityFunctions.h
proc GetFullQualifier { entity } {
	set name ""
	set owner [GetA $entity Owner]
	while {($owner != 0) && (![IsKindOf $owner Session]) } {
		if [IsKindOf $owner Namespace] {
			set qualifier [GetA $owner Name]
			# Add quotes to illegal names
			if ![string is wordchar $qualifier] {
				set qualifier \'${qualifier}\'
			}
			if {$name != ""} {
				set name ${qualifier}::$name
			} else {
				set name $qualifier
			}
		}
		set owner [GetA $owner Owner]
	}
	return $name
}
# Returns a relative qualifier of the entity given a context
# If there are no common upper scopes of entity and context,
# a complete qualifier is returned, otherwise returns the 
# qualifier starting from the nearest common scope.
 
proc GetRelativeQualifier { entity context } {
	set entityPath [SplitQualifier [GetFullQualifier $entity]]
	set contextPath [SplitQualifier [GetFullQualifier $context]]

	set epLen [llength $entityPath]
	set cpLen [llength $contextPath]

	set i 0
	while {	($i < $epLen) && 
		($i < $cpLen) && 
		([lindex $entityPath $i ] == [lindex $contextPath $i]) } {
		incr i
	}
	return [join [lrange $entityPath $i end] ::]

}

# Returns a list where each element is one part of the qualifier
proc SplitQualifier { qualifier } {
	set split ""
	while {[string length $qualifier] > 0} {
		if {[string index $qualifier 0] == "\'" } {
			set wordEnd [string first "\'" $qualifier 1]
			lappend split [string range $qualifier 0 $wordEnd]
			set qualifier [string range $qualifier [expr $wordEnd + 3] end]
		} else {
			set wordEnd [string first :: $qualifier]
			if {$wordEnd != -1} {
				lappend split [string range $qualifier 0 [expr $wordEnd - 1]]
				set qualifier [string range $qualifier [expr $wordEnd + 2] end]
			} else {
				lappend split $qualifier
				set qualifier ""
			}
		}
	}
	return $split
}

# Given a metaclass and a contex, a unique name is returned for
# signatures of the metaclass in the given context. The metaclass 
# name will be used as a starting point. Optionally a name can be 
# supplied as a starting point
#
# TODO: This procedure shall use NamedElement when the new object model
#	version is available.
proc GetUniqueSignatureName { metaclass context {name ""} } {
	set counter 1
	set signatureName $name
	if {$signatureName == ""} {
		set signatureName $metaclass
	}
	switch $metaclass {
		Interface {
			set signatures [GetA $context Signature]
		}
		Attribute {
			set signatures [GetA $context Attribute]
		}
		Port {
			set signatures [GetA $context Port]
		}
		default  {
			set signatures [GetA $context Signature]
		}
	} 
	set signatures [GetA $context Signature]
	set signatureNames ""
	foreach sig $signatures {
		if [IsKindOf $sig $metaclass] {
			lappend signatureNames [GetA $sig Name]
		}
	}
	while {[lsearch -exact $signatureNames ${signatureName}${counter}] != -1} {
		incr counter
	}
	return ${signatureName}${counter}
}


proc GetSession { elem } {
	set session 0
	catch {set session [u2::GetEntity $elem Session]}
	if {0 == $session} {
		set proj [std::GetProject $elem]
		set session [std::GetModels -kind "U2" -project $proj]
	}
	if {0 == $session} {
		error "Unable to locate Session of $elem.\nActive Modeller can only be used on models contained in projects."
	}
	return $session
}
