# Generated for IBM Rational Tau 4.3.0.0.15182 at Wed Sep 30 04:38:17 MEST 2009
# Licensed Materials - Property of IBM 
# (c) Copyright IBM Corporation 2002,2009.  All Rights Reserved.
# 
# U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
###############################################################
#
# Update Model Part and Connector Line
#	
#	The purpose of these procedures is to simplify the creation of a
#	executable architecture. The idea is that only a few
#	editing actions should be needed before it is possible
#	to execute the architecture in the Model Verifier.
#	An an example, do the following:
#	
#	- Create an active class representing the application
#	- Create an architecture diagram inside the application class
#	- Create a few parts.
#	- Choose the "Update Model All" command. This will create the necessary
#	  active classes that type the parts. It will also create statemachines and 
#	  ports for the classes.
#	- Connect the parts with connectors and add some signals on the connectors.
#	- Choose the "Update Model All" command again. This will create the necessary
#	  signal defintions and add the correct signals to the required/realized
#	  interfaces of the correcty ports. 
#	- The application can now be executed!
#	- Run the Model Verifier. Note that to send signals between parts you need to 
#	  specify both the sender and receiver of the signal.
#	
#	
# Known bugs:
#	
#	Uses a TCL coded name resolution approximation in UpdateModelConnectorLine
#	that does not take e.g. use dependencies, qualifiers and inheritance into account
#	This will sometimes give wrong results, resulting in too few or too many
#	signals beeing created.
#	
####################################################################


#########################################################################
# UpdateModel procedures

proc UpdateModelConnectorLine { conSymbol } {
	global U2
	if ![IsKindOf $conSymbol ConnectorLine] {
		error "Element $conSymbol is a '[Class $conSymbol]', but should be a 'ConnectorLine'"
	}

	# Set up context variables
	set diagram [u2::GetEntity $conSymbol Diagram]
	set class [u2::GetEntity $diagram Owner]
	set context [u2::GetEntity $class Owner]
	set con [u2::GetEntity $conSymbol Connector]
	set newCon 0
        
	# If the ConnectorLine is unbound, create new connector
	if {$con == 0} {
		set connectorName [u2::GetValue $conSymbol Text]
		if { "" == $connectorName } {
			set connectorName Connector[expr [llength [u2::GetEntities $class Connector]] + 1]
		}
		SetText $conSymbol $connectorName
		AMReport $conSymbol "Create" "Created connector '$connectorName' in class '[u2::GetValue $class Name]'"
		set con [u2::GetEntity $conSymbol Connector]
		if { 0 == $con } {
			return
		}
		set newCon 1
        }

	set ends [u2::GetEntities $con ConnectorEnd]
	if { 2 != [llength $ends] } {
		return
	}
	set end1 [lindex $ends 0]
        set end2 [lindex $ends 1]

	set port1 [u2::GetEntity $end1 Port]
	set port2 [u2::GetEntity $end2 Port]

	set part1 [u2::GetEntity $end1 Part]
	set part2 [u2::GetEntity $end2 Part]

	# If the connector was created update its labels and enabled 
	# direction from the connected ports
	if {$newCon == 1} {
		if [NotEmpty $port1] {
			set port1required [u2::GetEntities $port1 Required]
			set port1realized [u2::GetEntities $port1 Realized]
		} else {
			set port1required ""
			set port1realized ""
		}
		if [NotEmpty $port2] {
			set port2realized [u2::GetEntities $port2 Realized]
			set port2required [u2::GetEntities $port2 Required]
		} else {
			set port2realized ""
			set port2required ""
		}

		foreach interface $port1required {
			if { $interface == 0 } {
				continue
			}
			if {[lsearch $port2realized $interface] != -1} {
				u2::SetEntity $end2 Carried $interface
			} 
		}
		foreach interface $port1realized {
			if { $interface == 0 } {
				continue
			}
			if {[lsearch $port2required $interface] != -1} {
				u2::SetEntity $end1 Carried $interface
			} 
			
		}
		if {[llength [u2::GetEntities $end1 Carried]] == 0} {
			u2::SetValue $end1 Enabled false
		}
		if {[llength [u2::GetEntities $end2 Carried]] == 0} {
			u2::SetValue $end2 Enabled false
		}
	} else {

		# Check interface lists to find undefined signals
		set ceLabels [u2::GetEntities $conSymbol ConnectorEndLabel]
		foreach ceLabel $ceLabels {
			set labelText [u2::GetValue $ceLabel Text]
			if {"" == $labelText} {
				continue
			}
			set labelInterfaces [split $labelText ,]
			foreach def $labelInterfaces {
				set defT [string trim $def " "]
				if [NotDefined $defT $class] {
					set newSignal [u2::Create $context Signal]
					u2::SetValue $newSignal Name $defT
					AMReport $newSignal "Create" "Created signal '$defT' in '[u2::GetValue $context Name]'"
				}
			}
		}
	
		# Check if the connected ports have the transported signatures in their
		# required/realized lists. If not: Add them
		# Use "Bind" to make sure that everything is bound. GetA will loop if not bound!!!
		u2::Bind $con 

		set portList [list $port1 $port2]
		foreach port $portList {
			if ![NotEmpty $port] {
				continue
			}
			u2::Bind $port
			set realizedInterfaces [u2::GetEntities $port Realized]
			u2::Bind $end1
			set addList ""
			foreach cDef [u2::GetEntities $end1 Carried] {
				set found 0
				foreach rDef $realizedInterfaces {
					if {$cDef == $rDef} {
						set found 1
						break
					}
				}
				if {! $found } {
					lappend addList $cDef
				}
			}
			foreach def $addList {
				u2::SetEntity $port Realized $def
				AMReport $port "Change" "Added signal '[u2::GetValue $def Name]' to realized list"
			}
	
			set requiredInterfaces [u2::GetEntities $port Required]
			u2::Bind $end2
			set addList ""
			foreach cDef [u2::GetEntities $end2 Carried] {
				set found 0
				foreach rDef $requiredInterfaces {
					if {$cDef == $rDef} {
						set found 1
						break
					}
				}
				if {! $found } {
					lappend addList $cDef
				}
			}
			foreach def $addList {
				u2::SetEntity $port Required $def
				AMReport $port "Change" "Added signal '[u2::GetValue $def Name]' to required list"
			}
		}
	}
	u2::Bind $diagram
}
