# Generated for IBM Rational Tau 4.3.0.0.15182 at Wed Sep 30 04:38:17 MEST 2009
# Licensed Materials - Property of IBM 
# (c) Copyright IBM Corporation 2002,2009.  All Rights Reserved.
# 
# U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
###############################################################
#
# Update Model / Architects edition add-in
#	
#	The purpose of this addin is to simplify the creation of a
#	executable architecture. The idea is that only a few
#	editing actions should be needed before it is possible
#	to execute the architecture in the Model Verifier.
#	An an example, do the following:
#	
#	- Create an active class representing the application
#	- Create an architecture diagram inside the application class
#	- Create a few parts.
#	- Choose the "Update Model All" command. This will create the necessary
#	  active classes that type the parts. It will also create statemachines and 
#	  ports for the classes.
#	- Connect the parts with connectors and add some signals on the connectors.
#	- Choose the "Update Model All" command again. This will create the necessary
#	  signal defintions and add the correct signals to the required/realized
#	  interfaces of the correcty ports. 
#	- The application can now be executed!
#	- Run the Model Verifier. Note that to send signals between parts you need to 
#	  specify both the sender and receiver of the signal.
#	
#	
#	This addin actually contains two parts: A general-purpose 
#	UpdateModel framework and an application of this framework to 
#	archictecture diagrams.
#	
#	The general purpose part is defines a two menu commands:
#	
#	- "Update Model": Updates the model for the selected symbol. Shortcut "Ctrl-M"
#	- "Update Model All": Updates the model for all symbols in the current diagram.
#	  Shortcut "Alt-M"
#	
#	When these menu choices are selected a TCL procedure is called that must have the
#	name "UpdateModel" + the name of the symbol. For example, the procedure that updates
#	a connector line should be called "UpdateModelConnectorLine". The procedures should
#	accept one parameter that is a reference to the presentation element representing
#	the symbol.
#	
#	The architecture diagrams specific part of the addin provides UpdateModel procedures
#	for part symbols and connector lines.
#	
#
#
# Known limitations:
#	
#	Only work for parts and connector lines. But can easily be extended!
#		
# Known bugs:
#	
#	Uses a TCL coded name resolution approximation in UpdateModelConnectorLine
#	that does not take e.g. use dependencies, qualifiers and inheritance into account
#	This will sometimes give wrong results, resulting in too few or too many
#	signals beeing created.
#
####################################################################

proc UpdateModelPartSymbol { PartSymbol } {

	if ![IsKindOf $PartSymbol PartSymbol] {
		error "Element $PartSymbol is a '[Class $PartSymbol]', but should be a 'PartSymbol'"
	}

	set attr [UpdateAttribute $PartSymbol]
	set attrType [GetA $attr Type]
	set attrName [GetA $attr Name]
	set owner [GetA $attr Owner]

	if {[GetA $attrType IsActive] == "false" } {
		SetA $attrType IsActive "true"
		AMReport $attrType "Change" "Made class '[GetA $attrType Name]' active"
	}
	if {[GetA $owner IsActive] == "false" } {
		SetA $owner IsActive "true"
		AMReport $attrType "Change" "Made class '[GetA $owner Name]' active"
	}
	if {[GetA $attr Aggregation] != "AkComposite" } {
		SetA $attr Aggregation "AkComposite"
		AMReport $attr "Change" "Made attribute '$attrName' composite"
	}

	# Check if type has ports. If not: Create a port
	set ports [GetA $attrType Port]
        if {$ports == ""} {
		set newPort [CreateIn $attrType Port]
		SetAtt $newPort IsBehaviorPort true		
		AMReport $newPort "Create" "Created port '[GetA $newPort Name]' in '[GetA $attrType Name]'"
		set portSymbol [Create $PartSymbol PortSymbol]
		SetA $portSymbol Definition $newPort
		SetA $portSymbol Autosize true
                set partPos [GetA $PartSymbol Position]
                set xPartPos [lindex $partPos 0]
                set yPartPos [lindex $partPos 1]
                set xPortPos [expr $xPartPos - 20]
                set yPortPos [expr $yPartPos + 20]
                set portPos "$xPortPos $yPortPos"
		SetA $portSymbol Position $portPos
		SetA $portSymbol Size "40 40"
		AddA $PartSymbol PortSymbol $portSymbol 
        }

	# Check if class has statemachine. If not: Create one.
	set signatures [GetA $attrType Signature]
        set foundSM 0
	foreach signature $signatures {
          if [IsKindOf $signature StateMachine] {
            set foundSM 1
          }
        }
        if {$foundSM == 0} {
		# Create diagram
		set tmpStateMachine [CreateIn $attrType StateMachine]
		AMReport $tmpStateMachine "Create" "Created statemachine '[GetA $tmpStateMachine Name]' in '[GetA $attrType Name]'"
		set tmpSMImpl [CreateIn $tmpStateMachine SimpleStateMachine]
		set tmpSMDiag [CreateIn $tmpStateMachine StatechartDiagram]
		# Create state
		set tmpState [CreateIn $tmpSMImpl State]
		SetAtt $tmpState Name "Idle"
		# Create first state symbol
		set stateSymbol [CreateIn $tmpSMDiag StateSymbol]
		SetText $stateSymbol "Idle"
		SetA $stateSymbol Position "600 400"
		# Create input symbol
		set inputSymbol [CreateIn $tmpSMDiag InputSymbol]
		SetText $inputSymbol "*"
		SetA $inputSymbol Position "600 600"
		Connect FlowLine $stateSymbol $inputSymbol
		# Create next state symbol
		set stateSymbol3 [CreateIn $tmpSMDiag StateSymbol]
		SetText $stateSymbol3 "Idle"
		SetA $stateSymbol3 Position "600 800"
		Connect FlowLine $inputSymbol $stateSymbol3
		# Create start symbol
		set startSymbol [CreateIn $tmpSMDiag StartSymbol]
		SetA $startSymbol Position "280 400"
		# Create next state symbol
		set stateSymbol2 [CreateIn $tmpSMDiag StateSymbol]
		SetText $stateSymbol2 "Idle"
		SetA $stateSymbol2 Position "200 600"
		Connect FlowLine $startSymbol $stateSymbol2
	}

}
