// Generated for IBM Rational Tau 4.3.0.0.15182 at Wed Sep 30 04:38:20 MEST 2009
// Licensed Materials - Property of IBM 
// (c) Copyright IBM Corporation 2002,2009.  All Rights Reserved.
// 
// U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#ifndef TOR_NO_COMMUNICATION

#include "torPlatform.h"

#include "torCommunication.h"
#include "torExceptions.h"
#include <assert.h>
#include <algorithm>

using namespace std;

namespace tor
{
    using namespace os;

    //////////////////////////////////////////////////////////////////////////
    // 
    //////////////////////////////////////////////////////////////////////////
    
    ConnectionData::ConnectionData()
      : m_mutex(),
        m_is_thread_running(false),
        m_ended(),
        m_behaviourId(0),
        m_useSemaphore(true),
        m_added_sockets(),
        m_removed_sockets(),
        m_internal_server(invalid_socket),
        m_internal_server_(invalid_socket),
        m_internal_client(invalid_socket),
        m_client(invalid_socket),
        m_server(invalid_socket)
    {
        m_mutex.init();
        m_ended.init(0);

        try 
        {
            tcpip::init();
            setupInternalConnection();
        }
        catch(...)
        {
            // Failed to set up the internal connection, can not continue...

            if (m_internal_client != invalid_socket)
            {
                tcpip::close(m_internal_client);
                m_internal_client = invalid_socket;
            }
            if (m_internal_server != invalid_socket)
            {
                tcpip::close(m_internal_server);
                m_internal_server = invalid_socket;
            }
            if (m_internal_server_ != invalid_socket)
            {
                tcpip::close(m_internal_server_);
                m_internal_server_ = invalid_socket;
            }

            throw;
        }
    }

    ConnectionData::~ConnectionData()
    {
        if (m_internal_client != invalid_socket)
        {
            tcpip::close(m_internal_client);
            m_internal_client = invalid_socket;
        }
        if (m_internal_server != invalid_socket)
        {
            tcpip::close(m_internal_server);
            m_internal_server = invalid_socket;
        }
        if (m_internal_server_ != invalid_socket)
        {
            tcpip::close(m_internal_server_);
            m_internal_server_ = invalid_socket;
        }
    }

    void ConnectionData::endConnection()
    {
        {
            auto_lock lock(m_mutex);
            
            if (!_isRunning())
                return;
            if (!_internalSend(1)) // quit
                throw Exception(failed_to_terminate_connection_thread);
            if (m_behaviourId == Thread::id()) // Is this a recursive (thread local) call
            {
                m_useSemaphore = false;
                return;
            }
        }
        m_ended.get(); // Wait for thread to terminate
    }

    void ConnectionData::addConnection(SOCKET s)
    {
        auto_lock lock(m_mutex);
        m_added_sockets.push_back(s); 
        if (!_internalSend(2)) // wakeup
            throw Exception(could_not_contact_the_communication_thread);
    }

    void ConnectionData::removeConnection(SOCKET s)
    {
        auto_lock lock(m_mutex);
        m_removed_sockets.push_back(s);
        if (!_internalSend(2)) // wakeup
            throw Exception(could_not_contact_the_communication_thread);
    }

    void ConnectionData::setRunning(bool v)
    {
        {
            auto_lock lock(m_mutex);
            m_is_thread_running = v;

            if (v)
            {
                m_behaviourId = Thread::id();
                return;
            }

            if (!m_useSemaphore)
                return;
        }
        m_ended.put();
    }

    bool ConnectionData::isRunning() const
    {
        auto_lock lock(m_mutex);
        return _isRunning();
    }

    SOCKET ConnectionData::getInternalServer() const
    {
        auto_lock lock(m_mutex);
        return m_internal_server;
    }

    SOCKET ConnectionData::getInternalClient() const
    {
        auto_lock lock(m_mutex);
        return m_internal_client;
    }

    void ConnectionData::updateConnections(std::vector<SOCKET> &sockets)
    {
        auto_lock lock(m_mutex);

        // Add requested connections
        if (!m_added_sockets.empty())
        {
            copy(m_added_sockets.begin(), m_added_sockets.end(), back_inserter(sockets));
            m_added_sockets.clear();
        }

        // Remove requested connections

        if (!m_removed_sockets.empty())
        {
            vector<SOCKET>::iterator i = m_removed_sockets.begin();
            while (i != m_removed_sockets.end())
            {
                vector<SOCKET>::iterator j = find(sockets.begin(), sockets.end(), *i++);
                if (j != sockets.end())
                {
                    tcpip::close(*j);
                    sockets.erase(j);
                }
            }
            m_removed_sockets.clear();
        }
    }

    void ConnectionData::setClient(os::SOCKET s)
    {
        auto_lock lock(m_mutex);
        m_client = s;
    }

    SOCKET ConnectionData::getClient() const
    {
        auto_lock lock(m_mutex);
        return m_client;
    }

    void ConnectionData::setServer(os::SOCKET s)
    {
        auto_lock lock(m_mutex);
        m_server = s;
    }

    SOCKET ConnectionData::getServer() const
    {
        auto_lock lock(m_mutex);
        return m_server;
    }

    void ConnectionData::setupInternalConnection()
    {
        auto_lock lock(m_mutex);

        // Setup an internal server and client socket

        unsigned int count = 0, port = 56100;
        do 
        {
            m_internal_server_ = tcpip::create_server_socket(port++, false);
            ++count;
        }
        while (m_internal_server_ == invalid_socket && count < 50);
        if (m_internal_server_ == invalid_socket)
            throw Exception(connectiondata__failed_to_set_up_internal_server);
        --port;

        m_internal_client = tcpip::create_client_socket("localhost", port);
        if (m_internal_client == invalid_socket)
            throw Exception(connectiondata__failed_to_set_up_internal_client);

        m_internal_server = tcpip::accept(m_internal_server_);
        if (m_internal_server == invalid_socket)
            throw Exception(connectiondata__failed_accept_connection_from_internal_client);

        m_added_sockets.push_back(m_internal_server); 
    }

    bool ConnectionData::_internalSend(char data)
    {
        return tcpip::send_bytes(m_internal_client, &data, 1);
    }

    bool ConnectionData::_isRunning() const
    {
        return m_is_thread_running; 
    }

    //////////////////////////////////////////////////////////////////////////
    // ConnectionBehavoiur
    //////////////////////////////////////////////////////////////////////////
    
    ConnectionBehaviour::ConnectionBehaviour()
      : Thread::Behaviour(),
        m_data()
    {}

    int ConnectionBehaviour::execute()
    {
        m_data->setRunning(true);

        std::vector<SOCKET> sockets;
        m_data->updateConnections(sockets);

        while (true)
        {
            vector<SOCKET> incomming;
            if (!tcpip::wait_for_data(sockets, incomming))
                break;

            vector<SOCKET>::iterator i = incomming.begin();
            while (i != incomming.end())
            {
                SOCKET s = *i++;

                if (s == m_data->getInternalServer())
                {
                    char data;
                    if (!tcpip::receive_bytes(m_data->getInternalServer(), &data, 1))
                        goto exit; // Break out of two loops

                    if (data == 1) // quit
                        goto exit; // Break out of two loops

                    if (data == 2) // wakeup
                        m_data->updateConnections(sockets);

                    continue;
                }

                onIncommingData(s);
            }
        }
exit:
        sockets.erase(find(sockets.begin(), sockets.end(), m_data->getInternalServer()));
        for_each(sockets.begin(), sockets.end(), tcpip::close);
        sockets.clear();

        m_data->setRunning(false);
        return 0;
    }

    const Thread::Data<ConnectionData> &ConnectionBehaviour::getData() const
    {
        return m_data;
    }

    void ConnectionBehaviour::addConnection(SOCKET s)
    {
        m_data->addConnection(s);
    }

    void ConnectionBehaviour::removeConnection(SOCKET s)
    {
        m_data->removeConnection(s);
    }

    void ConnectionBehaviour::setServer(SOCKET s)
    {
        m_data->setServer(s);
    }

    SOCKET ConnectionBehaviour::getServer() const
    {
        return m_data->getServer();
    }

    void ConnectionBehaviour::setClient(SOCKET s)
    {
        m_data->setClient(s);
    }

    SOCKET ConnectionBehaviour::getClient() const
    {
        return m_data->getClient();
    }

    // ThreadedConnection

    ThreadedConnection::ThreadedConnection(const Thread::Data<ConnectionData> &data) 
      : m_data(data) 
    {}

    ThreadedConnection::~ThreadedConnection()
    {
        m_data->endConnection();

    }

    bool ThreadedConnection::isRunning() const
    {
        return m_data->isRunning();
    }

    void ThreadedConnection::addConnection(SOCKET s)
    {
        m_data->addConnection(s);
    }

    void ThreadedConnection::removeConnection(SOCKET s)
    {
        m_data->removeConnection(s);
    }

    void ThreadedConnection::setServer(SOCKET s)
    {
        m_data->setServer(s);
    }

    SOCKET ThreadedConnection::getServer() const
    {
        return m_data->getServer();
    }

    void ThreadedConnection::setClient(SOCKET s)
    {
        m_data->setClient(s);
    }

    SOCKET ThreadedConnection::getClient() const
    {
        return m_data->getClient();
    }

    //////////////////////////////////////////////////////////////////////
    //
    //////////////////////////////////////////////////////////////////////

    ServerBehaviour::ServerBehaviour(unsigned int port)
      : ConnectionBehaviour()
    {
        SOCKET server = tcpip::create_server_socket(port, true);
        if (server == invalid_socket)
            return; // ERROR
        addConnection(server); // TODO: Check this
        setServer(server);
    }

    void ServerBehaviour::onIncommingData(SOCKET s)
    {
        SOCKET server = getServer();
        if (s == server)
        {
            SOCKET client = tcpip::accept(server);
            if (client == invalid_socket)
                return; // ERROR

            if (onConnect(client))
                addConnection(client);
            else
                tcpip::close(client);
            
            return;
        }

        // Communication from one of our clients
        onIncommingClientData(s);
    }

    bool ServerBehaviour::onConnect(SOCKET s)
    {
        return true;
    }

    void ServerBehaviour::onDisconnect(SOCKET s)
    {
    }

    //

    TextServerBehaviour::TextServerBehaviour(unsigned int port)
      : ServerBehaviour(port)
    {}

    bool TextServerBehaviour::onConnect(SOCKET s)
    {
        if (getClient() != invalid_socket)
            return false; // Denied

        setClient(s);
        return true;
    }

    void TextServerBehaviour::onDisconnect(SOCKET s)
    {
        if (s == getClient())
            setClient(invalid_socket);
        removeConnection(s);
    }

    void TextServerBehaviour::onIncommingClientData(SOCKET s)
    {
        // Communication from one of our clients

        int length = 0;
        if (!tcpip::receive_bytes(s, reinterpret_cast<char *>(&length), sizeof(length)))
        {
            onDisconnect(s);
            return;
        }

        char *buffer = new char [length + 1];

        if (!tcpip::receive_bytes(s, buffer, length))
        {
            onDisconnect(s);
            delete [] buffer;
            return;
        }
        buffer[length] = '\0';

        string str(buffer);
        delete [] buffer;

        Charstring message(str);
        onReceivedMessage(message);
    }

    //

    TextServer::TextServer(TextServerBehaviour *behaviour) 
      : ThreadedConnection(behaviour->getData())
    {
        Thread::launch(behaviour);
    }

    bool TextServer::sendMessage(const Charstring &message)
    {
        return tcpip::send_text(getClient(), message.getNarrowString());
    }

    //////////////////////////////////////////////////////////////////////
    // Client
    //////////////////////////////////////////////////////////////////////

    ClientBehaviour::ClientBehaviour(const char *host, unsigned int port)
      : ConnectionBehaviour()
    {
        // If we don't get a connection we assume that the server
        // has not started yet. Therefore we wait for some time and try 
        // again. We do this up to a certain limit and then we give up.

        unsigned int time_to_wait = 3;
        char *t=NULL;

        // Env. variable TTD_TOR_WAIT_TO_CONNECT can be used to set the max time to wait
#if defined(WIN32) && !defined(MAINWIN)
        size_t sz;
        getenv_s(&sz,NULL,0,"TTD_TOR_WAIT_TO_CONNECT"); // get size

        if(sz>0)
        {
          t=new char[sz];
          getenv_s(&sz,t,sz,"TTD_TOR_WAIT_TO_CONNECT");
        }
#else
        t = getenv("TTD_TOR_WAIT_TO_CONNECT");
#endif
        if (t != NULL)
        {
            time_to_wait = atoi(t);
            if (time_to_wait < 3)
                time_to_wait = 3;

#if defined(WIN32) && !defined(MAINWIN)
            delete []t;
#endif
        }


        SOCKET server = invalid_socket;
        unsigned int count = 0;
        while (server == invalid_socket && count < time_to_wait * 5)
        {
            server = tcpip::create_client_socket(host, port);
            if (server == invalid_socket)
                Thread::suspend(Time::now().add(0, 200000000L));
            ++count;
        }
        if (server == invalid_socket)
            return; // ERROR

        addConnection(server);
        setServer(server);
    }

    void ClientBehaviour::onDisconnect()
    {
    }
    
    //
    
    TextClientBehaviour::TextClientBehaviour(const char *host, unsigned int port)
      : ClientBehaviour(host, port)
    {}

    void TextClientBehaviour::onIncommingData(SOCKET s)
    {
        string text;
        if (!tcpip::receive_text(s, text))
        {
            onDisconnect();
            removeConnection(s);
            return;
        }

        Charstring message(text);
        onReceivedMessage(message);
    }

    //

    TextClient::TextClient(TextClientBehaviour *behaviour) 
      : ThreadedConnection(behaviour->getData())
    {
        Thread::launch(behaviour);
    }

    bool TextClient::sendMessage(const Charstring &message)
    {
        return tcpip::send_text(getServer(), message.getNarrowString());
    }

    bool ConnectionHelper::IsPortOccupied(unsigned int port) {
        SOCKET server = tcpip::create_server_socket(port, false);
        if (server == invalid_socket)
            return true;
        tcpip::close(server);
        return false;
    }
}

#endif

