/*************************************************************
Generated for IBM Rational Tau 4.3.0.0.15182 at Wed Sep 30 04:38:22 MEST 2009
Licensed Materials - Property of IBM 
(c) Copyright IBM Corporation 2002,2009.  All Rights Reserved.

U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*************************************************************/
#ifndef __torCommunication_h
#define __torCommunication_h

#ifndef TOR_NO_COMMUNICATION

#include "torPlatform.h"

#include "torOsRep.h"
#include "torSync.h"
#include "torThread.h"
#include "torTypes.h"
#include <vector>
#include <functional>

namespace tor 
{
    using namespace os;

    //////////////////////////////////////////////////////////////////////
    //
    //////////////////////////////////////////////////////////////////////

    class TOR_EXT_DEF ConnectionData
    {
    public:
        ConnectionData();
        ~ConnectionData();
        
        void endConnection();

        void addConnection(SOCKET s);
        void removeConnection(SOCKET s);

        void setRunning(bool v);
        bool isRunning() const;

        SOCKET getInternalServer() const;
        SOCKET getInternalClient() const;

        void updateConnections(std::vector<SOCKET> &sockets);

        void setClient(SOCKET s);
        SOCKET getClient() const;

        void setServer(SOCKET s);
        SOCKET getServer() const;
        
    protected:
        mutable Mutex m_mutex;

    private:
        void setupInternalConnection();

        bool _internalSend(char data);
        bool _isRunning() const;

    private:
        bool m_is_thread_running;
        Semaphore m_ended;
        unsigned long m_behaviourId;
        bool m_useSemaphore;

        std::vector<SOCKET> m_added_sockets;
        std::vector<SOCKET> m_removed_sockets;

        SOCKET m_internal_server;
        SOCKET m_internal_server_;
        SOCKET m_internal_client;

        SOCKET m_client;
        SOCKET m_server;
    };

    //

    class TOR_EXT_DEF ConnectionBehaviour : public Thread::Behaviour
    {
    public:
        ConnectionBehaviour();
        virtual ~ConnectionBehaviour() {}

        virtual int execute();

        const Thread::Data<ConnectionData> &getData() const;

        void addConnection(SOCKET s);
        void removeConnection(SOCKET s);

        void setServer(SOCKET s);
        SOCKET getServer() const;

        void setClient(SOCKET s);
        SOCKET getClient() const;
        
    protected:
        virtual void onIncommingData(SOCKET s) = 0;

    private:
        Thread::Data<ConnectionData> m_data;
    };

    //

    class TOR_EXT_DEF ThreadedConnection
    {
    public:
        ThreadedConnection(const Thread::Data<ConnectionData> &data);
        virtual ~ThreadedConnection();

        bool isRunning() const;
        SOCKET getServer() const; // Can be used to know if the connection is valid.

    protected:
        void addConnection(SOCKET s);
        void removeConnection(SOCKET s);

        void setServer(SOCKET s);        

        void setClient(SOCKET s);
        SOCKET getClient() const;

    private:
        Thread::Data<ConnectionData> m_data;
    };

    //////////////////////////////////////////////////////////////////////
    //
    //////////////////////////////////////////////////////////////////////

    class TOR_EXT_DEF ServerBehaviour : public ConnectionBehaviour
    {
    public:
        ServerBehaviour(unsigned int port);

    protected:
        virtual void onIncommingData(SOCKET s);
        virtual void onIncommingClientData(SOCKET s) = 0;

        virtual bool onConnect(SOCKET s);
        virtual void onDisconnect(SOCKET s);
    };

    //

    class TOR_EXT_DEF TextServerBehaviour : public ServerBehaviour
    {
    public:
        TextServerBehaviour(unsigned int port);

        virtual void onReceivedMessage(const Charstring &message) = 0;

    protected:
        virtual bool onConnect(SOCKET s);
        virtual void onDisconnect(SOCKET s);

        virtual void onIncommingClientData(SOCKET s);
    };

    //

    class TOR_EXT_DEF TextServer : public ThreadedConnection
    {
    public:
        TextServer(TextServerBehaviour *behaviour);

        virtual bool sendMessage(const Charstring &message);

    protected:
        SOCKET m_client;
    };

    class TOR_EXT_DEF ClientBehaviour : public ConnectionBehaviour
    {
    public:
        ClientBehaviour(const char *host, unsigned int port);

    protected:
        virtual void onDisconnect();
    };

    //

    class TOR_EXT_DEF TextClientBehaviour : public ClientBehaviour
    {
    public:
        TextClientBehaviour(const char *host, unsigned int port);

        virtual void onReceivedMessage(const Charstring &message) = 0;

    protected:
        virtual void onIncommingData(SOCKET s);
    };

    //

    class TOR_EXT_DEF TextClient : public ThreadedConnection
    {
    public:
        TextClient(TextClientBehaviour *behaviour);

        virtual bool sendMessage(const Charstring &message);
    };

    class TOR_EXT_DEF ConnectionHelper {
    public:
        static bool IsPortOccupied(unsigned int port);
    };
}

#endif
#endif

