// Generated for IBM Rational Tau 4.3.0.0.15182 at Wed Sep 30 04:38:20 MEST 2009
// Licensed Materials - Property of IBM 
// (c) Copyright IBM Corporation 2002,2009.  All Rights Reserved.
// 
// U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#if defined(TOR_USE_INSTRUMENTATION) && !defined(TOR_NO_EVENTMANAGER)

#include "torPlatform.h"

#include "tor.h"
#include "torTypes.h"
#include "torEventManager.h"
#include "torMetaEvents.h"
#include "torMetaStackContext.h"
#include "torEventManagerImpl.h" // Generated statemachine implementation
#include "torOs.h"
#include <assert.h>

namespace tor {

    namespace meta {

        // Representation of a pseudo-breakpoint, i.e. a breakpoint used internally by the eventManager in order
        // to implement debugger features such as "Break", "Step", "StepIn", "StepOut", "RunTo" etc.
        // N.B. Currently thread-focused pseudo-breakpoints are not supported.
        struct PseudoBreakpoint {
            // Returns true if the pseudo-breakpoint applies to the specified action when executed
            // by the specified thread. If it applies, execution should be halted at its location.
            // If the action already has been partially executed (in case the action contains operation calls)
            // the partiallyAlreadyExecuted flag should be set to true, otherwise false.
            virtual bool AppliesTo(const Charstring& actionGuid,
                ThreadInfo& threadInfo,
                bool partiallyAlreadyExecuted) = 0;

            virtual ~PseudoBreakpoint() {}
        };

        // Break on the next action to execute for any thread
        struct BreakOnNextAction : PseudoBreakpoint {
            virtual bool AppliesTo(const Charstring& actionGuid,
                ThreadInfo& threadInfo,
                bool partiallyAlreadyExecuted) {
                    return true;
            }

            virtual ~BreakOnNextAction() {}
        };

        // Break on a specific action for a certain thread
        struct BreakOnSpecificAction : PseudoBreakpoint {
            BreakOnSpecificAction(const Charstring& actionGuid, int threadId)
                : m_actionGuid(actionGuid), m_threadId(threadId) {}

            virtual bool AppliesTo(const Charstring& actionGuid,
                ThreadInfo& threadInfo,
                bool partiallyAlreadyExecuted) {
                    if (threadInfo.m_id != m_threadId)
                        return false; // Not the correct thread

                    return (actionGuid == m_actionGuid);
            }

            virtual ~BreakOnSpecificAction() {}

            Charstring m_actionGuid;
            int m_threadId;
        };

        // Break on the next action to execute that fulfills a condition on the stack level
        // for a certain thread
        struct BreakOnNextActionWithStackCondition : PseudoBreakpoint {
            enum Condition {
                SameOrOuter,    // Stack level must be less than or equal to the specified level (Step)
                Any,            // Any stack level allowed (StepIn)
                Outer           // Stack level must be less than the specified level (StepOut)
            };

            BreakOnNextActionWithStackCondition(unsigned int stackLevel, int threadId, Condition condition)
                : m_stackLevel(stackLevel), m_threadId(threadId), m_condition(condition) {}

            virtual bool AppliesTo(const Charstring& actionGuid,
                ThreadInfo& threadInfo,
                bool partiallyAlreadyExecuted) {
                    if (threadInfo.m_id != m_threadId)
                        return false; // Not the correct thread

                    if (m_condition == Any)
                        return true;

                    unsigned int stackLevel = static_cast<unsigned int>(threadInfo.m_callstack.size());

                    if (m_condition == SameOrOuter)
                        return (stackLevel <= m_stackLevel) && !partiallyAlreadyExecuted;

                    assert(m_condition == Outer);
                    return (stackLevel < m_stackLevel);
            }

            virtual ~BreakOnNextActionWithStackCondition() {}

            unsigned int m_stackLevel;
            int m_threadId;
            Condition m_condition;
        };


        //////////////////////////////////////////////////////////////////////////
        // EventManager implementation

        EventManager* EventManager::m_pInstance=NULL;
        os::Mutex EventManager::m_mutex;

        // The purpose of EventManagerInit is to force the event manager to be initialized
        // before and finalized after any object in the user application!

        static struct EventManagerInit {
            EventManagerInit()  { EventManager::initialize(); }
            ~EventManagerInit() { EventManager::finalize(); }
        } init_instance;

        EventManager::EventManager()
        {
            m_pseudoBreakpoint=NULL;
            emImpl=NULL;

            active = true;
#ifdef TOR_SYNC_META_EVENTS
            synchronous = true;
#else
            synchronous = false;
#endif
        }

        EventManager::~EventManager()
        {
            if (coder && (coder != &emImpl->defaultCoder) ) delete coder;

            if (emImpl) { delete emImpl; emImpl=NULL; }
        }

        void EventManager::finish_initialization()
        {
            emImpl = new EventManagerImpl;
            coder = &emImpl->defaultCoder;
        }

        void EventManager::initialize()
        {
            if(m_pInstance) return; // already initialized

            m_mutex.init();

            os::auto_lock lock(m_mutex);

            m_pInstance=new EventManager();
            m_pInstance->finish_initialization();
        }

        void EventManager::finalize()
        {
            if(!m_pInstance) return;    // already deleted

            // We may safely use m_pInstance here without synchronization because it can only be deleted
            // by this function (see below).

            EventManager* pEM = m_pInstance;
            { // synchronize
                os::auto_lock lock(m_mutex);

                m_pInstance=NULL;
            } // end synchronize

            delete pEM;

            // don't delete m_mutex since other threads could try using EventManager locking on m_mutex!
        }

        bool EventManager::receive(tor::Event* e)
        {
            // Just pass the event on to the event manager implementation.
            if(emImpl) return emImpl->receive(e);
        }

        void EventManager::registerMedia(EventMedia* m)
        {
            os::auto_lock sync(m_mutex);

            if(!m_pInstance) return;

            m_pInstance->emImpl->media.append(m);
        }

        void EventManager::unregisterMedia(EventMedia* m)
        {
            os::auto_lock lock(m_mutex);
            if(!m_pInstance) return;

            const tor::Natural length=m_pInstance->emImpl->media.length();
            for (tor::Natural i = 1; i <= length ; ++i )
            {
                if (m_pInstance->emImpl->media[i] == m)
                {
                    m_pInstance->emImpl->media.remove(i);
                    break;
                }
            }
        }

        void EventManager::emitToMedia(Event* e) {
            os::auto_lock lock(m_mutex);
            if(!m_pInstance) return;

            const tor::Natural length=m_pInstance->emImpl->media.length();
            for (tor::Natural i = 1; i <= length; ++i) {
                m_pInstance->emImpl->media[i]->emit(e->encodedInfo);
            }
        }

        void EventManager::deleteAllLogFiles()
        {
            os::auto_lock lock(m_mutex);
            if(!m_pInstance) return;

            tor::Natural length=m_pInstance->emImpl->media.length();
            for (tor::Natural i = 1; i <= length;) {
                EventMedia* media = m_pInstance->emImpl->media[i];

                if (dynamic_cast<LogFile*>(media)) {
                    m_pInstance->emImpl->media.remove(i);
                    delete media;
                    --length;
                }
                else
                    ++i;
            }
        }

        void EventManager::deleteAllHostTracers()
        {
            os::auto_lock lock(m_mutex);
            if(!m_pInstance) return;

            tor::Natural length=m_pInstance->emImpl->media.length();
            for (tor::Natural i = 1; i <= length;) {
                EventMedia* media = m_pInstance->emImpl->media[i];

                if (dynamic_cast<TauHostTracer*>(media)) {
                    m_pInstance->emImpl->media.remove(i);
                    delete media;
                    --length;
                }
                else
                    ++i;
            }
        }

        void EventManager::createNewLogFile(const char* path)
        {
            tor::Charstring strPath( path );
            new LogFile(strPath);
        }

        void EventManager::createNewHostTracer()
        {
#if _MSC_VER
            new TauHostTracer();
#endif
        }

        void EventManager::setCoder(Coder* c)
        {
            os::auto_lock lock(m_mutex);
            if(!m_pInstance) return;

            m_pInstance->coder = c;
        }

        Coder* EventManager::getDefaultCoder()
        {
            os::auto_lock lock(m_mutex);
            if(!m_pInstance) return NULL;

            return &m_pInstance->emImpl->defaultCoder;
        }

        void EventManager::setActive(tor::Boolean a)
        {
            os::auto_lock lock(m_mutex);
            if(!m_pInstance) return;

            m_pInstance->active = a;
        }

        bool EventManager::isActive()
        {
            os::auto_lock lock(m_mutex);
            if(!m_pInstance) return false;  // finalized => not active

            return m_pInstance->active;
        }

        bool EventManager::isSynchronous()
        {
            os::auto_lock lock(m_mutex);
            if(!m_pInstance) return false;  // don't know

            return m_pInstance->synchronous;
        }

        void EventManager::processEvent(tor::meta::Event* event) {
//            os::auto_lock lock(m_mutex); deadlocks!
            if (!m_pInstance || !isActive()) return;

            int threadId = os::Thread::id();
            ThreadInfo* threadInfo = getThreadInfo(threadId);

            // Different events require different handling here...
            if (cast<BreakpointHit_Charstring*>(event)) {
                // The current thread has reached a breakpoint. Block all other threads until execution
                // shall be resumed.
                blockAllThreadsExceptOne(threadId);
            }

            event->encode(m_pInstance->coder); // To get a string representation of the event (stored in the event itself)
            if (event->m_bSynchronouslySent) {
                threadInfo->m_gate.lock();
                tor::sendTo(event, m_pInstance->emImpl);
                threadInfo->m_gate.enter(); // Wait until the event has been processed (then the gate is opened)
            }
            else
                // Asynchronous sending of the event
                tor::sendTo(event, m_pInstance->emImpl);
        }

        ThreadInfo* EventManager::getThreadInfo(int threadId) {
            os::auto_lock lock(m_mutex);
            if(!m_pInstance) return NULL;

            return m_pInstance->_getThreadInfo(threadId);
        }


        void tor::meta::EventManager::beforeAction(const Charstring& strGuid, bool partiallyAlreadyExecuted /* = false */) {
            if (isFinalized())
                // The eventManager is finalized (probably explicitly finalized). Do nothing.
                return;

            bool bBreakpointHit = false;

            { // Start synchronized region
                os::auto_lock lock(m_mutex);
                if(!m_pInstance) return; // could be possible finalized after the check above

                int threadId = os::Thread::id();
                ThreadInfo* threadInfo = m_pInstance->_getThreadInfo(threadId);

                if (threadInfo->m_callstack.empty()) {
                    // This should normally not happen! How could we execute an action without having a callstack!?!
                    // However, one explanation could be that TOR was just initialized so that no stack frames
                    // exist yet. 'beforeAction' will only have an effect after at least one stack frame exist.
                    // Another explanation could be that the generated instrumentation was modified by the user.
                    return;
                }

                // Set up information about current execution point
                StackFrame& stackFrame = threadInfo->m_callstack.top();
                stackFrame.m_actionGuid = strGuid;

                // Check if we should break on this action.

                // First check for applicable pseudo-breakpoint
                if (m_pInstance->m_pseudoBreakpoint) {
                    if (m_pInstance->m_pseudoBreakpoint->AppliesTo(strGuid, *threadInfo, partiallyAlreadyExecuted))
                        // Pseudo-breakpoint hit
                        bBreakpointHit = true;
                }

                if (!bBreakpointHit) {
                    // No pseudo-breakpoint! Next check for applicable ordinary breakpoint
                    BreakpointMap::const_iterator it = m_pInstance->m_breakpointMap.find(strGuid);
                    if (it != m_pInstance->m_breakpointMap.end()) {
                        // Ordinary breakpoint hit.
                        bBreakpointHit = true;

                        // Make the current thread part of the active debug context (i.e. we now start to debug that thread if not already done)
                        threadInfo->m_active = true;
                    }
                }

            } // End synchronized region

            if (bBreakpointHit) {
                // N.B. No signal sending within synchronized region, or we will be dead-locked!
                processEvent(new BreakpointHit_Charstring(strGuid));
            }
            else {
                // Finally, check if the current thread shall be blocked. If so, block it now.
                int threadId = os::Thread::id();
                ThreadInfo* threadInfo = getThreadInfo(threadId);
                threadInfo->m_gate.enter();
            }
        }

        //////////////////////////////////////////////////////////////////////////


        ThreadInfo* EventManager::_getThreadInfo(int threadId) {
            ThreadInfos::const_iterator it = emImpl->threadInfos.find(threadId);
            if (it != emImpl->threadInfos.end())
                return (*it).second;

            ThreadInfo* threadInfo = new ThreadInfo(threadId);
            emImpl->threadInfos.insert(ThreadInfos::value_type(threadId, threadInfo));

            return threadInfo;
        }

        ThreadInfo* EventManager::_getActiveThread() {
            if (emImpl->threadInfos.size() == 0)
                return 0; // Strange. There should at least be one user thread.

            ThreadInfo* threadInfo = 0;
            for (ThreadInfos::iterator it = emImpl->threadInfos.begin();
                it != emImpl->threadInfos.end(); it++)
            {
                ThreadInfo* ti = (*it).second;
                if (ti->m_active) {
                    threadInfo = ti;
                    break;
                }
            }

            if (!threadInfo) {
                // No thread is in the active debug context. Make the first one active.
                ThreadInfos::iterator it = emImpl->threadInfos.begin();
                ThreadInfo* ti = (*it).second;
                ti->m_active = true;
                threadInfo = ti;
            }

            return threadInfo;
        }


        void EventManager::setBreakpoints(String<BreakpointInfo>& breakpointsToSet) {
            os::auto_lock lock(m_mutex);
            if(!m_pInstance) return;

            for (String<BreakpointInfo>::iterator it = breakpointsToSet.begin();
                it != breakpointsToSet.end(); ++it)
            {
                BreakpointInfo& bpInfo = *it;
                m_pInstance->m_breakpointMap.insert(BreakpointMap::value_type(bpInfo.actionGuid, bpInfo));
            }
        }

        void EventManager::removeBreakpoints(String<BreakpointInfo>& breakpointsToRemove) {
            os::auto_lock lock(m_mutex);
            if(!m_pInstance) return;

            if (breakpointsToRemove.empty()) {
                // Remove all breakpoints
                m_pInstance->m_breakpointMap.clear();
                return;
            }

            for (String<BreakpointInfo>::iterator it = breakpointsToRemove.begin();
                it != breakpointsToRemove.end(); ++it)
            {
                BreakpointInfo& bpInfo = *it;
                m_pInstance->m_breakpointMap.erase(bpInfo.actionGuid);
            }
        }

        void EventManager::blockAllThreadsExceptOne(int threadId) {
            os::auto_lock lock(m_mutex);
            if(!m_pInstance) return;

            for (ThreadInfos::iterator it = m_pInstance->emImpl->threadInfos.begin();
                it != m_pInstance->emImpl->threadInfos.end(); ++it)
            {
                ThreadInfo* ti = (*it).second;
                if (ti->m_id != threadId)
                    ti->m_gate.lock(); // Block the thread by locking its gate
            }
        }

        void EventManager::continueApplication(tor::Event* event) {
            os::auto_lock lock(m_mutex);
            if(!m_pInstance) return;

            Event* e = cast<Event*>(event);
            if (!e) {
                assert(0); // Should only process meta events here...
                return;
            }

            if (!e->m_bSynchronouslySent)
                return; // Nothing to do. The application is not waiting for an ack.

            for (ThreadInfos::iterator it = m_pInstance->emImpl->threadInfos.begin();
                it != m_pInstance->emImpl->threadInfos.end(); ++it++)
            {
                ThreadInfo* ti = (*it).second;
                ti->m_gate.unlock();
            }
        }

        tor::Charstring& EventManager::getCurrentExecutionPoint() {
            os::auto_lock lock(m_mutex);
            if(!m_pInstance) {
                static Charstring error(_T("Error! No EventManager available!")); // HCS
                return error;
            }

            ThreadInfo* threadInfo = m_pInstance->_getActiveThread();
            assert(threadInfo);
            if (!threadInfo) {
                static Charstring error(_T("Interal Error! No active threads!")); // HCS
                return error;
            }

            StackFrame& stackFrame = threadInfo->m_callstack.top();
            return stackFrame.m_actionGuid;
        }


        void EventManager::onStep() {
            os::auto_lock lock(m_mutex);
            if(!m_pInstance) return;

            if (!m_pInstance->m_pseudoBreakpoint) {
                // N.B. This function is always called from the context of the eventManager thread.
                // Locate one of the user threads that is in the active debug context and set a pseudobreakpoint on it...

                ThreadInfo* threadInfo = m_pInstance->_getActiveThread();
                if (!threadInfo) return;

                unsigned int stackLevel = static_cast<unsigned int>(threadInfo->m_callstack.size());
                m_pInstance->m_pseudoBreakpoint = new BreakOnNextActionWithStackCondition(stackLevel,
                    threadInfo->m_id,
                    BreakOnNextActionWithStackCondition::SameOrOuter);
            }
        }

        void EventManager::onStepIn() {
            os::auto_lock lock(m_mutex);
            if(!m_pInstance) return;

            if (!m_pInstance->m_pseudoBreakpoint) {
                // N.B. This function is always called from the context of the eventManager thread.
                // Locate one of the user threads that is in the active debug context and set a pseudobreakpoint on it...

                ThreadInfo* threadInfo = m_pInstance->_getActiveThread();
                if (!threadInfo) return;

                unsigned int stackLevel = static_cast<unsigned int>(threadInfo->m_callstack.size());
                m_pInstance->m_pseudoBreakpoint = new BreakOnNextActionWithStackCondition(stackLevel,
                    threadInfo->m_id,
                    BreakOnNextActionWithStackCondition::Any);
            }
        }

        void EventManager::onStepOut() {
            os::auto_lock lock(m_mutex);
            if(!m_pInstance) return;

            if (!m_pInstance->m_pseudoBreakpoint) {
                // N.B. This function is always called from the context of the eventManager thread.
                // Locate one of the user threads that is in the active debug context and set a pseudobreakpoint on it...

                ThreadInfo* threadInfo = m_pInstance->_getActiveThread();
                if (!threadInfo) return;

                unsigned int stackLevel = static_cast<unsigned int>(threadInfo->m_callstack.size());
                m_pInstance->m_pseudoBreakpoint = new BreakOnNextActionWithStackCondition(stackLevel,
                    threadInfo->m_id,
                    BreakOnNextActionWithStackCondition::Outer);
            }
        }

        void EventManager::onRunTo(const tor::Charstring& actionGuid) {
            os::auto_lock lock(m_mutex);
            if(!m_pInstance) return;

            if (!m_pInstance->m_pseudoBreakpoint) {
                // N.B. This function is always called from the context of the eventManager thread.
                // Locate one of the user threads that is in the active debug context and set a pseudobreakpoint on it...

                ThreadInfo* threadInfo = m_pInstance->_getActiveThread();
                if (!threadInfo) return;

                m_pInstance->m_pseudoBreakpoint = new BreakOnSpecificAction(actionGuid, threadInfo->m_id);
            }
        }

        void EventManager::onBreak() {
            os::auto_lock lock(m_mutex);
            if(!m_pInstance) return;

            if (m_pInstance->m_pseudoBreakpoint)
                // There was already a pseudo breakpoint set. Delete it, since Break has higher priority
                // (It shall always be possible to Break the program while running.).
                delete m_pInstance->m_pseudoBreakpoint;

            m_pInstance->m_pseudoBreakpoint = new BreakOnNextAction;
        }


        void EventManager::removeAllPseudoBreakpoints()
        {
            os::auto_lock lock(m_mutex);
            if(!m_pInstance) return;

            if (m_pInstance->m_pseudoBreakpoint)
            {
                delete m_pInstance->m_pseudoBreakpoint;
                m_pInstance->m_pseudoBreakpoint = NULL;
            }
        }

        bool EventManager::shouldWaitForAttach(tor::Charstring& host, tor::Natural& port)
        {
            host = _T("localhost");
            port = 56500;
            bool bShouldWaitForAttach;
            tor::Charstring hostDbgInfo;

#if defined(WIN32) && !defined(MAINWIN)
            size_t sz=0;
            getenv_s(&sz,NULL,0,"TTD_TOR_WAIT_FOR_ATTACH");

            bShouldWaitForAttach = (sz != 0);

            getenv_s(&sz,NULL,0,"TTD_TOR_HOST_DBG");
            char *t=NULL;
            if(sz>0)
            {
                t=new char[sz];
                getenv_s(&sz,t,sz,"TTD_TOR_HOST_DBG");
                hostDbgInfo=tor::Charstring(t);
                delete []t;
            }
#else
            bShouldWaitForAttach = (getenv("TTD_TOR_WAIT_FOR_ATTACH") != 0);

            char *host_dbg=getenv("TTD_TOR_HOST_DBG");
            hostDbgInfo=tor::Charstring(host_dbg?host_dbg:"");
#endif

            if (!hostDbgInfo.stringRep.empty()) {
                Charstring::StringRepType::size_type s = hostDbgInfo.stringRep.find_last_of(_T("#"));
                if ((s != Charstring::StringRepType::npos) && (s != 0) && (s != hostDbgInfo.stringRep.size() - 1)) {
                    host = hostDbgInfo.stringRep.substr(0, s);
                    Charstring strPort(hostDbgInfo.stringRep.substr(s+1));
                    port = atoi(strPort.getNarrowString().c_str());
                }
            }

            return bShouldWaitForAttach;
        }

        AppInfo EventManager::getAppInfo()
        {
            // Currently the application information only includes the GUID of the build artifacts that was used when building
            // the application.
            AppInfo appInfo(theApplication->guid);
            return appInfo;
        }


    } // namespace meta

} // namespace tor

// Include generated implementation
#include "torEventManagerImpl.cpp"

//////////////////////////////////////////////////////////////////////////



void tor::trace(const tor::Charstring& message) {
    tor::meta::EventManager::processEvent(new tor::meta::Message(message));
}

void tor::assertFailure(const TCHAR* condition, const TCHAR* file, int line) {
    tor::Charstring str(_T("Assertion failed!\nCondition: "));
    str += condition;
    str += _T("\nSource file: ");
    str += file;
    str += _T("\nLine: ");
    TCHAR buf[16];
    tor::stprintf_number(buf, line);
    str += buf;
    str += _T("\n");

    trace(str);
}



#endif
