// Generated for IBM Rational Tau 4.3.0.0.15182 at Wed Sep 30 04:38:20 MEST 2009
// Licensed Materials - Property of IBM 
// (c) Copyright IBM Corporation 2002,2009.  All Rights Reserved.
// 
// U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#include "torPlatform.h"

#include "torInstanceManager.h"
#include "torDispatchableClass.h"
#include <assert.h>
#include <algorithm>

namespace 
{
    tor::InstanceManager *instanceManager = 0;
}

namespace tor
{
    using std::map;

    //
    tor::InstanceManager::ObjectId tor::InstanceManager::m_objectIdSeed = 1;

    //

    InstanceManager::InstanceManager()
      : m_mutex(),
        m_executors(),
        m_receivers()
    {
        m_mutex.init();
    }

    InstanceManager::~InstanceManager() {}

    // Executor
    
    bool InstanceManager::isValidExecutor(const ObjectId &id) const
    {
        os::auto_lock lock(m_mutex);
        return m_executors.find(id) != m_executors.end();
    }

    InstanceManager::ObjectId InstanceManager::getIdFromExecutor(EventExecutor *instance) const
    {
        os::auto_lock lock(m_mutex);

        ExecutorMap::const_iterator i = find_if(m_executors.begin(), m_executors.end(), bind2nd(EqualValue<ExecutorMap>(), instance));
        if (i == m_executors.end())
            return 0;
        return i->first;
    }

    EventExecutor *InstanceManager::getExecutorFromId(const ObjectId &id) const
    {
        os::auto_lock lock(m_mutex);

        ExecutorMap::const_iterator i = m_executors.find(id);
        if (i == m_executors.end())
            return 0;
        return i->second;
    }

    InstanceManager::ObjectId InstanceManager::addExecutor(EventExecutor *instance)
    {
        os::auto_lock lock(m_mutex);
        
        ObjectId id = newId();
        m_executors.insert(ExecutorMap::value_type(id, instance));
    
        return id;
    }

    void InstanceManager::removeExecutor(EventExecutor *instance)
    {
        os::auto_lock lock(m_mutex);

        ExecutorMap::const_iterator i = find_if(m_executors.begin(), m_executors.end(), bind2nd(EqualValue<ExecutorMap>(), instance));
        if (i == m_executors.end())
            return; // No instance present, consider us done
        m_executors.erase(i->first);
    }

    void InstanceManager::removeExecutor(const ObjectId &id)
    {
        os::auto_lock lock(m_mutex);
        ExecutorMap::const_iterator i = m_executors.find(id);
        if (i == m_executors.end())
            return;
        m_executors.erase(id);
    }

    // Receiver

    bool InstanceManager::isValidReceiver(const ObjectId &id) const
    {
        os::auto_lock lock(m_mutex);
        return m_receivers.find(id) != m_receivers.end();
    }

    InstanceManager::ObjectId InstanceManager::getIdFromReceiver(EventReceiver *instance) const
    {
        os::auto_lock lock(m_mutex);

        ReceiverMap::const_iterator i = find_if(m_receivers.begin(), m_receivers.end(), bind2nd(EqualValue<ReceiverMap>(), instance));
        if (i == m_receivers.end())
            return 0;
        return i->first;
    }

    EventReceiver *InstanceManager::getReceiverFromId(const ObjectId &id) const
    {
        os::auto_lock lock(m_mutex);

        ReceiverMap::const_iterator i = m_receivers.find(id);
        if (i == m_receivers.end())
            return 0;
        return i->second;
    }

    InstanceManager::ObjectId InstanceManager::addReceiver(EventReceiver *instance)
    {
        os::auto_lock lock(m_mutex);

        ObjectId id = newId();
        m_receivers.insert(ReceiverMap::value_type(id, instance));

        return id;
    }

    void InstanceManager::removeReceiver(EventReceiver *instance)
    {
        os::auto_lock lock(m_mutex);

        ReceiverMap::const_iterator i = find_if(m_receivers.begin(), m_receivers.end(), bind2nd(EqualValue<ReceiverMap>(), instance));
        if (i == m_receivers.end())
            return; // No instance present, consider us done
        m_receivers.erase(i->first);
    }

    void InstanceManager::removeReceiver(const ObjectId &id)
    {
        os::auto_lock lock(m_mutex);
        ReceiverMap::const_iterator i = m_receivers.find(id);
        if (i == m_receivers.end())
            return;
        m_receivers.erase(id);
    }

    // Utility

    InstanceManager::ObjectId InstanceManager::newId()
    {
        return InstanceManager::m_objectIdSeed++;
    }

    InstanceManager &InstanceManager::instance()
    {
        if (instanceManager == 0)
            instanceManager = new InstanceManager;
        return *instanceManager;
    }
}

