/*************************************************************
Generated for IBM Rational Tau 4.3.0.0.15182 at Wed Sep 30 04:38:22 MEST 2009
Licensed Materials - Property of IBM 
(c) Copyright IBM Corporation 2002,2009.  All Rights Reserved.

U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*************************************************************/
#ifndef __torInstanceManager_h
#define __torInstanceManager_h

#include "torPlatform.h"

#include "torOs.h"
#include <map>
#include <functional>

namespace tor
{
    class EventExecutor;
    class EventReceiver;

    // Singleton, i.e there must only exist one instance of the InstanceManager in an entire application.
    // Make sure to use TOR as a dynamic (shared) library in case your application consists of
    // multiple modules that depend on TOR.
    class TOR_EXT_DEF InstanceManager
    {
    public:
        ~InstanceManager();

        typedef unsigned long ObjectId; // Potential receiver/executor

        // Executor

        bool isValidExecutor(const ObjectId &id) const;

        ObjectId getIdFromExecutor(EventExecutor *executor) const;
        EventExecutor *getExecutorFromId(const ObjectId &id) const;

        ObjectId addExecutor(EventExecutor *executor);

        void removeExecutor(EventExecutor *executor);
        void removeExecutor(const ObjectId &id);

        // Receiver

        bool isValidReceiver(const ObjectId &id) const;

        ObjectId getIdFromReceiver(EventReceiver *receiver) const;
        EventReceiver *getReceiverFromId(const ObjectId &id) const;

        ObjectId addReceiver(EventReceiver *receiver);

        void removeReceiver(EventReceiver *receiver);
        void removeReceiver(const ObjectId &id);

        // Returns the one and only instance manager
        static InstanceManager &instance();

    protected:
        typedef std::map<ObjectId, EventExecutor *> ExecutorMap;
        typedef std::map<ObjectId, EventReceiver *> ReceiverMap;

        ObjectId newId();

        template <class Map>
        struct EqualValue : std::binary_function<typename Map::value_type, typename Map::value_type::second_type, bool>
        {
            inline bool operator()(const typename Map::value_type &a1, const typename Map::value_type::second_type &a2) const
            {
                return a1.second == a2;
            }
        };

    private:
        InstanceManager();
        InstanceManager(const InstanceManager &); // No implementation

        InstanceManager &operator=(const InstanceManager &); // No implementation

        static ObjectId m_objectIdSeed;
        mutable os::Mutex m_mutex;

        ExecutorMap m_executors;
        ReceiverMap m_receivers;

	friend class HasNoFriends; // Removes compiler warnings
    };
}

#endif

