/*************************************************************
Generated for IBM Rational Tau 4.3.0.0.15182 at Wed Sep 30 04:38:22 MEST 2009
Licensed Materials - Property of IBM 
(c) Copyright IBM Corporation 2002,2009.  All Rights Reserved.

U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*************************************************************/
///////////////////////////////////////////////////////////////////////////////
// torMetaData.h
// -------------
// Definition of a metamodel for representing data of a TOR-based application.
///////////////////////////////////////////////////////////////////////////////

#ifndef __TOR_METADATA__H__
#define __TOR_METADATA__H__

#ifdef TOR_USE_INSTRUMENTATION

#include "torPlatform.h"

#include "torTypes.h"

#include <list>

namespace tor {

    namespace meta {

	class Value;
	class StructuredValue;
	class StructuralFeature;
	class Operation;

	////////////////
	// Type model //
	////////////////
    
	class TOR_EXT_DEF Definition {
	public:
	    const tor::Charstring name;
	    const tor::Charstring guid;
            Definition* owner; // Link to owning definition

	    Definition(const tor::Charstring& strName, const tor::Charstring& strGuid, Definition* theOwner) : name(strName), guid(strGuid), owner(theOwner) {}

	    virtual ~Definition() {}
	};

	class TOR_EXT_DEF Type : public Definition {
	public:
	    Type(const tor::Charstring& strName, const tor::Charstring& strGuid, Definition* theOwner) : Definition(strName, strGuid, theOwner) {}

            virtual ~Type() {}

	    // Constructs a value representing the value of the data pointer (which is assumed to
	    // be an instance of this Type).
	    virtual Value* getValue(const void* data) {
		// Default implementation does nothing!
		return 0;
	    }

	    // Sets the value of the data pointer (which is assumed to be an instance of this Type),
	    // according to the specified value structure.
	    virtual void setValue(void* data, Value* value) {
		// Default implementation does nothing!
	    }
	};

	class TOR_EXT_DEF Typed {
	public:
	    Type* type;

	    Type* getType() const {
		return type;
	    }
	};

	class TOR_EXT_DEF StructuredType : public Type {
	public:
	    StructuredType(const tor::Charstring& strName, const tor::Charstring& strGuid, Definition* theOwner) : Type(strName, strGuid, theOwner) {}

	    typedef std::list<StructuredType*> BaseList;
	    BaseList bases;

	    // Let this StructuredType inherit from a base StructuredType
	    void inherit(StructuredType* base) {
		bases.push_back(base);
	    }

	    // As getValue but puts the assignments for member values into 'value' instead of
	    // creating a new StructuredValue.
	    void getMemberValues(const void* data, StructuredValue* value);

	protected:
	    StructuredValue* currentStructuredValue; // Used to hold the current structured value when building a value
						     // representation for inherited members.
	};

	class TOR_EXT_DEF StructuralFeature : public Definition, public Typed {
	public:
	    StructuralFeature(const tor::Charstring& strName, const tor::Charstring& strGuid, Definition* theOwner) : Definition(strName, strGuid, theOwner) {}
	};

	class TOR_EXT_DEF Operation : public Type {
	public:            
            enum Kind 
            {
                Constructor,
                Destructor,
                Ordinary
            } kind;

	    Operation(const tor::Charstring& strName, const tor::Charstring& strGuid, Definition* theOwner,
                      const Kind operationKind = Ordinary) : Type(strName, strGuid, theOwner), kind(operationKind) {}

            virtual ~Operation() {}

	    // Perform a call to the Operation, on the object. inArgs should be the input arguments, and
	    // outArgs will after the call contain the output arguments.
	    // A value representing the return value of the call will be returned.
	    virtual Value* call(void* object, StructuredValue* inArgs, StructuredValue* outArgs) 
            {
		// Default version does nothing!
		return 0;
	    }
	};

	class TOR_EXT_DEF PrimitiveType : public Type {
	public:
	    PrimitiveType(const tor::Charstring& strName, const tor::Charstring& strGuid, Definition* theOwner) : Type(strName, strGuid, theOwner) { }

	    virtual Value* getValue(const void* data);
	    void setValue(void* data, Value* value);
	};

	class TOR_EXT_DEF Enumeration : public Type {
	public:
	    Enumeration(const tor::Charstring& strName, const tor::Charstring& strGuid, Definition* theOwner) : Type(strName, strGuid, theOwner) { }
	};

	class TOR_EXT_DEF Literal : public Definition {
	public:
	    Literal(const tor::Charstring& strName, const tor::Charstring& strGuid, Definition* theOwner) : Definition(strName, strGuid, theOwner) { }
	};

	/////////////////
	// Value model //
	/////////////////

	class TOR_EXT_DEF Value : public Typed {
	public:
	    Value(Type* valueType) {
		type = valueType;
	    }
	    
	    virtual ~Value() { }
	};

	class TOR_EXT_DEF Assignment {
	public:
	    StructuralFeature* LHS;
	    Value* RHS;

	    Assignment(StructuralFeature& lhs, Value* rhs) : LHS(&lhs), RHS(rhs) {}
	};

	class TOR_EXT_DEF StructuredValue : public Value {
	public:
	    StructuredValue(Type* type) : Value(type) {}
	    
	    typedef std::list<Assignment*> List;
	    List assignments;

	    void add(Assignment* a) {
		assignments.push_back(a);
	    }
	};

	class TOR_EXT_DEF SimpleValue : public Value {
	public:
	    SimpleValue(Type* type);

	    union {
		tor::Integer m_integer;
		tor::Boolean m_boolean;
		void* m_pointer;
		Definition* m_definition;
	    };

	    enum { k_Integer, k_Boolean, k_Pointer, k_Identifier } kind;

	    void setInteger(tor::Integer i);
	    void setBoolean(tor::Boolean b);
	    void setPointer(void* p);
	    void setIdentifier(Definition* d);
	    	    
	    bool isInteger() const;
	    bool isBoolean() const;
	    bool isPointer() const;
	    bool isIdentifier() const;
	};

        ///////////////////////
	// Application model //
	///////////////////////

        class TOR_EXT_DEF Application : public Definition {
        public:
            
            // Constructor to use for a non-instrumented application
            Application() : Definition(_T(""), _T(""), 0) {}

            // Constructor to use for an instrumented application
            Application(const tor::Charstring& strName, const tor::Charstring& strGuid);

            // Determines if the application is instrumented
            bool isInstrumented();
        };

        // A reference to the one-and-only application object
        TOR_EXT_DEF extern Application* theApplication;

	TOR_EXT_DEF extern PrimitiveType _Integer;
	TOR_EXT_DEF extern PrimitiveType _Boolean;
	TOR_EXT_DEF extern PrimitiveType _Pointer;

    } // namespace meta

    // An tor::Entity represents an entity of a running TOR-based application.
    // It associates the pointer to the entity with the corresponding meta definition.
    typedef std::pair<void*, meta::Definition*> Entity;

} // namespace tor

namespace std{
    template struct TOR_EXT_DEF std::pair<void*, tor::meta::Definition*>;
} // namespace std


#endif

#endif // __TOR_METADATA__H__

