/*************************************************************
Generated for IBM Rational Tau 4.3.0.0.15182 at Wed Sep 30 04:38:22 MEST 2009
Licensed Materials - Property of IBM 
(c) Copyright IBM Corporation 2002,2009.  All Rights Reserved.

U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*************************************************************/
///////////////////////////////////////////////////////////////////////////////
// torMetaEvents.h
// ---------------
// Definition of meta events, i.e. descriptions of events occurring in
// a TOR-based application.
///////////////////////////////////////////////////////////////////////////////

#ifndef __TOR_METAEVENTS__H__
#define __TOR_METAEVENTS__H__

#ifdef TOR_USE_INSTRUMENTATION

#include "torPlatform.h"

#include "torEvent.h"
#include "torTypes.h"
#include "torUtilities.h"
#include "torCoders.h"
#include "torMetaData.h"
#include "torContainers.h"

namespace tor {

    namespace meta {
	    // Common base for all meta events
        class TOR_EXT_DEF Event : virtual public tor::InternalEvent {
        public:
            Event();
            static bool isTypeOf(tor::Event* e) {
                return tor::cast<Event*>(e) != 0;
            }

            // Encodes the meta event into a string
            virtual void encode(Coder* coder) = 0;

            // Decodes the string into a meta event
            static Event* decode(const Charstring& str);

            Charstring encodedInfo; // String encoded meta event data

            bool m_bSynchronouslySent; // Specifies if the meta event is sent synchronously
        };

        // Sent when a new instance of a signature is created.
        // The event is sent from the context of the creator.
        class TOR_EXT_DEF Create : virtual public Event {	
        public:
            // These events are always followed by another event (Called or Returned) so
            // send these asynchronously.
            Create(const Entity& c, Definition* iT, int id) 
                : creator(c), instantiatedType(iT), threadId(id) { m_bSynchronouslySent = false; }

                static bool isTypeOf(tor::Event* e) {
                    return tor::cast<Create*>(e) != 0;
                }

                // Add a constructor actual argument to the event data
                void addArgument(Value* arg);

                virtual void encode(Coder* coder);

        protected:
            Entity creator; // The object performing the create, or NULL if the create takes place in a non-object context
            Definition* instantiatedType; // Instantiated type
            int threadId; // Id of the thread performing the create
            typedef std::list<Charstring> ConstructorArgs;
            ConstructorArgs actualConstructorArgs; // List of actual constructor arguments
        };

        // Sent when an instance of a signature is deleted.
        // The event is sent from the context of the deleter.
        class TOR_EXT_DEF Delete : virtual public Event {	
        public:
            // These events are always followed by another event (Called or Returned) so
            // send these asynchronously.
            Delete(const Entity& d, Definition* dT, int id) : deleter(d), deletedType(dT), threadId(id) { 
                m_bSynchronouslySent = false; 
            }

            static bool isTypeOf(tor::Event* e) {
                return tor::cast<Delete*>(e) != 0;
            }

            virtual void encode(Coder* coder);

        protected:
            Entity deleter; // The object performing the delete, or NULL if the delete takes place in a non-object context
            Definition* deletedType; // Type of the deleted instance
            int threadId; // Id of the thread performing the delete
        };

        // Sent when an operation is called.
        // The event is sent from the context of the caller.
        class TOR_EXT_DEF Call : virtual public Event {
        public:
            // These events are always followed by another event (Called or Returned) so
            // send these asynchronously.
            Call(const Entity& caller, Definition* calledOperation, int id) 
                : caller(caller), calledOperation(calledOperation), threadId(id) 
            { m_bSynchronouslySent = false; }

                // Set trace information about an actual argument for the traced call
                void addArgument(Value* arg);

                static bool isTypeOf(tor::Event* e) {
                    return tor::cast<Call*>(e) != 0;
                }


                virtual void encode(Coder* coder);

        protected:
            Entity caller; // The caller object 	
            Definition* calledOperation; // The called operation
            int threadId; // Id of the thread performing the call
            typedef std::list<Charstring> ActualArgs;
            ActualArgs actualCallArgs; // List of actual call arguments
        };

        // Sent when an operation is called.
        // The event is sent from the context of the called operation.
        class TOR_EXT_DEF Called : virtual public Event {	
        public:
            Called(const Entity& called, Definition* calledOperation, int id) 
                : called(called), calledOperation(calledOperation), threadId(id) {}

                static bool isTypeOf(tor::Event* e) {
                    return tor::cast<Called*>(e) != 0;
                }

                // Add a constructor actual argument to the event data
                void addArgument(Value* arg);

                virtual void encode(Coder* coder);

        protected:
            Entity called; // The called object 
            Definition* calledOperation; // The called operation
            int threadId; // Id of the thread performing the call
            typedef std::list<Charstring> ConstructorArgs;
            ConstructorArgs actualConstructorArgs; // Description of actual constructor arguments
        };

        // Sent when a called operation is returned from.
        // The event is sent from the context of the called operation.
        class TOR_EXT_DEF Return : virtual public Event {
        public:
            // These events are always followed by another event (Called or Returned) so
            // send these asynchronously.
            Return(const Entity& called, Definition* calledOperation, int id) 
                : called(called), calledOperation(calledOperation), threadId(id) 
                { m_bSynchronouslySent = false; }

                static bool isTypeOf(tor::Event* e) {
                    return tor::cast<Called*>(e) != 0;
                }

                virtual void encode(Coder* coder);

                // Set the return value
                void setReturnValue(Value* val);

        protected:
            Entity called; // The called object 
            Definition* calledOperation; // The called operation from which the return is performed
            int threadId; // Id of the thread performing the return
            Charstring returnValue;
        };

        // Sent when a called operation is returned from.
        // The event is sent from the context of the operation to which the return is made.
        class TOR_EXT_DEF Returned : virtual public Event {
        public:
            Returned(const Entity& returnedTo, Definition* returnedToOperation, int id) 
                : returnedTo(returnedTo), returnedToOperation(returnedToOperation), threadId(id) {}

                static bool isTypeOf(tor::Event* e) {
                    return tor::cast<Called*>(e) != 0;
                }

                virtual void encode(Coder* coder);

        protected:
            Entity returnedTo; // The object to which the return is made 
            Definition* returnedToOperation; // The operation to which the return is made
            int threadId; // Id of the thread performing the return
        };

        // Sent when an event is sent.
        class TOR_EXT_DEF EventSent : virtual public Event {
        public:
            EventSent(tor::Event* event, const Entity& sender, StructuredValue* eventInstance) : 
              event(event), sender(sender), eventInstance(eventInstance) { }
            ~EventSent();

            static bool isTypeOf(tor::Event* e) {
                return tor::cast<EventSent*>(e) != 0;
            }

            virtual void encode(Coder* coder);

        protected:
            tor::Event* event; // The event object that is sent
            Entity sender; // The sender object 	
            StructuredValue* eventInstance; // Value representation of the event instance being sent
        };

        // Sent when an event is received.
        class TOR_EXT_DEF EventReceived : virtual public Event {
        public:
            EventReceived(tor::Event* event, const Entity& receiver, StructuredValue* eventInstance) : 
              event(event), receiver(receiver), eventInstance(eventInstance) { }
            ~EventReceived();

            static bool isTypeOf(tor::Event* e) {
                return tor::cast<EventReceived*>(e) != 0;
            }

            virtual void encode(Coder* coder);

        protected:
            tor::Event* event; // The event object that is received
            Entity receiver; // The sender object 	
            StructuredValue* eventInstance; // Value representation of the event instance being sent
        };

        // Sent when a text message is emitted from the application.
        class TOR_EXT_DEF Message : virtual public Event {
        public:
            Message(const Charstring& message) : message(message) {}

            static bool isTypeOf(tor::Event* e) {
                return tor::cast<Message*>(e) != 0;
            }

            virtual void encode(Coder* coder);

            const Charstring message;
        };

        // Sent when the application is about to exit.
        // This event is used internally for synchronizing with the EventManager
        // so that all pending events are processed before terminating the application.
        class Exit : virtual public tor::Event {
        public:
            static bool isTypeOf(tor::Event* e) {
                return tor::cast<Exit*>(e) != 0;
            }
        };

        //////////////////////////////////////////////////////////////////////////
        // Debugger events
        //////////////////////////////////////////////////////////////////////////

        // Representation of a breakpoint
        struct TOR_EXT_DEF BreakpointInfo {
            BreakpointInfo() : actionGuid(_T("")) {}
            BreakpointInfo(const Charstring& actionGuid) : actionGuid(actionGuid) {}

            Charstring actionGuid; // Guid of action on which the breakpoint is set

            Charstring encode(Coder* coder) const;
        };

        // Removes breakpoints.
        class TOR_EXT_DEF RemoveBreakpoints_BreakpointInfo : virtual public Event {
        public:
            RemoveBreakpoints_BreakpointInfo(const String<BreakpointInfo>& breakpoints) : breakpoints(breakpoints) {}

            static bool isTypeOf(tor::Event* e) {
                return tor::cast<RemoveBreakpoints_BreakpointInfo*>(e) != 0;
            }

            virtual void encode(Coder* coder);

            const String<BreakpointInfo> breakpoints;
        };

        // Sets breakpoints.
        class TOR_EXT_DEF SetBreakpoints_BreakpointInfo : virtual public Event {
        public:
            SetBreakpoints_BreakpointInfo(const String<BreakpointInfo>& breakpoints) : breakpoints(breakpoints) {}

            static bool isTypeOf(tor::Event* e) {
                return tor::cast<SetBreakpoints_BreakpointInfo*>(e) != 0;
            }

            virtual void encode(Coder* coder);

            const String<BreakpointInfo> breakpoints;
        };

        // Breaks the running application into the debugger.
        class TOR_EXT_DEF Break : virtual public Event {
        public:
            static bool isTypeOf(tor::Event* e) {
                return tor::cast<Break*>(e) != 0;
            }

            virtual void encode(Coder* coder);
        };

        // Sent by the application when a breakpoint has been hit
        // (i.e. it is just about to execute an action with an attached breakpoint).
        class TOR_EXT_DEF BreakpointHit_Charstring : virtual public Event {
        public:
            BreakpointHit_Charstring(const Charstring& actionGuid) : actionGuid(actionGuid) { 
                m_bSynchronouslySent = true; 
            }

            static bool isTypeOf(tor::Event* e) {
                return tor::cast<BreakpointHit_Charstring*>(e) != 0;
            }

            virtual void encode(Coder* coder);

            const Charstring actionGuid;
        };

        // Attach the debugger to a running application in order to start a debug session.
        class TOR_EXT_DEF Attach : virtual public Event {
        public:
            static bool isTypeOf(tor::Event* e) {
                return tor::cast<Attach*>(e) != 0;
            }

            virtual void encode(Coder* coder);
        };

        // Information about the application attached to
        struct TOR_EXT_DEF AppInfo {
            AppInfo(const Charstring& BAguid = _T("")) : buildArtifactGuid(BAguid) {}

            Charstring buildArtifactGuid;

            Charstring encode(Coder* coder) const;
        };

        class TOR_EXT_DEF Attached_AppInfo : virtual public Event {
        public:
            Attached_AppInfo(AppInfo appInfo) : appInfo(appInfo) {}

            static bool isTypeOf(tor::Event* e) {
                return tor::cast<Attached_AppInfo*>(e) != 0;
            }

            virtual void encode(Coder* coder);

            AppInfo appInfo;
        };

        // Runs the application until a specified action is reached, then stops at that action.
        class TOR_EXT_DEF RunTo_Charstring : virtual public Event {
        public:
            RunTo_Charstring(const Charstring& actionGuid) : actionGuid(actionGuid) {}

            static bool isTypeOf(tor::Event* e) {
                return tor::cast<RunTo_Charstring*>(e) != 0;
            }

            virtual void encode(Coder* coder);

            const Charstring actionGuid;
        };

        // Steps one action, then stops at that action.
        class TOR_EXT_DEF Step : virtual public Event {
        public:
            static bool isTypeOf(tor::Event* e) {
                return tor::cast<Step*>(e) != 0;
            }

            virtual void encode(Coder* coder);
        };

        // Steps into the next inner stack frame. If no such stack frame exists, StepIn
        // is equivalent with Step.
        class TOR_EXT_DEF StepIn : virtual public Event {
        public:
            static bool isTypeOf(tor::Event* e) {
                return tor::cast<StepIn*>(e) != 0;
            }

            virtual void encode(Coder* coder);
        };

        // Steps out from the current stack frame.
        class TOR_EXT_DEF StepOut : virtual public Event {
        public:
            static bool isTypeOf(tor::Event* e) {
                return tor::cast<StepOut*>(e) != 0;
            }

            virtual void encode(Coder* coder);
        };

        // Runs the application after having been stopped in the debugger.
        class TOR_EXT_DEF Go : virtual public Event {
        public:
            static bool isTypeOf(tor::Event* e) {
                return tor::cast<Go*>(e) != 0;
            }

            virtual void encode(Coder* coder);
        };

        // Sent to the host debugger when execution is halted (for example because a breakpoint has been hit).
        class TOR_EXT_DEF ExecutionHalted_Charstring : virtual public Event {
        public:
            ExecutionHalted_Charstring(const Charstring& actionGuid) : actionGuid(actionGuid) {}

            static bool isTypeOf(tor::Event* e) {
                return tor::cast<ExecutionHalted_Charstring*>(e) != 0;
            }

            virtual void encode(Coder* coder);

            const Charstring actionGuid;
        };

    } // namespace meta

} // namespace tor

#endif

#endif // __TOR_METAEVENTS__H__


