// Generated for IBM Rational Tau 4.3.0.0.15182 at Wed Sep 30 04:38:20 MEST 2009
// Licensed Materials - Property of IBM 
// (c) Copyright IBM Corporation 2002,2009.  All Rights Reserved.
// 
// U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#if defined(TOR_USE_INSTRUMENTATION) && !defined(TOR_NO_EVENTMANAGER)

#include "torPlatform.h"

#include "torMetaStackContext.h"
#include "torEventManager.h"
#include "torTypes.h"
#include <assert.h>

using namespace tor::meta;

namespace {
    inline Definition* getOwner(Definition* d) {
        return d ? d->owner : 0;
    }
}

StackContext::StackContext(Operation* metaDef, const tor::Charstring& compoundActionGuid, void* objectContext /* = 0 */) 
: m_bTraceCall(true), m_bSynthesized(compoundActionGuid.length() == 0)
{
    if (EventManager::isFinalized())
        // The eventManager is finalized (probably explicitly finalized). Do nothing.
        return;

    if (!metaDef) 
    {
        assert(0); // Must know which meta definition this stack context belongs to...
        return;
    }

    m_id = os::Thread::id();
    ThreadInfo* threadInfo = EventManager::getThreadInfo(m_id);

    void* callerObjectContext = 0;
    Definition* callerDef = 0;

    // Trace this call unless the called operation has an empty name.
    // Then it is typically a transition, and we do not yet support tracing of these.
    // Also don't trace if the called operation is synthesized during code generation and is not a constructor or destructor.
    if ((metaDef->name.length() == 0) || (m_bSynthesized && (metaDef->kind == Operation::Ordinary)))
        m_bTraceCall = false;

    if (!threadInfo->m_callstack.empty()) 
    {
        // Extract info about caller
        StackFrame& stackFrame = threadInfo->m_callstack.top();
        Entity& caller = stackFrame.m_calledOperation;
        callerObjectContext = caller.first;
        callerDef = caller.second;
    }

    if (metaDef->kind == Operation::Constructor) 
    {
        // TODO: Detect the case when a constructor for a base class is called, as a result
        // of instantiating a derived class. Then the Create event should not be sent!

        Create* createEvent = new Create(std::make_pair(callerObjectContext, getOwner(callerDef)), 
            getOwner(metaDef), m_id);
        EventManager::processEvent(createEvent);
    }
    else if (metaDef->kind == Operation::Destructor) 
    {
        // TODO: Detect the case when a destructor for a base class is called, as a result
        // of deleting an instance of a derived class. Then the Delete event should not be sent!

        Delete* deleteEvent = new Delete(std::make_pair(callerObjectContext, getOwner(callerDef)), 
            getOwner(metaDef), m_id);
        EventManager::processEvent(deleteEvent);
    }
    else 
    {
        // Call of ordinary operation

        if (m_bTraceCall) 
        {
            Call* callEvent = new Call(std::make_pair(callerObjectContext, getOwner(callerDef)), metaDef, m_id);
            EventManager::processEvent(callEvent);
        }
    }

    // When we get here the operation is considered to have been called. So push it on
    // the callstack and send the Called event.
    // Note: Do not send the Called event for "synthesized" stack frames, i.e. calls to operations
    //       that have been synthesized during code generation (except for
    //       synthesized constructors / destructors since that event is required for a correct tracing
    //       of an instance create / delete).

    Entity calledEntity = std::make_pair(objectContext, dynamic_cast<tor::meta::Definition*>(metaDef));
    threadInfo->m_callstack.push(StackFrame(calledEntity, compoundActionGuid));

    // Calling a non-synthesized operation implies per definition an execution of its compound action. 
    if (!m_bSynthesized)
        EventManager::beforeAction(compoundActionGuid);

    if (m_bTraceCall) 
    {
        Called* calledEvent = new Called(std::make_pair(objectContext, getOwner(metaDef)), metaDef, m_id);
        EventManager::processEvent(calledEvent);
    }
}

StackContext::~StackContext() 
{
    if (EventManager::isFinalized())
        // The eventManager is finalized (probably explicitly finalized). Do nothing.
        return;

    ThreadInfo* threadInfo = EventManager::getThreadInfo(m_id);

    if (threadInfo->m_callstack.empty()) {
        // This should normally not happen! How could we close a stack context without having a callstack!?!
        // However, one explanation could be that TOR was just initialized so that no stack frames
        // exist yet. 
        // Another explanation could be that the generated instrumentation was modified by the user.
        return; 
    }

    StackFrame& stackFrame = threadInfo->m_callstack.top();
    Entity& currentContext = stackFrame.m_calledOperation;

    if (m_bTraceCall) 
    {
        Return* returnEvent = new Return(std::make_pair(currentContext.first, getOwner(currentContext.second)), currentContext.second, m_id);
        EventManager::processEvent(returnEvent);
    }

    // When we get here the operation is considered to have returned. So pop it from
    // the callstack and send the Returned event.
    // Note: Do not send the Returned event for "synthesized" stack frames, i.e. calls to operations
    //       that have been synthesized during code generation (except for
    //       synthesized constructors / destructors since that event is required for a correct tracing
    //       of an instance create / delete).

    threadInfo->m_callstack.pop();

    void* callerObjectContext = 0;
    Definition* callerDef = 0;

    if (!threadInfo->m_callstack.empty()) 
    {
        // Extract info about caller
        StackFrame& callingStackFrame = threadInfo->m_callstack.top();
        Entity& caller = callingStackFrame.m_calledOperation;

        callerObjectContext = caller.first;
        callerDef = caller.second;

        // When returning from an operation call we should do a "beforeAction" on the action that
        // triggered the call. The reason is that only part of that action may have been executed.
        // For example: f(g());
        // When the call of g() returns we still haven't performed the call of f().        
        // By doing a "beforeAction" on the calling action upon return, we enable execution to
        // be halted before the call of f(). 
        if (!m_bSynthesized)
            EventManager::beforeAction(callingStackFrame.m_actionGuid, true /* partiallyAlreadyExecuted */);

        if (m_bTraceCall) 
        {
            Returned* returnedEvent = new Returned(std::make_pair(callerObjectContext, getOwner(callerDef)), callerDef, m_id);
            EventManager::processEvent(returnedEvent);
        }
    }
}

#endif

