// Generated for IBM Rational Tau 4.3.0.0.15182 at Wed Sep 30 04:38:20 MEST 2009
// Licensed Materials - Property of IBM 
// (c) Copyright IBM Corporation 2002,2009.  All Rights Reserved.
// 
// U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#include "torOs_solaris.h"
#include "torTime.h"

#include <assert.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <signal.h>
#include <sys/time.h>
#include <sys/wait.h>

#ifndef _MSC_VER // SOCKET
# include <netdb.h>
# include <sys/types.h>
# include <sys/socket.h>
# include <netinet/in.h>
# include <arpa/inet.h>
#endif

#include <memory>
#include <functional>
#include <utility>
#include <iomanip>

namespace 
{
    // tcp/ip helper
    bool build_client_address(const char *host, unsigned int port, sockaddr_in &socket_address)
    {
        hostent *host_entry = gethostbyname(host);
        if (host_entry == 0)
            return false;

        const char *local_ip = inet_ntoa(*(in_addr *) *host_entry->h_addr_list);
        if (local_ip == 0)
            return false;

        unsigned long local_ip_number = inet_addr(local_ip);
        if (local_ip_number <= 0)
            return false;

        socket_address.sin_family = AF_INET;
        socket_address.sin_addr.s_addr = local_ip_number;
        socket_address.sin_port = htons(port);

        return true;
    }

    bool build_server_address(unsigned int port, sockaddr_in &socket_address)
    {
        socket_address.sin_family = AF_INET;
        socket_address.sin_addr.s_addr = htonl(INADDR_ANY);
        socket_address.sin_port = htons(port);
        return true;
    }
}

namespace tor 
{
    namespace os 
    {
        const PID invalid_pid = -1;
        const FD invalid_fd = -1;
        const SOCKET invalid_socket = -1;

        namespace tcpip
        {
            void init()
            {}

            bool close(SOCKET s)
            {
                ::close(s);
                return true;
            }

            SOCKET create_client_socket(const char *host, unsigned int port)
            {
                sockaddr_in server;
                if (!build_client_address(host, port, server))
                    return invalid_socket;

                SOCKET s = ::socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
                if (s == invalid_socket)
                    return invalid_socket;

                if (::connect(s, reinterpret_cast<sockaddr *>(&server), sizeof(server)) < 0)
                {
                    close(s);
                    return invalid_socket;
                }

                return s;
            }

            SOCKET create_server_socket(unsigned int port, bool reuse)
            {
                sockaddr_in server;
                if (!build_server_address(port, server))
                    return invalid_socket;

                SOCKET s = ::socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
                if (s == invalid_socket)
                    return invalid_socket;

                if (reuse)
                {
                    int value = 1;
                    if (::setsockopt(s, SOL_SOCKET, SO_REUSEADDR, reinterpret_cast<char *>(&value), sizeof(int)) < 0)
                    {
                        close(s);
                        return invalid_socket;
                    }
                }

                if (::bind(s, reinterpret_cast<sockaddr *>(&server), sizeof(server)) < 0)
                {
                    close(s);
                    return invalid_socket;
                }

                if (::listen(s, SOMAXCONN) < 0)
                {
                    close(s);
                    return invalid_socket;
                }

                return s;
            }

            SOCKET accept(SOCKET s)
            {
                return ::accept(s, 0, 0);
            }

            bool wait_for_data(const std::vector<SOCKET> &wait_for, std::vector<SOCKET> &activity)
            {
                fd_set read_fds;
                FD_ZERO(&read_fds);

                unsigned int n = 0;
                std::vector<SOCKET>::const_iterator i = wait_for.begin();
                while (i != wait_for.end())
                {
                    if (n < *i)
                        n = *i;

                    FD_SET(*i, &read_fds);
                    ++i;
                }

                int result = ::select(n + 1, &read_fds, 0, 0, 0);
                if (result < 0) // ERROR
                    return false; // Should be throw

                i = wait_for.begin();
                while (i != wait_for.end())
                {
                    if (FD_ISSET(*i, &read_fds))
                        activity.push_back(*i);
                    ++i;
                }

                return true;
            }

            bool send_bytes(SOCKET s, const char *bytes, unsigned int left_to_send)
            {
                fd_set write_fds;
                timeval timeout = { 10 /* tv_sec */, 0 /* tv_usec */ };

                unsigned int bytes_sent = 0;
                while (bytes_sent < left_to_send)
                {
                    FD_ZERO(&write_fds);
                    FD_SET(s, &write_fds);

                    int result = ::select(s + 1, 0, &write_fds, 0, &timeout);
                    if (result < 0 || result == 0) // Error or timeout
                        return false;
                    assert(result == 1);
                    assert(FD_ISSET(s, &write_fds));

                    int n = ::send(s, bytes + bytes_sent, left_to_send - bytes_sent, 0);
                    if (n < 0 || n == 0)
                        return false;
                    bytes_sent += n;
                }
                return true;
            }

            bool receive_bytes(SOCKET s, char *buffer, unsigned int left_to_recv)
            {
                fd_set read_fds;
                timeval timeout = { 10 /* tv_sec */, 0 /* tv_usec */ };

                unsigned int bytes_recvd = 0;
                while (bytes_recvd < left_to_recv)
                {
                    FD_ZERO(&read_fds);
                    FD_SET(s, &read_fds);

                    int result = ::select(s + 1, &read_fds, 0, 0, &timeout);
                    if (result < 0 || result == 0) // Error or timeout
                        return false;
                    assert(result == 1);
                    assert(FD_ISSET(s, &read_fds));

                    int n = ::recv(s, buffer + bytes_recvd, left_to_recv - bytes_recvd, 0);
                    if (n == 0 || n < 0) // socket close or error
                        return false;
                    bytes_recvd += n;
                }
                return true;
            }

            bool send_text(SOCKET s, const std::string &text)
            {
                if (s == invalid_socket)
                    return false;

                unsigned int length = static_cast<unsigned int>(text.size());
                if (!tcpip::send_bytes(s, reinterpret_cast<char *>(&length), sizeof(length)))
                    return false;

                if (!tcpip::send_bytes(s, text.c_str(), length))
                    return false;

                return true;
            }

            bool receive_text(SOCKET s, std::string &text)
            {
                if (s == invalid_socket)
                    return false;

                int length = 0;
                if (!tcpip::receive_bytes(s, reinterpret_cast<char *>(&length), sizeof(length)))
                    return false;

                text.reserve(length + 1);
                std::auto_ptr<char> buffer(new char [length + 1]);

                if (!tcpip::receive_bytes(s, buffer.get(), length))
                    return false;

                *(buffer.get() + length) = '\0';

                text = buffer.get();
                return true;
            }
        }

        // TimeRep

        double TimeRep::unit = 1;
        const long NS_PER_S = 1000000000L;

        void TimeRep::norm()
        {
            if (m_ts.tv_nsec < 0)
            {
                long r = -m_ts.tv_nsec / NS_PER_S + 1;
                m_ts.tv_nsec += r * NS_PER_S;
                m_ts.tv_sec -= r;
            }
            else if (m_ts.tv_nsec > NS_PER_S)
            {
                long r = m_ts.tv_nsec / NS_PER_S;
                m_ts.tv_nsec -= r * NS_PER_S;
                m_ts.tv_sec += r;
            }
        }

        void TimeRep::zero() {set(0, 0);}

        void TimeRep::setNow()
        {
            // We ignore errors here!
            clock_gettime(CLOCK_REALTIME, &m_ts);
        }

        void TimeRep::set(const TimeRep &x) 
        { 
            set(x.m_ts.tv_sec, x.m_ts.tv_nsec); 
        }
        void TimeRep::set(double d) 
        {
            d *= unit;
            long s = static_cast<long>(d);
            long ns = static_cast<long>((d - s) * NS_PER_S);
            set(s, ns); 
        }
        void TimeRep::add(const TimeRep &x) { set(m_ts.tv_sec + x.m_ts.tv_sec, m_ts.tv_nsec + x.m_ts.tv_nsec); }
        void TimeRep::add(long s, long ns) { set(m_ts.tv_sec + s, m_ts.tv_nsec + ns); }
        void TimeRep::sub(const TimeRep &x) { set(m_ts.tv_sec - x.m_ts.tv_sec, m_ts.tv_nsec - x.m_ts.tv_nsec); }
        void TimeRep::sub(long s, long ns) { set(m_ts.tv_sec  - s, m_ts.tv_nsec - ns); }
        void TimeRep::sub(const TimeRep &a, const TimeRep &b) { set(a.m_ts.tv_sec  - b.m_ts.tv_sec, a.m_ts.tv_nsec - b.m_ts.tv_nsec); }

        int TimeRep::cmp(const TimeRep &x) const
        {
            if (m_ts.tv_sec < x.m_ts.tv_sec) 
                return -1;
            if (m_ts.tv_sec > x.m_ts.tv_sec)
                return 1;
            if (m_ts.tv_nsec < x.m_ts.tv_nsec)
                return -1;
            if (m_ts.tv_nsec > x.m_ts.tv_nsec) 
                return 1;

            return 0;
        }

        double TimeRep::to_double() const
        {
            return (static_cast<double>(m_ts.tv_nsec) / NS_PER_S + m_ts.tv_sec) / unit;
        }

        long TimeRep::to_ms() const
        {
            if (m_ts.tv_sec > 1000000) // prevent overflow!
                return 1000000000;

            return m_ts.tv_sec * 1000 + m_ts.tv_nsec / 1000000;
        }

        std::ostream &TimeRep::print(std::ostream &os) const
        {
            os << '{' << m_ts.tv_sec << ',' << std::setw(9) << std::setfill('0') << m_ts.tv_nsec << '}';
            return os;
        }

        void TimeRep::set(long s, long ns)
        {
            m_ts.tv_sec = s;
            m_ts.tv_nsec = ns;
            norm();
        }

        double TimeRep::setUnit(double u)
        {
            std::swap(TimeRep::unit, u);
            return u;
        }

        // MutexRep

        MutexRep::MutexRep() : m_mutex(), m_bInit(false) {}
        MutexRep::~MutexRep() { destroy(); }

        Status MutexRep::init(bool initialOwner)
        {
            if(m_bInit) return StatusFAIL; // already init

            pthread_mutexattr_t attr;

            if( pthread_mutexattr_init(&attr) ||
                pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_RECURSIVE) ||
                pthread_mutex_init(&m_mutex, &attr) )
                return StatusFAIL;

            pthread_mutexattr_destroy(&attr);
            m_bInit=true;

            if (initialOwner)
                return pthread_mutex_lock(&m_mutex) == 0 ? StatusOK : StatusFAIL;

            return StatusOK;
        }

        Status MutexRep::destroy()
        {
            if(!m_bInit) return StatusOK;   // already destroyed
            
            if(pthread_mutex_destroy(&m_mutex) == 0)
            {
                m_bInit=false;
                return StatusOK;
            }

            return StatusFAIL;
        }

        Status MutexRep::lock()
        {
            if(!m_bInit) return StatusFAIL;
            return pthread_mutex_lock(&m_mutex) == 0 ? StatusOK : StatusFAIL;
        }

        Status MutexRep::unlock()
        {
            if(!m_bInit) return StatusFAIL;
            return pthread_mutex_unlock(&m_mutex) == 0 ? StatusOK : StatusFAIL;
        }

        // SemaphoreRep

        SemaphoreRep::SemaphoreRep() : m_mutex(), m_cond(), m_count(0), m_bInit(false) {}

        SemaphoreRep::~SemaphoreRep()
        {
            destroy();
        }

        Status SemaphoreRep::init(unsigned int initialCount, unsigned int maxCount)
        {
            if(!m_bInit)
            {
                // This always returns 0 (so we do not check it).
                pthread_mutex_init(&m_mutex, 0);
                pthread_cond_init(&m_cond, 0);
                m_bInit=true;
            }
            m_count = initialCount;
            return StatusOK;
        }

        Status SemaphoreRep::destroy()
        {
            if(!m_bInit) return StatusOK;   // already destroyed

            if (pthread_mutex_destroy(&m_mutex) != 0)
                return StatusFAIL;

            if (pthread_cond_destroy(&m_cond) != 0)
                return StatusFAIL;

            m_bInit=false;
            return StatusOK;
        }



        Status SemaphoreRep::put(unsigned int count)
        {
            if(!m_bInit) return StatusFAIL;

            if (pthread_mutex_lock(&m_mutex) != 0)
                return StatusFAIL;

            if (m_count == 0)
                pthread_cond_signal(&m_cond);
            ++m_count;

            if (pthread_mutex_unlock(&m_mutex) != 0)
                return StatusFAIL;

            return StatusOK;
        }

        Status SemaphoreRep::get()
        {
            if(!m_bInit) return StatusFAIL;

            if (pthread_mutex_lock(&m_mutex) != 0)
                return StatusFAIL;

            while (m_count == 0)
                pthread_cond_wait(&m_cond, &m_mutex);
            --m_count;

            if (pthread_mutex_unlock(&m_mutex) != 0)
                return StatusFAIL;

            return StatusOK;
        }

        Status SemaphoreRep::get(Time timeout)
        {
            if(!m_bInit) return StatusFAIL;

            if (pthread_mutex_lock(&m_mutex) != 0)
                return StatusFAIL;

            timespec ts;
            double d = timeout.to_double();
            ts.tv_sec = static_cast<long>(d);
            ts.tv_nsec = static_cast<long>((d - ts.tv_sec) * NS_PER_S);

            int status = 0;
            while (m_count == 0 && status == 0)
            {
                status = pthread_cond_timedwait(&m_cond, &m_mutex, &ts);
            }
            if (status == 0)
                --m_count;

            if (pthread_mutex_unlock(&m_mutex) != 0)
                return StatusFAIL;

            if (status == 0)
                return StatusOK;

            if (status == ETIMEDOUT )
                return StatusTIMEOUT;

            return StatusFAIL;
        }

        // OS dependent part of the tor::os::Thread interface

        namespace Thread
        {
            extern "C" {
                typedef void *(*ttt)(void *);
            }
            bool createThread(ThreadFunc behaviour, void *arg)
            {
                pthread_attr_t attr;
                pthread_t thread_id;

                pthread_attr_init(&attr);
                pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);

                int id = 0;
                if (pthread_create(&thread_id, &attr, reinterpret_cast<ttt>(behaviour), arg) != 0)
                    return false;

                return thread_id != 0 ? StatusOK : StatusFAIL;
            }

            void suspend(Time timeout)
            {
                Time now = Time::now();
                if (timeout <= now)
                    return;

                Time duration(timeout, now);

                timespec ts;
                ts.tv_sec = duration.to_ms() / 1000;
                ts.tv_nsec = (duration.to_ms() - ts.tv_sec * 1000) * 1000000;

                if (ts.tv_sec > 0 || ts.tv_nsec > 0)
                    nanosleep(&ts, 0);
            }

            unsigned long id()
            {
                return static_cast<unsigned long>(pthread_self());
            }
        }

        //

        ProcessRep::ProcessRep() 
          : m_pid(invalid_pid)          
        {}

        ProcessRep::~ProcessRep() {}

        bool ProcessRep::launch(const tstring &cmd, const std::vector<tstring> &args, const tstring &dir)
        {
            m_pid = fork();
            if (m_pid == invalid_pid)
                return false;

            if (m_pid == 0) 
            {
                char **argv = new char *[args.size() + 1];
                char **a = argv;

                std::vector<tstring>::const_iterator i = args.begin();
                while (i != args.end())
                {
                    *a = new char [i->size() + 1];
		    Charstring arg(*i);	// to be able to get char* even for wchar string
                    strcpy(*a, arg.getNarrowString().c_str());
                    ++i;
                    ++a;
                }
                *a = 0;

		Charstring _cmd(cmd);	// to be able to get char* even for wchar string
                ::execvp(_cmd.getNarrowString().c_str(), const_cast<char * const *>(argv));
                _exit(0);
            }
            return true;
        }

        bool ProcessRep::terminate()
        {
            if (m_pid != invalid_pid)
            {
                int result = ::kill(m_pid, 0);
                if (result == 0 || result == -1 && errno == EPERM) // is process running (killable)?
                {
                    if (::kill(m_pid, SIGKILL) < 0)
                        return false;
                }
                ::waitpid(m_pid, 0, 0);
                m_pid = invalid_pid;
            }
            return true;
        }

        bool ProcessRep::isTerminated(int& terminationValue)
        {
            int status;
            if(!waitpid(m_pid,&status,WNOHANG)) return false; // 0 if not finished

            if( WIFEXITED(status) ) 
            { terminationValue=WEXITSTATUS(status); return true; } // exited
    
            if( WIFSIGNALED(status) ) 
            { terminationValue=WTERMSIG(status); return true; } // was killed

            // very strange: if not exited and not killed!
            // shouldn't reach here!

            return false;
        }

        PID GetCurrentPID()
        {
            return static_cast<PID>(::getpid());
        }
    }
}

