// Generated for IBM Rational Tau 4.3.0.0.15182 at Wed Sep 30 04:38:20 MEST 2009
// Licensed Materials - Property of IBM 
// (c) Copyright IBM Corporation 2002,2009.  All Rights Reserved.
// 
// U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#include "torOs_win32.h"
#include "torTime.h"
#include "torProcess.h"

#include <windows.h>
#include <winsock.h>
#include <limits.h>
#include <time.h>
#include <assert.h>
#include <sys\timeb.h>

#include <memory>
#include <functional>
#include <algorithm>
#include <utility>
#include <iomanip>
#include <vector>

#ifdef TOR_DLL

namespace tor
{
    extern "C" int APIENTRY DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
    {
        if (dwReason == DLL_PROCESS_ATTACH)
        {
        }
        else if (dwReason == DLL_PROCESS_DETACH)
        {
        }
        return 1;   // ok
    }

}

#endif

namespace 
{

    using namespace tor::os;

    // Helper
    tstring quote(const tstring &str)
    {
        tstring result;

        if (str.empty())
            return result;

        bool quote = *str.rbegin() != _T('\"') && str.find_first_of(_T(" \t")) != tstring::npos;

        if (quote) 
            result.append(_T("\""));

        result.append(str);
        if (quote) 
            result.append(_T("\""));

        return result;
    }

    // This is not used at the moment, but I'm keeping it around 
    // in case we need this transform later.
#ifdef _UNICODE
    struct NarrowCharToWideChar : std::unary_function<char, wchar_t>
    {
        inline wchar_t operator()(char mbc)
        {
            wchar_t wc;
            mbtowc(&wc, &mbc, 1);
            return wc;
        }
    };
#endif

    // tcp/ip helper
    bool build_client_address(const char *host, unsigned int port, sockaddr_in &socket_address)
    {
        hostent *host_entry = gethostbyname(host);
        if (host_entry == 0)
            return false;

        const char *local_ip = inet_ntoa(*(in_addr *) *host_entry->h_addr_list);
        if (local_ip == 0)
            return false;

        unsigned long local_ip_number = inet_addr(local_ip);
        if (local_ip_number <= 0)
            return false;

        socket_address.sin_family = AF_INET;
        socket_address.sin_addr.s_addr = local_ip_number;
        socket_address.sin_port = htons(port);

        return true;
    }

    bool build_server_address(unsigned int port, sockaddr_in &socket_address)
    {
        socket_address.sin_family = AF_INET;
        socket_address.sin_addr.s_addr = htonl(INADDR_ANY);
        socket_address.sin_port = htons(port);
        return true;
    }
}

namespace tor 
{
    namespace os 
    {
        const SOCKET invalid_socket = INVALID_SOCKET;
        const PID invalid_pid = -1;
        const FD invalid_fd = INVALID_HANDLE_VALUE;

        namespace tcpip
        {
            void init()
            {
                static bool initDone = false;
                if (initDone)
                    return;

                WORD version_requested = MAKEWORD(2, 2);
                WSADATA wsa_data;

                int result = WSAStartup(version_requested, &wsa_data);
                if (result != 0)
                {
                    switch (result)
                    {
                    case WSASYSNOTREADY: 
                        throw Exception(indicates_that_the_underlying_network_subsystem_is_not_ready_for_network_communication);
                    case WSAVERNOTSUPPORTED: 
                        throw Exception(the_version_of_windows_sockets_support_requested_is_not_provided_by_this_particular_windows_sockets_implementation);
                    case WSAEINPROGRESS: 
                        throw Exception(a_blocking_windows_sockets_1_1_operation_is_in_progress);
                    case WSAEPROCLIM: 
                        throw Exception(limit_on_the_number_of_tasks_supported_by_the_windows_sockets_implementation_has_been_reached);
                    case WSAEFAULT: 
                        throw Exception(the_wsadata_is_not_a_valid_pointer);
                    default:
                        throw Exception(failed_to_load_ws2_32_dll);
                    }
                }
                if (LOBYTE(wsa_data.wVersion) != 2 || HIBYTE(wsa_data.wVersion) != 2) 
                {
                    WSACleanup();
                    throw Exception(could_not_find_a_ws2_32_dll_with_the_correct_version);
                }
                initDone = true;
            }

            bool close(SOCKET s)
            {
                ::closesocket(s);
                return true;
            }

            SOCKET create_client_socket(const char *host, unsigned int port)
            {
                sockaddr_in server;
                if (!build_client_address(host, port, server))
                    return invalid_socket;

                SOCKET s = ::socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
                if (s == invalid_socket)
                    return invalid_socket;

                if (::connect(s, reinterpret_cast<sockaddr *>(&server), sizeof(server)) < 0)
                {
#ifdef _DEBUG
                    int error = WSAGetLastError();
#endif
                    close(s);
                    return invalid_socket;
                }

                return s;
            }

            SOCKET create_server_socket(unsigned int port, bool reuse)
            {
                sockaddr_in server;
                if (!build_server_address(port, server))
                    return invalid_socket;

                SOCKET s = ::socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
                if (s == invalid_socket) 
                {
#ifdef _DEBUG
                    int error = WSAGetLastError();
#endif

                }

                if (reuse)
                {
                    int value = 1;
                    if (::setsockopt(s, SOL_SOCKET, SO_REUSEADDR, reinterpret_cast<char *>(&value), sizeof(int)) < 0)
                    {
                        close(s);
                        return invalid_socket;
                    }
                }
                if (::bind(s, reinterpret_cast<sockaddr *>(&server), sizeof(server)) < 0)
                {
                    close(s);
                    return invalid_socket;
                }

                // Go into listening mode

                if (::listen(s, SOMAXCONN) < 0)
                {
                    close(s);
                    return invalid_socket;
                }

                return s;
            }

            SOCKET accept(SOCKET s)
            {
                return ::accept(s, 0, 0);
            }

            bool wait_for_data(const std::vector<SOCKET> &wait_for, std::vector<SOCKET> &activity)
            {
                fd_set read_fds;
                FD_ZERO(&read_fds);

                std::vector<SOCKET>::const_iterator i = wait_for.begin();
                while (i != wait_for.end())
                    FD_SET(*i++, &read_fds);

                int result = ::select(static_cast<unsigned int>(wait_for.size()), &read_fds, 0, 0, 0);
                if (result < 0) // ERROR
                    return false; // Should be throw

                i = wait_for.begin();
                while (i != wait_for.end())
                {
                    if (FD_ISSET(*i, &read_fds))
                        activity.push_back(*i);
                    ++i;
                }

                return true;
            }

            bool send_bytes(SOCKET s, const char *bytes, unsigned int left_to_send)
            {
                fd_set write_fds;
                timeval timeout = { 10 /* tv_sec */, 0 /* tv_usec */ };

                unsigned int bytes_sent = 0;
                while (bytes_sent < left_to_send)
                {
                    FD_ZERO(&write_fds);
                    FD_SET(s, &write_fds);

                    int result = ::select(1, 0, &write_fds, 0, &timeout);
                    if (result < 0 || result == 0) // Error or timeout
                        return false;
                    assert(result == 1);
                    assert(FD_ISSET(s, &write_fds));

                    int n = ::send(s, bytes + bytes_sent, left_to_send - bytes_sent, 0);
                    if (n < 0 || n == 0)
                        return false;
                    bytes_sent += n;
                }
                return true;
            }

            bool receive_bytes(SOCKET s, char *buffer, unsigned int left_to_recv)
            {
                fd_set read_fds;
                timeval timeout = { 10 /* tv_sec */, 0 /* tv_usec */ };

                unsigned int bytes_recvd = 0;
                while (bytes_recvd < left_to_recv)
                {
                    FD_ZERO(&read_fds);
                    FD_SET(s, &read_fds);

                    int result = ::select(1, &read_fds, 0, 0, &timeout);
                    if (result < 0 || result == 0) // Error or timeout
                        return false;
                    assert(result == 1);
                    assert(FD_ISSET(s, &read_fds));

                    int n = ::recv(s, buffer + bytes_recvd, left_to_recv - bytes_recvd, 0);
                    if (n == 0 || n < 0) // socket close or error
                        return false;
                    bytes_recvd += n;
                }
                return true;
            }

            bool send_text(SOCKET s, const std::string &text)
            {
                if (s == invalid_socket)
                    return false;

                unsigned int length = static_cast<unsigned int>(text.size());
                if (!tcpip::send_bytes(s, reinterpret_cast<char *>(&length), sizeof(length)))
                    return false;

                if (!tcpip::send_bytes(s, text.c_str(), length))
                    return false;

                return true;
            }

            bool receive_text(SOCKET s, std::string &text)
            {
                if (s == invalid_socket)
                    return false;

                int length = 0;
                if (!tcpip::receive_bytes(s, reinterpret_cast<char *>(&length), sizeof(length)))
                    return false;

                text.reserve(length + 1);
                std::auto_ptr<char> buffer(new char [length + 1]);

                if (!tcpip::receive_bytes(s, buffer.get(), length))
                    return false;

                *(buffer.get() + length) = '\0';

                text = buffer.get();
                return true;
            }
        }

        // TimeRep

        double TimeRep::unit = 1;
        const long NS_PER_S = 1000000000L;

        void TimeRep::norm()
        {
            if (m_ns < 0) 
            {
                __time64_t r = -m_ns / NS_PER_S + 1;
                m_ns += r * NS_PER_S;
                m_s -= r;
            }
            else if (m_ns > NS_PER_S) 
            {
                __time64_t r = m_ns / NS_PER_S;
                m_ns -= r * NS_PER_S;
                m_s += r;
            }
        }

        void TimeRep::zero() {set(0, 0);}

        void TimeRep::setNow()
        {
            struct _timeb current_time;
#if defined(WIN32) && !defined(MAINWIN)
            _ftime_s(&current_time);
#else
            _ftime(&current_time);
#endif
            set(current_time.time, current_time.millitm * 1000000);
        }

        void TimeRep::set(const TimeRep &x) { set(x.m_s, x.m_ns); }
        void TimeRep::set(double d) 
        {
            d *= unit;
            long s = static_cast<long>(d);
            long ns = static_cast<long>((d - s) * NS_PER_S);
            set(s, ns); 
        }
        void TimeRep::add(const TimeRep &x) { set(m_s + x.m_s, m_ns + x.m_ns); }
        void TimeRep::add(__time64_t s, __time64_t ns) { set(m_s + s, m_ns + ns); }
        void TimeRep::sub(const TimeRep &x) { set(m_s - x.m_s, m_ns - x.m_ns); }
        void TimeRep::sub(__time64_t s, __time64_t ns) { set(m_s  - s, m_ns - ns); }
        void TimeRep::sub(const TimeRep &a, const TimeRep &b) { set(a.m_s  - b.m_s, a.m_ns - b.m_ns); }

        int TimeRep::cmp(const TimeRep &x) const
        {
            if (m_s < x.m_s) 
                return -1;
            if (m_s > x.m_s)
                return 1;
            if (m_ns < x.m_ns)
                return -1;
            if (m_ns > x.m_ns) 
                return 1;

            return 0;
        }

        double TimeRep::to_double() const
        {
            return (static_cast<double>(m_ns) / NS_PER_S + m_s) / unit;
        }

        long TimeRep::to_ms() const
        {
            if (m_s > 1000000) // prevent overflow!
                return 1000000000;

            return (long)(m_s * 1000 + m_ns / 1000000);
        }

        std::ostream &TimeRep::print(std::ostream &os) const
        {
            os << '{' << m_s << ',' << std::setw(9) << std::setfill('0') << m_ns << '}';
            return os;
        }

        void TimeRep::set(__time64_t s, __time64_t ns)
        {
            m_s = s;
            m_ns = ns;
            norm();
        }

        double TimeRep::setUnit(double unit)
        {
            std::swap(TimeRep::unit, unit);
            return unit;
        }

        // MutexRep

        MutexRep::MutexRep() : m_handle(INVALID_HANDLE_VALUE) {}

        MutexRep::~MutexRep() 
        {
            destroy();
        }

        Status MutexRep::init(bool initialOwner)
        {
            if (m_handle != INVALID_HANDLE_VALUE)
                return StatusFAIL;

            m_handle = CreateMutex(0, initialOwner, 0);
            if (m_handle == 0)
            {
                m_handle = INVALID_HANDLE_VALUE;
                return StatusFAIL;
            }
            return StatusOK;
        }

        Status MutexRep::destroy()
        {
            if (m_handle == INVALID_HANDLE_VALUE)
                return StatusOK;    // already destroyed

            HANDLE h = m_handle;
            m_handle = INVALID_HANDLE_VALUE;

            return CloseHandle(h) ? StatusOK : StatusFAIL;
        }

        Status MutexRep::lock()
        {
            if (m_handle == INVALID_HANDLE_VALUE)
                return StatusFAIL;

            return WaitForSingleObject(m_handle, INFINITE) == WAIT_OBJECT_0 ? StatusOK : StatusFAIL;
        }

        Status MutexRep::unlock()
        {
            if (m_handle == INVALID_HANDLE_VALUE)
                return StatusFAIL;

            return ReleaseMutex(m_handle) ? StatusOK : StatusFAIL;
        }

        // SemaphoreRep

        SemaphoreRep::SemaphoreRep() : m_handle(INVALID_HANDLE_VALUE) {}

        SemaphoreRep::~SemaphoreRep()
        {
            destroy();
        }

        Status SemaphoreRep::init(unsigned int initialCount, unsigned int maxCount)
        {
            if (m_handle != INVALID_HANDLE_VALUE)
                return StatusFAIL;

            m_handle = CreateSemaphore(0, initialCount, maxCount, 0);
            if (m_handle == 0)
            {
                m_handle = INVALID_HANDLE_VALUE;
                return StatusFAIL;
            }
            return StatusOK;
        }

        Status SemaphoreRep::destroy()
        {
            if (m_handle == INVALID_HANDLE_VALUE)
                return StatusFAIL;

            HANDLE h = m_handle;
            m_handle = INVALID_HANDLE_VALUE;

            return CloseHandle(h) ? StatusOK : StatusFAIL;
        }

        Status SemaphoreRep::put(unsigned int count)
        {
            if (m_handle == INVALID_HANDLE_VALUE)
                return StatusFAIL;
            return ReleaseSemaphore(m_handle, count, 0) ? StatusOK : StatusFAIL;
        }

        Status SemaphoreRep::get()
        {
            if (m_handle == INVALID_HANDLE_VALUE)
                return StatusFAIL;
            return WaitForSingleObject(m_handle, INFINITE) == WAIT_OBJECT_0 ? StatusOK : StatusFAIL;
        }

        Status SemaphoreRep::get(Time timeout)
        {
            if (m_handle == INVALID_HANDLE_VALUE)
                return StatusFAIL;

            DWORD duration = 0;

            Time now = Time::now();
            if (timeout > now)
                duration = Time(timeout, now).to_ms();

            DWORD status = WaitForSingleObject(m_handle, duration);
            if (status == WAIT_OBJECT_0)
                return StatusOK;
            if (status == WAIT_TIMEOUT)
                return StatusTIMEOUT;
            return StatusFAIL;
        }

        // OS dependent part of the tor::os::Thread interface

        namespace Thread
        {
            bool createThread(ThreadFunc behaviour, void *arg)
            {
                return CreateThread(0, 0, reinterpret_cast<LPTHREAD_START_ROUTINE>(behaviour), arg, 0, 0) != 0;
            }

            void suspend(Time timeout)
            {
                DWORD duration = 0;

                Time now = Time::now();
                if (timeout > now)
                    duration = Time(timeout, now).to_ms();

                Sleep(duration);
            }

            unsigned long id()
            {
                return static_cast<unsigned long>(GetCurrentThreadId());
            }
        }

        //

        ProcessRep::ProcessRep() 
          : m_handle(INVALID_HANDLE_VALUE),
            m_pid(invalid_pid)
        {}

        ProcessRep::~ProcessRep() {}

        bool ProcessRep::launch(const tstring &cmd, const std::vector<tstring> &argv, const tstring &dir)
        {
            // Convert argv to a command line with spaces separating each argument.
            tstring cmdline;
            cmdline.append(_T("\"")).append(cmd).append(_T("\"")).append(_T(" ")); // N.B. Path may contain spaces
            
            std::vector<tstring>::const_iterator i = argv.begin();
            while (i != argv.end())
                cmdline.append((i++)->c_str()).append(_T(" "));;

            // Setup startup info and process info

            STARTUPINFO startupInfo;
            memset(&startupInfo, 0, sizeof(startupInfo));
            startupInfo.cb = sizeof(startupInfo);

            PROCESS_INFORMATION processInformation;

            // Options
            DWORD creationFlags = NORMAL_PRIORITY_CLASS;
            if (m_options & Process::NO_CONSOLE)
                creationFlags |= CREATE_NO_WINDOW;
            else
                creationFlags |= CREATE_NEW_CONSOLE;

            // Create process

            if (!CreateProcess(cmd.c_str(),                             // ApplicationName
                               const_cast<TCHAR *>(cmdline.c_str()),    // CommandLine
                               0,                                       // ProcessAttributes
                               0,                                       // ThreadAttributes
                               FALSE,                                   // InheritHandles
                               creationFlags,                           // CreationFlags
                               0,                                       // Environment
                               dir.empty() ? 0 : dir.c_str(),           // CurrentDirectory
                               &startupInfo,                            // StartupInfo
                               &processInformation))                    // ProcessInformation
            {
#ifdef _DEBUG
                DWORD error = GetLastError();
                LPVOID message;
                FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM, 0, error, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) &message, 0, 0);
#endif
                return false;
            }
            CloseHandle(processInformation.hThread);

            m_handle = processInformation.hProcess;
            m_pid = processInformation.dwProcessId;

            return true;
        }

        bool ProcessRep::terminate()
        {
            if (m_handle != INVALID_HANDLE_VALUE)
            {
                if (TerminateProcess(m_handle, EXIT_FAILURE) == 0)
                {
#ifdef _DEBUG
                    DWORD error = GetLastError();
                    LPVOID message;
                    FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM, 0, error, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) &message, 0, 0);
#endif
                    return false;
                }

                if (WaitForSingleObject(m_handle, INFINITE) != WAIT_OBJECT_0)
                    return false;
            }

            return true;
        }

        bool ProcessRep::isTerminated(int& terminationValue)
        {
            DWORD exitCode;
            terminationValue = 0;
            if (!::GetExitCodeProcess(m_handle, &exitCode))
                return false; // Could for example mean that the process is never launched.

            if (exitCode == STILL_ACTIVE)
                return false; // Process is still running.

            terminationValue = exitCode;

            return true;
        }

        PID GetCurrentPID()
        {
            return GetCurrentProcessId();
        }
    }
}

