/*************************************************************
Generated for IBM Rational Tau 4.3.0.0.15182 at Wed Sep 30 04:38:22 MEST 2009
Licensed Materials - Property of IBM 
(c) Copyright IBM Corporation 2002,2009.  All Rights Reserved.

U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*************************************************************/
#ifndef __torOs_win32_h
#define __torOs_win32_h

#include "torTypes.h"
#include <ostream>
#include <vector>

namespace tor 
{
    namespace os 
    {
        typedef void * HANDLE; // Windows handle

        typedef _W64 unsigned int SOCKET;
        TOR_EXT_DEF extern const SOCKET invalid_socket;

        typedef unsigned long PID;
        TOR_EXT_DEF extern const PID invalid_pid;

        typedef HANDLE FD;
        TOR_EXT_DEF extern const FD invalid_fd;

        //
        // tcp/ip
        //

        namespace tcpip
        {
            void init();
            bool close(SOCKET s);
            SOCKET create_client_socket(const char *host, unsigned int port);
            SOCKET create_server_socket(unsigned int port, bool reuse);

            SOCKET accept(SOCKET s);

            bool wait_for_data(const std::vector<SOCKET> &wait_for, std::vector<SOCKET> &activity);

            bool send_bytes(SOCKET s, const char *bytes, unsigned int left_to_send);
            bool receive_bytes(SOCKET s, char *buffer, unsigned int left_to_recv);

            bool send_text(SOCKET s, const std::string &text);
            bool receive_text(SOCKET s, std::string &text);
        }

        //
        // TimeRep
        //

        class TOR_EXT_DEF TimeRep
        {
        public:
            TimeRep() : m_s(0), m_ns(0)  {}
            TimeRep(long s, long ns) : m_s(), m_ns() { set(s, ns); }
            TimeRep(const TimeRep &a, const TimeRep &b) : m_s(), m_ns() { set(a.m_s - b.m_s, a.m_ns - b.m_ns); }
            TimeRep(const TimeRep &a) : m_s(), m_ns() { set(a.m_s, a.m_ns); }
            ~TimeRep() {}

            void norm();
            void zero();
            void setNow();

            void set(const TimeRep &x);
            void set(__time64_t s, __time64_t ns);
            void set(double d);
            
            void add(const TimeRep &x);
            void add(__time64_t s, __time64_t ns);

            void sub(const TimeRep &x);
            void sub(__time64_t s, __time64_t ns);
            void sub(const TimeRep &a, const TimeRep &b);

            int cmp(const TimeRep &x) const;

            double to_double() const;
            long to_ms() const;

            std::ostream &print(std::ostream &os) const;

            static double setUnit(double u);

        private:
            // Times are represented according to the standard: second, nano-second.
            __time64_t m_s;
            __time64_t m_ns;

            static double unit;
        };

        //
        // MutexRep
        //

        class TOR_EXT_DEF MutexRep 
        {
        public:
            MutexRep();
            ~MutexRep();

            Status init(bool initialOwner);
            Status destroy();

            Status lock();
            Status unlock();

        private:
            HANDLE m_handle;
        };

        //
        // SemaphoreRep
        //

        class Time;
        class TOR_EXT_DEF SemaphoreRep
        {
        public:
            SemaphoreRep();
            ~SemaphoreRep();

            Status init(unsigned int initialCount, unsigned int maxCount);
            Status destroy();

            Status put(unsigned int count);

            Status get();
            Status get(Time timeout);

        private:
            HANDLE m_handle;
        };

        //
        // ProcessRep
        //

        class ProcessRep : public ProcessRepCommon
        {
        public:
            ProcessRep();
            ~ProcessRep();

            bool launch(const tstring &cmd, const std::vector<tstring> &argv, const tstring &dir);
            bool terminate();

            bool isTerminated(int& terminationValue);

            PID getPID() const { return m_pid; }
        private:
            HANDLE m_handle;
            PID m_pid;
        };
    }
}

#endif

