// Generated for IBM Rational Tau 4.3.0.0.15182 at Wed Sep 30 04:38:20 MEST 2009
// Licensed Materials - Property of IBM 
// (c) Copyright IBM Corporation 2002,2009.  All Rights Reserved.
// 
// U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#include "torPlatform.h"

#include "torRegion.h"

#include <list>
#include "torEntryPoint.h"
#include "torExitPoint.h"
#include "torState.h"
#include "torStateMachine.h"
#include "torEvent.h"

tor::Region::~Region()
{
}
  

tor::Dispatchable::EventAction 
tor::Region::execute( Event* e) {

  Dispatchable::EventAction result = Dispatchable::NoMatch;

  // Handle event in current state if any.
  State* state = m_current;
  
  // Otherwise use owner of this region.
  if (!m_current) {
      state = m_owner;
  }

  if (!state) {
    return result;
  }

  StateMachine* submachine = state->getStateMachine();
  if (submachine) {
    // Execute event in submachine
    result = submachine->execute(e);
  } else {
    // Loop over all regions and try to execute the event
    std::list<Region*>::const_iterator citer = state->m_regions.begin();
    while ( citer != state->m_regions.end() ) {
	      result = (*citer)->execute( e );
        if ( result != Dispatchable::NoMatch ) {
	        // Event was consumed or deferred, evaluate transitions triggered only by guards.
          (*citer)->execute( 0 );    
        }
	    ++citer;
    }
  }

  // If event was not consumed by submachine nor regions, handle in current state
  if ( result == Dispatchable::NoMatch ) {
    // Event is about to be received!
    e->received(); 

    result = state->execute(e);
    /*
    if ( result != Dispatchable::NoMatch ) {
      // Event was consumed or deferred, evaluate transitions triggered only by guards.
      state->execute( 0 );
    }*/
  }

  return result;
}

void 
tor::Region::enter()
{
  // Mark region as not finished.
  m_finished = false;

  // Enter region using currently active state. 
  // If there's no current state, use initial transition.
  if (m_current) {
      m_current->enter();
  } else {
      callInitialTransition();
  }
}

void 
tor::Region::enterHistory(bool deepHistory /* = false */)
{
  // Mark region as not finished.
  m_finished = false;

  // Enter previous state if any, otherwise use default history transition.
  if (m_previous) {
    if (deepHistory)
      m_previous->enterHistory(deepHistory);
    else
      m_previous->enter();
  } else {		    
    defaultHistoryTransition();
  }
}

void 
tor::Region::leave()
{
  // Leave the region by leaving the currently active state.
  if (m_current)
    m_current->leave();
}

void 
tor::Region::finish(ExitPoint* ep /* = 0 */)
{
  if (ep && ep->m_owner == this) {
    StateMachine* sm = m_owner->getStateMachine();
    if (sm)
      sm->leave(ep);
  } else {
    // Mark region as finished.
    m_finished = true;

    if (m_owner) {
      // If all regions of the owning state are finished, finish the state.
      if (m_owner->isFinished())
          m_owner->finish();
    }
  }
}

