// Generated for IBM Rational Tau 4.3.0.0.15182 at Wed Sep 30 04:38:20 MEST 2009
// Licensed Materials - Property of IBM 
// (c) Copyright IBM Corporation 2002,2009.  All Rights Reserved.
// 
// U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#include "torPlatform.h"

#include "torState.h"

#include "torTypes.h"
#include "torCompletedEvent.h"
#include "torExitPoint.h"
#include "torRegion.h"
#include "torTopRegion.h"
#include "torStateMachine.h"

tor::State::~State()
{
  // Delete owned submachine if present
  StateMachine* submachine = getStateMachine();
  if (submachine)
      delete submachine;

  // Delete all owned regions
  std::list<Region*>::const_iterator citer = m_regions.begin();
  while ( citer != m_regions.end() ) {
    if (*citer)
      delete *citer;
    ++citer;
  }
}

void
tor::State::enter(EntryPoint* ep)
{
    // Update the previous and currently active states of the owning region.
    // Note: Done before entering the submachine / owned regions
    // to get correct semantics.
    m_owner->m_previous = m_owner->m_current;
    m_owner->m_current = this;

    // Enter the submachine if present, otherwise enter all owned regions.
    StateMachine* subMachine = getStateMachine();
    if (subMachine) {
        subMachine->enter(ep);
    } 
    else {
        std::list<Region*>::const_iterator citer = m_regions.begin();
        while ( citer != m_regions.end() ) {
            (*citer)->enter();
            ++citer;
        }
    }

    // Evaluate transitions triggered only by guards.
    execute(0);
}

void
tor::State::enterFrom(State* source, EntryPoint* ep) 
{
  // Get least common ancestor region of this state and the source state.
  Region* lcr = GetLCR(this, source);

  if (!lcr) 
      return; // Should never happen, responsibility of the caller.

  Region* tmpRegion = 0;
  State* tmpState = this;

  // Set current and previous states for all regions up to the ancestor
  while (tmpRegion != lcr) {
    tmpRegion = tmpState->m_owner;
    tmpRegion->m_previous = tmpRegion->m_current;
    tmpRegion->m_current = this;
    tmpState = tmpRegion->m_owner;
  }

  // Get the parent of this state in the ancestor region.
  State* parent = GetParentInRegion(this, lcr);

  // Enter the parent and thereby recursively enter all children
  // until this state is entered.
  parent->enter(ep);
}

void
tor::State::enterHistory(bool deepHistory /* = false */)
{

  // Set current and previous states of the owning region.
  // Note: Done before entering the submachine / owned regions
  // to get correct semantics.
  m_owner->m_previous = m_owner->m_current;
  m_owner->m_current = this;

  // Using history, enter the submachine if present, otherwise enter all owned regions.
  StateMachine* subMachine = getStateMachine();
  if (subMachine) {
    subMachine->enterHistory(deepHistory);
  } else {
    std::list<Region*>::const_iterator citer = m_regions.begin();
    while ( citer != m_regions.end() ) {
      (*citer)->enterHistory(deepHistory);
      ++citer;
    }
  }
}


void
tor::State::enterHistoryFrom(State* source) 
{
  // Get least common ancestor region of this state and the source state.
  Region* lcr = GetLCR(this, source);

  if (!lcr) 
      return; // Should never happen, responsibility of the caller.

  Region* tmpRegion = 0;
  State* tmpState = this;

  // Set current and previous states for all regions up to the ancestor
  while (tmpRegion != lcr) {
    tmpRegion = tmpState->m_owner;
    tmpRegion->m_previous = tmpRegion->m_current;
    tmpRegion->m_current = this;
    tmpState = tmpRegion->m_owner;
  }

  // Enter the parent using history, and thereby recursively enter 
  // all children until this state is entered.
  State* parent = GetParentInRegion(this, lcr);
  parent->enterHistory();

}

void
tor::State::leave()
{
  // Leave the submachine if present otherwise leave all owned regions.
  StateMachine* subMachine = getStateMachine();
  if (subMachine) {
    subMachine->leave();
  } else {
    std::list<Region*>::const_iterator citer = m_regions.begin();
    while ( citer != m_regions.end() ) {
      (*citer)->leave();
      ++citer;
    }
  }

  // Set current and previous states of the owning region.
  // Note: Done after leaving the submachine/ owned regions
  // to get correct semantics.
  m_owner->m_previous = m_owner->m_current;
  m_owner->m_current = 0;

}

void
tor::State::leaveFor(State* target) 
{
    // Get least common ancestor region of this state and the target state.
    Region* lcr = GetLCR(this, target);

    if (!lcr) 
        return; // Should never happen, responsibility of the caller.

    Region* tmpRegion = 0;
    State* tmpState = this;

    // Leave all states from this one up to one owned by the ancestor region.
    // Note: Will loop if the ancestor region doesn't own any ancestor to this
    // state. In that case we should have returned above. 
    while (tmpRegion != lcr) {
      tmpState->leave();
      tmpRegion = tmpState->m_owner;
      tmpState = tmpRegion->m_owner;
    }

}

void
tor::State::leaveFor(ExitPoint* target) 
{
  if (!target || !(target->m_owner)) 
     return;

  // Get ancestor state owned by the region owning the exit point.
  State* exitPointState = GetParentInRegion(this, target->m_owner);

  if (!exitPointState) 
      return; // Should never happen, responsibility of the caller.

  // Leave for the ancestor.
  this->leaveFor(exitPointState);
}

void
tor::State::finish()
{
  // Create a completion event.
  CompletedEvent* e = new CompletedEvent();

  // Execute event synchronously.
  Dispatchable::EventAction result = execute(e);

  // Delete event if not consumed by the call to execute.
  if (result != Dispatchable::Consumed)
    delete e;
}

bool
tor::State::isFinished() {

  // f the state has a submachine, return the status of it.
  StateMachine* submachine = getStateMachine();
  if (submachine)
      return submachine->isFinished();

  // Otherwise loop over owned regions and return false
  // if any region is not finished.
  std::list<Region*>::const_iterator citer = m_regions.begin();
  while ( citer != m_regions.end() ) {
    if (!(*citer)->m_finished)
        return false;
    ++citer;
  }
  
  // All regions are finished, return true.
  return true;
}

void 
tor::State::addRegion(Region* r) 
{
  // Insert region into list.
  m_regions.push_back(r);
}

tor::Region* 
tor::State::GetLCR(const State* s1, const State* s2) 
{
  // This algorithm is O(n^2) based on the number of hierarchical levels
  // of regions. This should normally not be a problem since the number of
  // hierarchical levels is typically small, and rarely larger than 3.

  if (!s1 || !s2) 
      return 0;

  // If states has the same parent region, return this parent.
  if (s1->m_owner == s2->m_owner)
      return s1->m_owner;

  // Collect all parent regions of s1 into a list.
  std::list<Region*> s1parents;
  Region* tmp1 = s1->m_owner;
  while ( tmp1 ) {
    s1parents.push_back(tmp1);
    if (tmp1->m_owner) {
      tmp1 = tmp1->m_owner->m_owner;
    } else {
      tmp1 = 0;
    }
  }

  // Collect all parent regions of s2 into a list.
  std::list<Region*> s2parents;
  tmp1 = s2->m_owner;
  while ( tmp1 ) {
    s2parents.push_back(tmp1);
    if (tmp1->m_owner) {
      tmp1 = tmp1->m_owner->m_owner;
    } else {
      tmp1 = 0;
    }
  }

  // Loop over all parent regions of s1
  std::list<Region*>::const_iterator s1iter = s1parents.begin();
  while ( s1iter != s1parents.end() ) {
    
    // Loop over all parent regions of s2
    std::list<Region*>::const_iterator s2iter = s2parents.begin();
    while ( s2iter != s2parents.end() ) {
      // If region is a parent of both s1 and s2, return this region.
      if (*s1iter == *s2iter)
          return *s1iter;
      ++s2iter;
    }
    ++s1iter;
  }
  
  // No common ancestor region found, return 0
  return 0;
}

tor::State*
tor::State::GetParentInRegion(State* s, Region* r) {

  State* result = 0;
  State* tmp = s;

  // Recursively get the parent state of this state searching in the 
  // enclosing region until the enclosing region is equal to the 
  // supplied region.
  while (tmp->m_owner && (tmp->m_owner != r)) {
     tmp = tmp->m_owner->m_owner;
  }

  result = tmp;
  
  return result;
}

