// Generated for IBM Rational Tau 4.3.0.0.15182 at Wed Sep 30 04:38:20 MEST 2009
// Licensed Materials - Property of IBM 
// (c) Copyright IBM Corporation 2002,2009.  All Rights Reserved.
// 
// U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#include "torPlatform.h"

#include "torStateMachine.h"

#include "torDispatchableClass.h"
#include "torDispatcher.h"
#include "torEntryPoint.h"
#include "torEvent.h"
#include "torEventQueue.h"
#include "torState.h"
#include "torTopRegion.h"

tor::StateMachine::~StateMachine()
{
  // TODO: Clean other regions than this TopRegion
  // Delete all owned regions
  //std::list<TopRegion*>::const_iterator citer = m_regions.begin();
  //while ( citer != m_regions.end() ) {
  //  if (*citer && (*citer != dynamic_cast<TopRegion*>(this)))
  //    delete *citer;
  //  ++citer;
  //}
}


tor::Dispatchable::EventAction
tor::StateMachine::execute( Event* e )
{
  Dispatchable::EventAction rv = Dispatchable::NoMatch;

  if ( !e )
    return Dispatchable::Consumed;


  // Loop over non-finished top level regions and try to dispatch the event
  std::list<TopRegion*>::const_iterator citer = m_regions.begin();
  while ( citer != m_regions.end() ) {
    if (!((*citer)->m_finished)) {
      rv = (*citer)->execute( e );
      if ( rv != Dispatchable::NoMatch )
        break;    // Event was consumed by this region
    }
    ++citer;
  }

  // If event wasn't consumed by any region and this statemachine is not 
  // a submachine, then consume it. If this statemachine is a submachine 
  // the event can be handled by the parent and shouldn't be consumed.
  if (( rv == Dispatchable::NoMatch ) && ( !isSubStateMachine() ) ) {
    rv = Dispatchable::Consumed;
    delete e;  
  }
    
  return rv;
}


void
tor::StateMachine::start()
{
  if (m_stateOfOperation == Started)
      return; // Already started.
  
  m_stateOfOperation = Started;
  
  // Enter the top level regions without passing any entry point.
  enter(0);
}

bool
tor::StateMachine::isFinished()
{
  // Check if any region is not finished and in that case return false.
  std::list<TopRegion*>::const_iterator citer = m_regions.begin();
  while ( citer != m_regions.end() ) {
    if (!(*citer)->m_finished)
        return false;
    ++citer;
  }

  // Otherwise return true.
  return true;
}

void
tor::StateMachine::finish() 
{
  if(m_stateOfOperation==Finished) return;

  m_stateOfOperation = Finished;

  if(isSubStateMachine()) return; // can't finish dispatchableClass

  DispatchableClass* dispatchableClass = getDispatchableClass();
  if (!dispatchableClass) 
    return;

  // Caution, this may delete ourselves, do not use data members after that
  dispatchableClass->finish();

}

bool tor::StateMachine::isSubStateMachine() const
{
  return (0 != getOwningState());
}

void
tor::StateMachine::addRegion(TopRegion* r) {
    m_regions.push_back(r); // Add region to region list.
}

void
tor::StateMachine::enter(EntryPoint* ep) 
{
  // Try to enter all top level regions using the supplied entry point
  std::list<TopRegion*>::const_iterator citer = m_regions.begin();
  while ( citer != m_regions.end() ) {    

    // Enter region through entry point if ep is owned by region,
    // otherwise enter region without entry point.
    if (ep && (ep->m_owner == (*citer))) 
        //ep->enter();
	ep->m_owner->entryPointTransitions(ep);
    else
      (*citer)->enter();
    ++citer;

  }
}

void
tor::StateMachine::enterHistory(bool deepHistory /* = false */ )
{
  // Enter all top level regions using history
  std::list<TopRegion*>::const_iterator citer = m_regions.begin();
  while ( citer != m_regions.end() ) {    
    // Enter region using history
    (*citer)->enterHistory(deepHistory);
    ++citer;
  }
}

void
tor::StateMachine::leave(ExitPoint* ep)
{
  // Leave all non-finished top level regions.
  std::list<TopRegion*>::const_iterator citer = m_regions.begin();
  while ( citer != m_regions.end() ) {
    if (!(*citer)->m_finished)
        (*citer)->leave();
    ++citer;
  }

  // If this is a submachine and an exit point was specified, 
  // search the owning state for matching transitions.
  if (isSubStateMachine() && ep) {
    Region* owningRegion = getOwningState()->m_owner;
    if (owningRegion)
        owningRegion->exitPointTransitions(ep);
  }
}

tor::StateMachine*
tor::StateMachine::getOwningStateMachine()
{
  // Recursively retrieve the owning region until we reach a 
  // top level region.

  State* owner = getOwningState();
  if (owner) {
    Region* region = owner->m_owner;
    while (region && !dynamic_cast<TopRegion*>(region)) {
      State* tmpState = region->m_owner;
      if (tmpState) {
        region = tmpState->m_owner;
      } else {
        region = 0;
      }
    }
    if (region) {
      // Return the statemachine of the top level region
      TopRegion* topRegion = dynamic_cast<TopRegion*>(region);
      return topRegion->getStateMachine();
    }
  }
  return 0;
}

