/*************************************************************
Generated for IBM Rational Tau 4.3.0.0.15182 at Wed Sep 30 04:38:22 MEST 2009
Licensed Materials - Property of IBM 
(c) Copyright IBM Corporation 2002,2009.  All Rights Reserved.

U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*************************************************************/
#ifndef __torStateMachine_h__
#define __torStateMachine_h__ (1)

#include "torPlatform.h"

#include "torDispatchable.h"
#include <list>

namespace tor {

    class DispatchableClass;
    class EntryPoint;
    class Event;
    class ExitPoint;
    class State;
    class TopRegion;

    class TOR_EXT_DEF StateMachine {
    public:

        // The state of operation of the statemachine.
        enum StateOfOperation {
            Initial,	    // Created, but not yet started
            Started,	    
            Finished
        } m_stateOfOperation;

        // Construction
        StateMachine() : m_stateOfOperation(Initial) {}

        // Deletes all top level regions owned by the statemachine. Subclasses are responsible
        // of deleting any states and entry/exit points.
        virtual ~StateMachine();

        // Executes an event by looping over the top regions to see if they can consume the event.
        // The first matching region consumes the event, deletes it and returns Consumed.
        // If no match is found, NoMatch is returned and the event is not deleted.
        Dispatchable::EventAction execute( Event* );

        // Initializes the statemachine by creating regions, states, entry/exit points, etc.
        // After calling init(), the statemachine is ready to be started using start().
        // This operation should be implemented by the concrete subclasses of StateMachine.
        virtual void init() { };

        // Starts the statemachine instance, executing the initial transition of all top level
        // regions. If the target of an initial transition is a composite state or a submachine,
        // their initial transitions are executed and so on.
        // After the call to start, the statemachine is running and is responding to events.
        // If the statemachine is already started when this function is called, nothing happens.
        void start();

        // Force the statemachine instance to finish. Removes the owning dispatchable
        // from its dispatcher. As a result of this the statemachine will not receive
        // any more events, and is considered to be finished.
        // Note: Finishing the statemachine does not imply leaving it.
        void finish();

        // Returns true if the statemachine is finished, false otherwise. The statemachine is
        // finished iff all top level regions are finished.
        bool isFinished();

        // Returns true if this statemachine is a submachine, false otherwise.
        // The getOwningState function is used to decide if it is a submachine or not.
        bool isSubStateMachine() const;

        // Enters the statemachine by entering all top level regions. If a valid entry point
        // is supplied, the regions will be entered using this entry point.
        void enter(EntryPoint* ep = 0);
    
        // Enter the statemachine using history, i.e. enter all top level regions using history.
        virtual void enterHistory(bool deepHistory = false);

        // Leaves the statemachine by leaving all non-finished top level regions. If an
        // exit point is supplied, the statemachine will be exited using this exit point.
        // after leacing the statemachine, any matching transition in the owning statemachine 
        // is executed by calling exitPointTransitions supplying the exit point as a parameter.
        void leave(ExitPoint* ep = 0);

        // Adds a new top level region to the statemachine.
        void addRegion(TopRegion*);

        // Returns the dispatchable class in which the statemachine is contained.
        virtual DispatchableClass* getDispatchableClass() const = 0;

        // A list of all top level regions of the statemachine.
        std::list<TopRegion*> m_regions;

        // Returns the owning statemachine. Returns 0 if this is a top-level statemachine.
        StateMachine* getOwningStateMachine();

        // Returns the owning state. Returns 0 if this is a top-level statemachine.    
        virtual State* getOwningState() const {
            return 0;
        }
    };

} // tor namespace

#endif  // __torStateMachine_h__

