/*************************************************************
Generated for IBM Rational Tau 4.3.0.0.15182 at Wed Sep 30 04:38:22 MEST 2009
Licensed Materials - Property of IBM 
(c) Copyright IBM Corporation 2002,2009.  All Rights Reserved.

U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*************************************************************/
#ifndef __torThread_h
#define __torThread_h

#include "torPlatform.h"

#include "torOsRep.h"
#include "torSync.h"

namespace tor
{
    namespace os
    {
        namespace Thread
        {
            // An interface that can be passed to our thread 
            // launch function

            class TOR_EXT_DEF Behaviour
            {
            public:
                Behaviour() {}
                virtual ~Behaviour() {}

                // Start executing the behaviour in a new thread
                virtual int execute() = 0;

                // Determines if the behaviour should be automatically deleted when the thread terminates.
                // Default is true. You may want to overload this if your behaviour shall continue to
                // exist after the thread has terminated.
                virtual bool deleteOnTerminate() { return true; }
            };

            //
            //
            //

            template <class T>
            class Data
            {
            public:
                Data() : m_mutex(new Mutex), m_data(new T), m_count(new unsigned int) 
                {
                    m_mutex->init();
                    auto_lock lock(*m_mutex);
                    *m_count = 1;
                }
                Data(T *t) : m_mutex(new Mutex), m_data(t), m_count(new unsigned int) 
                {
                    m_mutex->init();
                    auto_lock lock(*m_mutex);
                    *m_count = 1;
                }
                Data(const Data &d) : m_mutex(0), m_data(0), m_count(0)
                {
                    addReference(d);
                }
                ~Data()
                {
                    removeReference();
                }

                Data &operator=(const Data &d)
                {
                    if (this == &d)
                        return *this;

                    removeReference();
                    addReference(d);

                    return *this;
                }

                T *operator->() { return m_data; }
                const T *operator->() const { return m_data; }

            protected:
                void addReference(const Data &d)
                {
                    auto_lock lock(*d.m_mutex);
                    m_mutex = d.m_mutex;
                    m_data = d.m_data;
                    m_count = d.m_count;
                    ++(*m_count); 
                }
                void removeReference() 
                {
                    if (m_count == 0)
                        return;

                    auto_lock lock(*m_mutex);
                    if (--(*m_count) == 0)
                    {
                        delete m_mutex;
                        delete m_count;
                        delete m_data;
                    }
    #ifdef _DEBUG
                    m_data = 0;
                    m_mutex = 0;
                    m_count = 0;
    #endif
                }
                
            protected:
                Mutex *m_mutex;
                unsigned int *m_count;
                T *m_data;
            };


            // Launches a thread with the indicated behavior

            TOR_EXT_DEF bool launch(Behaviour *toExecute);
        }
    }
}

#endif

