# Generated for IBM Rational Tau 4.3.0.0.15182 at Wed Sep 30 04:38:17 MEST 2009
# Licensed Materials - Property of IBM 
# (c) Copyright IBM Corporation 2002,2009.  All Rights Reserved.
# 
# U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
Output "\nEclipse integration loading ... \n"

package require commands
package require dialogs

set installDir [std::GetInstallationDirectory]
if { (2 == [string length $installDir]) && (":" == [string index $installDir end]) } {
	set installDir ${installDir}/
}

source 	[file join $installDir addins/EclipseIntegration/Script/EclipseUtils.tcl]

##############################################################################
#	Eclipse related procedures
#

# Synchronize Tau project with Eclipse project
proc OnSynchronizeEclipse  {cmdName} {
	set project [std::GetActiveProject]
	if {$project != ""} {
		SynchronizeProject $project
	}
}

proc OnEnableSynchronizeEclipse {cmdName} {
	set project [std::GetActiveProject]
	if {"" == $project} {
		return 0
	}
	return [IsEclipseProject $project]
}

proc OnChangeEclipseExecutable {cmdName} {
	set optionsFile [file join [GetUserOptionsDir] eclipseOptions.ini]	
		set eclipseExe [GetOptionValue $optionsFile ExecutablePath]		
		set platformOptions [GetOptionValue $optionsFile PlatformOptions]	
			set eclipseExe [Dialog -variable "LocateEclipseExecutable" -name "Locate Eclipse Executable" -w 365 -h 180 -onbuilddialog OnBuildResetEclipseDialog -oninitdialog OnInitLocateEclipseDialog -onclosedialog OnCloseLocateEclipseDialog -closecmddialog CloseDialog]
			if {($eclipseExe != 0) && [file exists $eclipseExe]} {
				SetOptionValue $optionsFile ExecutablePath $eclipseExe
  			} else {
				Output "\nError: Invalid Eclipse executable specified."
				return 0
			}		

}

# Locate modelel element in java source code in Eclipse
proc OnLocateInEclipse {cmdName} {
	set elem [std::GetSelection]
	if [u2::IsKindOf $elem PresentationElement] {
		set target [u2::GetEntity $elem ModelElement]
	} else {
		set target $elem
	}
	if [LocateElementInEclipse [GetLocatableElement $target]] {
		Output "Done"		
	}
}

proc OnEnableLocateInEclipse {cmdName} {
	set elem [std::GetSelection]
	if {([llength $elem] == 1) && ([GetKind $elem] == "U2")} {
		set project [std::GetProject $elem]
		if {"" == $project} {
			return 0
		}
		if [u2::IsKindOf $elem PresentationElement] {
			set target [u2::GetEntity $elem ModelElement]
		} else {
			set target $elem
		}
		if ![IsElementRef $target] {
			return 0
		}
		
		if ![IsJavaPackage [GetPackage $target]] {
			return 0
		}
		return [expr [IsEclipseProject $project] && [IsLocatable $target]]
	}
	return 0
}


proc OnActivateEclipse {cmdName} {
	set project [std::GetActiveProject]
	if {"" == $project} {
		return
	}
	set name [file tail [file root [Get $project pathname]]]
	Output "\nExporting active project '$name' to Eclipse ... "
	if [ExportProjectToEclipse $project] {
		Output "\nDone."
	} else {
		Output "\nFailed to export project."
	}

}

##############################################################################
#	Java related procedures
#

proc OnEnableActiveProject {e} {
  set activeProject [std::GetActiveProject]
  if {"" != $activeProject} {
	  return 1
  }
  return 0
}    

proc OnImportDirectory {command} {
  output "\n----------------------------------------\n"
  set dir [DirectoryDialog]
  if {$dir == 0} {
     return
  }

  StoreAutoCheckOptions
    ##################### Invokation of agent ###############
    set curProject [std::GetActiveProject]
    set model [std::GetModels -kind "U2" -project $curProject]
    set agent [u2::FindByGuid $model "@JavaAgent@ImportDirectory"]
        
    if { 0 == $agent } {
        std::Output "Failed to find Import Java agent \n"
        # failed to find agent
        return 0
    }
    set param [lappend param $dir]

    if { [catch {
      u2::InvokeAgent $model $agent $model param
     } e ] } {
        std::Output "Error when invoking Import Java agent : $e\n"
    }
  RestoreAutoCheckOptions  
  output "\nDone\n"
}


proc OnExportPackage {command} {
  output "\n----------------------------------------\n"

  set elem [std::GetSelection]
  if [u2::IsKindOf $elem PackageSymbol] {
    set target [u2::GetEntity $elem ModelElement]
  } else {
    set target $elem
  }

  # Preselect src folder of Eclipse project if found, otherwise use current directory
  set project [std::GetProject $target]
  set srcLocations [GetSourceLocations [GetSession $target]]
  if { 0 < [llength $srcLocations] } {
  	set srcDir [lindex $srcLocations 0]
  } else {
  	set srcDir ""
  }
  set dir [DirectoryDialog -fromdir $srcDir]
  if {($dir == 0) || ![file exists $dir]} {
     return
  }

  output "Exporting package [u2::GetValue $target Name] to directory: $dir ... \n"

  set curProject [std::GetActiveProject]
  set model [std::GetModels -kind "U2" -project $curProject]
  set agent [u2::FindByGuid $model @TTDJavaModelCodeSync@Menu@ExportToDirectory]
        
  if { 0 == $agent } {
      std::Output "Failed to find Import Java agent \n"
      # failed to find agent
      return 0
  }
  set param [list $dir]

  if { [catch {
    u2::InvokeAgent $model $agent $elem param
   } e ] } {
     std::Output "Error when invoking Export Java agent : $e\n"
  }
        
  # Refresh project in Eclipse
  if {("" != $project) && [IsEclipseProject $project]} {
  	if [CanConnectToEclipse] {
	  	set projectName [file tail [file root [Get $project pathname]]]
		RefreshResource $projectName
	}
  }


  output "Done\n"
}


###################  Update Java from Model #####################

proc OnUpdateJava {command} {
  output "\n----------------------------------------\n"
  Output "\n\nUpdating files from model ...\n"
  set elements [std::GetSelection]
  foreach elem $elements {
	set target $elem
  	if [u2::IsKindOf $elem PresentationElement] {
    		set target [u2::GetEntity $elem ModelElement]
  	}
  	set curProject [std::GetActiveProject]
        set model [std::GetModels -kind "U2" -project $curProject]
        set agent [u2::FindByGuid $model "@JavaAgent@UpdateJava"]
        
        if { 0 == $agent } {
                std::Output "Failed to find Update Java agent \n"
                # failed to find agent
                return 0
        }
        set param [lappend param [GetProjectDir $curProject]]

        if { [catch {
          u2::InvokeAgent $model $agent $elem param
         } e ] } {
                std::Output "Error when invoking Update Java agent : $e\n"
        }
  }
  # Refresh project in Eclipse
  if [CanConnectToEclipse] {
	set project [std::GetProject [lindex $elements 0]]
	if {"" != $project} {
		set projectPath [Get $project pathname]
		set projectName [file tail [file root $projectPath]]
		RefreshResource $projectName
	}
  }
  Output "\nDone\n"

}


# Update model from files
proc OnUpdateModel {command} {
  output "\n----------------------------------------\n"
  Output "\n\nUpdating model from files ...\n"
  StoreAutoCheckOptions
  set elements [std::GetSelection]
  
  set curProject [std::GetActiveProject]
  set model [std::GetModels -kind "U2" -project $curProject]
  set agent [u2::FindByGuid $model "@JavaAgent@UpdateModel"]
  if { 0 == $agent } {
      std::Output "Failed to find Update Java agent \n"
      # failed to find agent
      return 0
  }
  set param [lappend param [GetProjectDir $curProject]]
        
  foreach elem $elements {
	set target $elem
  	if [u2::IsKindOf $elem PresentationElement] {
    		set target [u2::GetEntity $elem ModelElement]
  	}
  	 ########### Invoke Agent ################
        if { [catch {
          u2::InvokeAgent $model $agent $elem param
         } e ] } {
                std::Output "Error when invoking Update Java agent : $e\n"
        }
  }
  RestoreAutoCheckOptions
  Output "\nDone\n"
}

proc OnUpdateModelProject {command} {
  set activeProject [std::GetActiveProject]
  if {$activeProject != ""} {
    output "\n\nUpdating active project from java source files\n"
    StoreAutoCheckOptions
    ##################### Invokation of agent ###############
        set curProject [std::GetActiveProject]
        set model [std::GetModels -kind "U2" -project $curProject]
        set agent [u2::FindByGuid $model @TTDJavaModelCodeSync@Menu@UpdateAll]
        
        if { 0 == $agent } {
                std::Output "Failed to find Update Java agent \n"
                # failed to find agent
                return 0
        }
        set param {1}

        if { [catch {
          u2::InvokeAgent $model $agent $model param
         } e ] } {
                std::Output "Error when invoking Update Java agent : $e\n"
        }
        
    RestoreAutoCheckOptions
    output "\nDone."
  }
}

proc OnUpdateModelProjectForce {command} {
  set activeProject [std::GetActiveProject]
  if {$activeProject != ""} {
    output "\n\nUpdating active project from java source files\n"
    StoreAutoCheckOptions
    ##################### Invokation of agent ###############
        set curProject [std::GetActiveProject]
        set model [std::GetModels -kind "U2" -project $curProject]
        set agent [u2::FindByGuid $model @TTDJavaModelCodeSync@Menu@UpdateAll]
        
        if { 0 == $agent } {
                std::Output "Failed to find Update Java agent \n"
                # failed to find agent
                return 0
        }
        
        set param {0}

        if { [catch {
          u2::InvokeAgent $model $agent $model param
         } e ] } {
                std::Output "Error when invoking Update Java agent : $e\n"
        }
    RestoreAutoCheckOptions
    output "\nDone."
  }
}

proc OnUpdateJavaProject {command} {
  set activeProject [std::GetActiveProject]
  if {$activeProject != ""} {
    output "\n\nUpdating java source files of active project...\n"
    set curProject [std::GetActiveProject]
        set model [std::GetModels -kind "U2" -project $curProject]
        set agent [u2::FindByGuid $model @TTDJavaModelCodeSync@Menu@GenerateAll]
        
        if { 0 == $agent } {
                std::Output "Failed to find Update Java agent \n"
                # failed to find agent
                return 0
        }
        set param {1}

        if { [catch {
          u2::InvokeAgent $model $agent $model param
         } e ] } {
                std::Output "Error when invoking Update Java agent : $e\n"
        }
        
    output "\nDone."
    # Refresh project in Eclipse
    if [CanConnectToEclipse] {
      set projectPath [Get $activeProject pathname]
      set projectName [file tail [file root $projectPath]]
      RefreshResource $projectName
    }
  }
}

proc OnUpdateJavaProjectForce {command} {
  set activeProject [std::GetActiveProject]
  if {$activeProject != ""} {
    output "\n\nUpdating java source files of active project...\n"
    set curProject [std::GetActiveProject]
        set model [std::GetModels -kind "U2" -project $curProject]
        set agent [u2::FindByGuid $model @TTDJavaModelCodeSync@Menu@GenerateAll]
        
        if { 0 == $agent } {
                std::Output "Failed to find Update Java agent \n"
                # failed to find agent
                return 0
        }
        
        set param {0}

        if { [catch {
          u2::InvokeAgent $model $agent $model param
         } e ] } {
                std::Output "Error when invoking Update Java agent : $e\n"
        }

    output "\nDone."
    # Refresh project in Eclipse
    if [CanConnectToEclipse] {
      set projectPath [Get $activeProject pathname]
      set projectName [file tail [file root $projectPath]]
      RefreshResource $projectName
    }
  }
}


###################  Import JAR file #####################

proc OnUpdateModelFromJarFile {command} {
  output "\n----------------------------------------\n"
  set jarFile [FileOpenDialog -filter "JAR files (*.jar)|*.jar||"]
  if {$jarFile == 0} {
     return
  }

  output "\nUpdating model from .jar file: $jarFile ... "
  StoreAutoCheckOptions
  ######################## invoke agent ###################
  set curProject [std::GetActiveProject]
  set model [std::GetModels -kind "U2" -project $curProject]
  set agent [u2::FindByGuid $model "@JavaAgent@ImportJar"]
        
  if { 0 == $agent } {
        std::Output "Failed to find Import Java agent \n"
        # failed to find agent
        return 0
  }
  set param [lappend param $jarFile [GetProjectDir $curProject]]
  if { [catch {
    u2::InvokeAgent $model $agent $model param
   } e ] } {
        std::Output "Error when invoking Import Java agent : $e\n"
  }
  RestoreAutoCheckOptions
  output "Done\n"
}

###################  Use Java syntax #####################

proc OnChangeJavaSyntax {command} {
  set elements [std::GetSelection]
  if {[llength $elements] == 0} {
  	return 0
  }
  foreach elem $elements {
    if {[GetKind $elem] == "U2"} {
  	if [u2::IsKindOf $elem PackageSymbol] {
  		set target [u2::GetEntity $elem ModelElement]
  	} else {
	    	set target $elem
  	}
	if ![JavaSyntax $target] {
		return 0
	}
    } else {
    	return 0
    }
  }
  return 1
}

proc OnEnableJavaSyntax { command } {
  set elements [std::GetSelection]
  if {[llength $elements] == 0} {
  	return 0
  }
  foreach elem $elements {
	# PackageSymbol should be changed to PresentationElement when presentationLanguage overrides packageLanguage
	if {([GetKind $elem] != "U2") || (![u2::IsKindOf $elem Package] && ![u2::IsKindOf $elem PackageSymbol])} {
		return 0
	}
  }
  return 1
}

proc OnUseJavaSyntax {command} {
  set elements [std::GetSelection]
  foreach elem $elements {
	if [u2::IsKindOf $elem PackageSymbol] {
    		set target [u2::GetEntity $elem ModelElement]
  	} else {
	    	set target $elem
  	}
  	if [JavaSyntax $target] {
	  	JavaSyntax $target 0
		Output "Syntax for '[u2::GetValue $elem Name]' set to U2.\n"
  	} else {
	  	JavaSyntax $target 1
		Output "Syntax for '[u2::GetValue $elem Name]' set to Java.\n"
  	}
   }
}

###################  UI Initialization #####################

proc OnEnablePackageContext { command } {
  set elements [std::GetSelection]
   if {[llength $elements] == 0} {
   	return 0
   }
   foreach elem $elements {
     if {[GetKind $elem] == "U2"} {
	set target $elem
	if [u2::IsKindOf $elem PackageSymbol] {
		set target [u2::GetEntity $elem ModelElement]
	}
	if {![u2::IsKindOf $target Package] || ("" == [GetJavaDirectory $target]) || [InJar $target]} {
		return 0
	}
     } else {
	return 0
     }
   }
   return 1
}

proc OnPackageSelected {command} {
  set elem [std::GetSelection]
  if { [llength $elem] == 1 } {
  	if { [GetKind $elem] == "U2" } {
		if { [u2::IsKindOf $elem Package] || [u2::IsKindOf $elem PackageSymbol]} {
			set target $elem
			if [u2::IsKindOf $elem PackageSymbol] {
				set target [u2::GetEntity $elem ModelElement]
			}
  			return [expr ![IsJarPackage $target]]
		}
	}
  }
  return 0
}

proc OnEnableJavaUpdate {command} {
  set elements [std::GetSelection]
   if {[llength $elements] == 0} {
   	return 0
   }
   foreach elem $elements {
	if {[GetKind $elem] == "U2"} {
		set target $elem
		if [u2::IsKindOf $elem PresentationElement] {
			set target [u2::GetEntity $elem ModelElement]
		}
		set javaDir [GetJavaDirectory $target]
		if {("" == $javaDir) || [InJar $target]} {
			return 0
		}
	} else {
		return 0
	}
  }
  return 1
}

proc OnEnableContextMenu { command } {
  set elements [std::GetSelection]
   if {[llength $elements] == 0} {
   	return 0
   }
   foreach elem $elements {
	if {[GetKind $elem] == "U2"} {
		set target $elem
		if {[u2::IsKindOf $elem PresentationElement] && ![u2::IsKindOf $elem Diagram]} {
			set target [u2::GetEntity $elem ModelElement]
		}
		
		if {(0 == $target) || [u2::IsKindOf $target Package] || ![HasJavaFileMapping $target] || [InJar $target]} {
			return 0
		}
	} else {
		return 0
	}
   }
   return 1
}

proc OnEnableFeatureContext { command } {
  set elem [std::GetSelection]
   if {([llength $elem] != 1) || ([GetKind $elem] != "U2")} {
   	return 0
   }
   set target $elem
   if {[u2::IsKindOf $target PresentationElement] && ![u2::IsKindOf $target Diagram]} {
	set target [u2::GetEntity $elem ModelElement]
   }
   if {0 == $target} {
      return 0
   }
   if { [IsLocatable $target] } {
	  return 1
   }
}


###################################################################
proc Init {} {
	# Contextual update commands
	AddCommand -variable cmdEclipseUpdateJava -name "Update source code" -statusmessage "Update java source code for selected elements." -tooltip "Update source from model" -accelerator "" -imagefile "" -OnActivateCommand OnUpdateJava -OnEnableCommand OnEnableJavaUpdate
	AddCommand -variable cmdEclipseUpdateModel -name "Update model" -statusmessage "Update the selected element from java file." -tooltip "Update model from source" -accelerator "" -imagefile "" -OnActivateCommand OnUpdateModel -OnEnableCommand OnEnableJavaUpdate
	
	# Non contextual update commands
	AddCommand -variable cmdEclipseUpdateAllClasses -name "Update model" -statusmessage "Update the entire model from java source files." -tooltip "Update model" -accelerator "Ctrl+Alt+M" -imagefile "" -OnActivateCommand OnUpdateModelProject -OnEnableCommand OnEnableActiveProject
	AddCommand -variable cmdEclipseUpdateAllClassesForce -name "Force update model" -statusmessage "Update the entire model from java source files. Up-to-date entitied are also updated." -tooltip "Force update model" -accelerator "Ctrl+Alt+Shift+M" -imagefile "" -OnActivateCommand OnUpdateModelProjectForce -OnEnableCommand OnEnableActiveProject
	AddCommand -variable cmdEclipseUpdateAllFiles -name "Update source code" -statusmessage "Update all classes from changed java files." -tooltip "Update source code" -accelerator "Ctrl+Alt+S" -imagefile "" -OnActivateCommand OnUpdateJavaProject -OnEnableCommand OnEnableActiveProject
	AddCommand -variable cmdEclipseUpdateAllFilesForce -name "Force update source code" -statusmessage "Update all classes from changed java files. Up-to-date files are also updated." -tooltip "Force update source code" -accelerator "Ctrl+Alt+Shift+S" -imagefile "" -OnActivateCommand OnUpdateJavaProjectForce -OnEnableCommand OnEnableActiveProject
		
	# Import commands
	#AddCommand -variable cmdEclipseImportDir -name "Source code from directory..." -statusmessage "Import package from directory structure." -tooltip "Import package from directory structure" -imagefile "" -OnActivateCommand OnImportDirectory -OnEnableCommand OnEnableActiveProject
	#AddCommand -variable cmdEclipseUpdateJar -name "JAR file..." -statusmessage "Import contents of a JAR file into model." -tooltip "Import from .jar file" -imagefile "" -OnActivateCommand OnUpdateModelFromJarFile -OnEnableCommand OnEnableActiveProject
	
	# Export commands
	AddCommand -variable cmdEclipseExportPkg -name "Package..." -statusmessage "Export package to directory" -tooltip "Export package to directory" -imagefile "" -OnActivateCommand OnExportPackage -OnEnableCommand OnPackageSelected
	
	# Java syntax
	AddCommand -variable cmdEclipseUseJavaSyntax -name "Use Java syntax" -statusmessage "Toggles the syntax used in text diagrams and text symbols between Java and U2." -tooltip "Toggle syntax" -imagefile "" -OnActivateCommand OnUseJavaSyntax -OnEnableCommand OnEnableJavaSyntax -oncheckcommand OnChangeJavaSyntax
	 
	#################################
	# Eclipse specific commands
	AddCommand -variable cmdEclipseCreateProject -name "Create Eclipse Project" -statusmessage "Create an Eclipse project for the active project" -tooltip "Create Eclipse project" -imagefile "" -OnActivateCommand OnActivateEclipse -OnEnableCommand OnEnableActiveProject
	AddCommand -variable cmdEclipseSynchronizeEclipse -name "Synchronize with Eclipse" -statusmessage "Synchronize active project with Eclipse project" -tooltip "Synchronize with Eclipse project" -imagefile "" -OnActivateCommand OnSynchronizeEclipse -OnEnableCommand OnEnableSynchronizeEclipse
	AddCommand -variable cmdEclipseLocateElement -name "Locate in Eclipse" -statusmessage "Locate selected element in Eclipse" -tooltip "locate in Eclipse" -imagefile "" -OnActivateCommand OnLocateInEclipse -OnEnableCommand OnEnableLocateInEclipse
	AddCommand -variable cmdEclipseChangeExecutablePath -name "Change Eclipse directory" -statusmessage "Specify the path to the wanted Eclipse executable" -tooltip "Specify the path to the wanted Eclipse executable" -imagefile "" -OnActivateCommand OnChangeEclipseExecutable -OnEnableCommand OnEnableActiveProject

	#################################
	# Menu definitions

	# Add menus in reverse order since -after doesn't seem to work for nested menus
	AddMenu -variable mnuEclipse -commands { cmdEclipseChangeExecutablePath cmdEclipseCreateProject cmdEclipseLocateElement cmdEclipseSynchronizeEclipse separator cmdEclipseUpdateAllClasses cmdEclipseUpdateAllClassesForce cmdEclipseUpdateAllFiles cmdEclipseUpdateAllFilesForce cmdEclipseUseJavaSyntax} -path { Eclipse } -position {after &Tools}
	AddMenu -variable mnuEclipseJavaExport -commands { cmdEclipseExportPkg } -path { Eclipse Export } -position {after Eclipse Import}
	#AddMenu -variable mnuEclipseJavaImport -commands { cmdEclipseImportDir cmdEclipseUpdateJar } -path { Eclipse Import }

	AddContextMenu -variable cmnEclJavaElement -commands { separator cmdEclipseUpdateJava cmdEclipseUpdateModel cmdEclipseLocateElement} -onenablemenu { OnEnableContextMenu }
	AddContextMenu -variable cmnEclPackage -commands { separator cmdEclipseUpdateJava cmdEclipseUpdateModel cmdEclipseLocateElement cmdEclipseUseJavaSyntax } -onenablemenu { OnEnablePackageContext}
	AddContextMenu -variable cmnEclFeature -commands { separator cmdEclipseLocateElement } -onenablemenu { OnEnableFeatureContext}
	
}

Init

output "Done.\n"
