package com.telelogic.tau.tor;

public abstract class Dispatchable implements EventExecutor, EventReceiver
{
	public Dispatchable()
	{
		m_executorId = InstanceManager.instance().addExecutor(this);
		m_receiverId = InstanceManager.instance().addReceiver(this);		
	}
	
	@Override
	public void finalize() 
	{
		InstanceManager.instance().removeExecutor(this);
		InstanceManager.instance().removeReceiver(this);
	}
	
	// EventExecutor implementation
	public abstract EventAction execute(Event event);
	
	public long getExecutorId()
	{
		return m_executorId;
	}

	// EventReceiver implementation
	public abstract boolean receive(Event e);

	public long getReceiverId()
	{
		return m_receiverId;
	}
	
	// Adds d to the current dispatcher.
	public void addToCurrentDispatcher(Dispatchable d)
	{
		if (d.m_dispatcher != null)
			return; // Already added to a dispatcher! Don't add it to another one!
		
		if (m_dispatcher != null)
			m_dispatcher.add(d);
	}
	
	// Adds all dispatchables of the collection to the current dispatcher.
    public <T extends Dispatchable> void addToCurrentDispatcher(Iterable<T> collection)
    {
    	for (T t : collection)
		{
    		if (t != null)
    			addToCurrentDispatcher(t);
		}
    }
    
	//////////////////////////////////////////////////////////////////////////////////////
	
	// The event queue associated with this dispatchable. The event queue
    // is normally owned by the associated dispatcher.
	public EventQueue m_eventQueue;
	
	// The event currently being processed.
	public Event m_currentEvent;
	
	// The dispatcher associated with this dispatchable
	public Dispatcher m_dispatcher;
	
	private long m_executorId;
	private long m_receiverId;
}
