package com.telelogic.tau.tor;

import com.telelogic.tau.tor.os.*;

public class ThreadSafeEventQueue extends EventQueue
{
	public ThreadSafeEventQueue()
	{
		m_read.init();
	}
	
	@Override
    public void insert(Event event)
    {
    	synchronized (this)
		{
			super.insert(event);
			m_read.put(); // Notify that a new event has arrived.
		}
    }
    
	@Override
    public boolean remove(Event event)
    {
    	Time now = Time.now();

        Status status = m_read.get(now);
        if (status == Status.StatusTIMEOUT)
            return false; // No event to remove.
        
        if (status != Status.StatusOK)
            return false; // ERROR

        synchronized (this)
        {
	        assert(!m_list.isEmpty());

	        if (!super.remove(event))
	        {
	            m_read.put();
	            return false;
	        }
	        return true;
        }
    }

    // Get next event from the queue. Block until one is obtained.
    @Override
    public Event get()
    {
    	if (m_read.get() != Status.StatusOK )
    		return null; // Error
    	
    	synchronized (this)
		{
			Event event = super.get();
			return event;
		}
    }
    
    // Get next event from the queue. Block until one is obtained but not longer than until the specified time.
    @Override
    public Event get(Time timeout)
    {
        Status status = m_read.get(timeout);
        if (status == Status.StatusTIMEOUT)
            return null; // No event obtained until timeout occurred.
        
        if (status != Status.StatusOK)
            return null; // ERROR

        synchronized (this)
		{
			Event event = super.get();
			return event;
		}
    	
    }
    
    @Override
    public boolean empty()
    {
    	synchronized (this)
		{
			return super.empty();
		}
    }

    private Semaphore m_read = new Semaphore();
}
