# Generated for IBM Rational Tau 4.3.0.0.15182 at Wed Sep 30 04:38:17 MEST 2009
# Licensed Materials - Property of IBM 
# (c) Copyright IBM Corporation 2002,2009.  All Rights Reserved.
# 
# U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
package require commands
package require dialogs

# Return active project
proc GetActiveSession {} {
    set activeProject [std::GetActiveProject]
    if { $activeProject == 0 } {
        return 0
    }
    set projects [std::GetModels -kind U2 -project $activeProject]
    foreach p $projects {
        return $p
    }
    return 0
}

# Load library
proc LoadLib { profile msg } {
    set etcDir [GetInstallationDirectory]etc
    set profilePath "$etcDir/$profile"
    output "Loading the $msg library: $profilePath\n"
    set err [catch { u2::LoadLibrary $profilePath } log ]
}

# Unload library by its GUID
proc UnloadLib { lib } {
    set curProject [std::GetActiveProject]
    set model [std::GetModels -kind U2 -project $curProject]
    set package [u2::FindByGuid $model $lib]
    if { $package != 0 } {
        set err [catch { u2::UnloadLibrary $package } log ]
        if { $err } {
            output "$log"
        }
    }
}

# check if <<xsdFile>> stereotype is applied to the artifact
proc IsXSDFileArtifact { context } {
    if { [ u2::IsKindOf $context Artifact] != 0}  {
        if { [ u2::GetTaggedValue $context "xsdFile(. .)"] != 0 } {
            return 1
        }
    }
    return 0
}

# return <<xsdFile>> artifact that manifest the package
proc GetXSDFileArtifact { p } {

    if { [IsXSDFileArtifact $p ] != 0} {
        return $p
    }

    if { [u2::IsKindOf $p "Package" ] != 0 } {
        foreach dep [u2::GetReferringEntities $p Supplier] {
            set owner [u2::GetEntity $dep Client]
            if { [u2::GetTaggedValue $dep "manifest(. .)"] != 0 } {
                if { [u2::GetTaggedValue $owner "xsdFile(..)"] != 0 }  {
                    return $owner
                }
            }
        }
    }
    return 0
}

# invoke GenerateXSD2File agent
proc InvokeExportAgent { context fileName } {
    set session [ GetActiveSession ]
    if { $session != 0 } {
        set agent [u2::FindByGuid $session "@XSDAgents@GenerateXSD2File"]
        if { $agent != 0 } {
            std::Output "Generating file $fileName..."
            set params { }
            set params [lappend params $fileName]
            if { [catch { u2::InvokeAgent $session $agent $context params } e ] } {
                std::Output "FAILED!\n"
                std::Output "Error when invoking GenerateXSD2File agent : $e\n"
            }
            std::Output "DONE!\n"
        }
    }
}

# invoke ReImportXSD agent
proc InvokeReImportAgent { context } {
    set session [ GetActiveSession ]
    if { $session != 0 } {
        set agent [u2::FindByGuid $session "@XSDAgents@ReImportXSD"]
        if { $agent != 0 } {
            std::Output "Re-importing XSD..."
            set params [list $context]
            if { [catch { u2::InvokeAgent $session $agent $session params } e ] } {
                std::Output "FAILED!\n"
                std::Output "Error when invoking ReImportXSD agent : $e\n"
            }
            std::Output "DONE!\n"
        }
    }
}

# Import XSD is enabled only on artifacts
proc OnEnableImportXSD { cmd } {    
    set sel [std::GetSelection]
    if { [catch { set r [u2::IsKindOf $sel Entity] }] } {
        return 0
    }

    if { [u2::IsKindOf $sel Artifact] } {
        if { [IsXSDFileArtifact $sel ] != 0 } {
            return 1
        }
    }
    return 0
}

# Generate XSD is enabled on packages and artifacts
proc OnEnableGenerateXSD { cmd } {    
    set sel [std::GetSelection]
    if { [catch { set r [u2::IsKindOf $sel Entity] }] } {
        return 0
    }

    if { [u2::IsKindOf $sel Package] } {
        return 1
    }

    if { [u2::IsKindOf $sel Artifact] } {
        if { [IsXSDFileArtifact $sel ] != 0 } {
            return 1
        }
    }
    
    return 0
}

proc OnEnableXSD { cmd } {
    if { [OnEnableImportXSD cmd] } {
        return 1
    }
    if { [OnEnableGenerateXSD cmd] } {
        return 1
    }
    return 0
}

# Re-import XSD
proc OnImportXSD { command } {
    set sel [std::GetSelection]
    set artifact [GetXSDFileArtifact $sel]

    # there is existing <<XSDFile>> artifact
    if { $artifact != 0 } {
        set v [u2::GetTaggedValue $artifact "xsdFile(..)" ]
        if { ($v != 0) && ([u2::IsKindOf $v InstanceExpr] != 0)} {
            InvokeReImportAgent $artifact
        }
        return 0
    }
}

# Export package to XSD
proc OnExportXSD { command } {
    set sel [std::GetSelection]
    set artifact [GetXSDFileArtifact $sel]

    # there is existing <<xsdFile>> artifact
    if { $artifact != 0 } {
        set v [u2::GetTaggedValue $artifact "xsdFile(. path .)" ]
        if { ($v != 0) && ([u2::IsKindOf $v ValueExpr] != 0)} {
            set f [u2::GetValue $v ValueString]
            InvokeExportAgent $artifact $f
        }
        return 0
    }

    # no <<xsdFile>> artifact for the context
    # it will be create by the agent
    if { [u2::IsKindOf $sel Package ] } {
        set defaultName "[u2::GetValue $sel Name].xsd"
        set res [std::FileSaveDialog -filename $defaultName -fileext "xsd"]
        if { $res != 0 } {
            InvokeExportAgent $sel $res
        }
    }
}

proc CreateMenu { } {
    AddCommand -variable cmdImportXSD -name "Import XSD..." -statusmessage "Importing XSD specification." -tooltip "Import XSD..." -imagefile "" -OnActivateCommand OnImportXSD -OnEnableCommand OnEnableImportXSD
    AddCommand -variable cmdExportXSD -name "Generate XSD..." -statusmessage "Generating XSD specification from UML model." -tooltip "Generate XSD..." -imagefile "" -OnActivateCommand OnExportXSD -OnEnableCommand OnEnableGenerateXSD

    AddContextMenu -variable cmnXSD -commands {separator cmdImportXSD cmdExportXSD } -onenablemenu { OnEnableXSD }
}

proc ActivateMetaModel { } {
    set models [std::GetModels -kind U2]
    set metamodel 0
    set wsdlmodel 0
    foreach m $models {
        set metamodel [u2::FindByGuid $m "@TTDXSDView"]
        set wsdlmodel [u2::FindByGuid $m "@TTDWSDLView"]
    }
    if { ($metamodel != 0) && ($wsdlmodel == 0) } {
        u2::SelectMetaModel $metamodel
    }
}

proc Init {} {

    LoadLib "TTDXMLFramework.u2" "XML Framework"
    LoadLib "xsd.u2" "XSD"

    CreateMenu
    ActivateMetaModel
}


proc BeforeUnload { } {
#    UnloadLib "@xsd"
#    UnloadLib "@TTDXMLFramework"
}

Init
