# Generated for IBM Rational Tau 4.3.0.0.15182 at Wed Sep 30 04:38:28 MEST 2009
# Licensed Materials - Property of IBM 
# (c) Copyright IBM Corporation 2002,2009.  All Rights Reserved.
# 
# U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
############################
# ClearCaseIntegration.tcl #
############################

if {[info exists env(SCC_DEBUG)]} {
    proc Verbose {s} {
        puts "SCC_DEBUG: $s"
    }
    Verbose [info script]
} else {
    proc Verbose {s} {
    }
}

# Once the OpenProject function has been called
# the script can use the procedure 	StudioOutput { "output message" }
# to display a message in the Studio SCC output tab.

# Variables valid for one session, they are read only in Initialize
set providerName 	"providerName"
set IDEName 		"IdeName"
set capacities 		"40008430"
set nCheckoutCommentLen 100
set nCommentLen		100

# Global variables used by the UNIX_SCC DLL
set gUser		""
set gProjectName	""
set gLocalPath		""
set gAuxProjPath	""
set glistStatus		""
set gNumFiles		0
set gFileName		""
set gStatus		0
# used in each procedure that wants to display a message in the Studio output window.
# this variable has to be cleaned at the begining of each procedure
set gMessage		""

# Global variables used only in the script
set myCLEARCASE         ""
set ct                  "cleartool"
set cteff               "clearexport_ffile"
set cti                 "clearimport"
set gExePath		"xclearcase"
set gViewTag            ""
set mkdir               "/bin/mkdir"
set tmp                 "/tmp"
set mkelemHasmkpath	0

proc OptStudioOutput {s} {
    if { ![string equal "$s" "\n"] } {
        StudioOutput "$s"
    }
}

proc GetVersion {} {
    Verbose "[info level 1]"
    return 1.0
}

proc Initialize { CallerName } {
    Verbose "[info level 1]"
    global myCLEARCASE ct cteff cti gExePath mkelemHasmkpath
    global providerName IDEName capacities nCheckoutCommentLen nCommentLen

    # initialize a session of the Configuration Manager

    # initialize the global variable which describes the provider
    # NB : the capacities must be initialized following the scc.h values of SCC_CAP ...

    set providerName "ClearCase"
    set IDEName	     $CallerName

    set bExists [info exists env(CLEARCASE)]
    if { $bExists } {
        set myCLEARCASE $env(CLEARCASE)
        set ct          "$myCLEARCASE/cleartool"
        set cteff       "$myCLEARCASE/clearexport_ffile"
        set cti         "$myCLEARCASE/clearimport"
        set gExePath    "$myCLEARCASE/xclearcase"
    } 
      
    catch {exec $ct -ver} version
    set nFind [string first "no such" $version]
    if {$nFind != -1} {
        StudioOutput "Cannot find ClearCase executables. Set CLEARCASE environment variable first.\n"
        return -1
    }

    # does ct mkelem have -mkpath option (introduced in v6)?
    if [catch { exec $ct mkelem -h } addMes] {
        set mkelemHasmkpath 0
    } else {
        set mkelemHasmkpath [expr [string first "-mkpath" $addMes] > 0]
    }
    return 0
}

proc Uninitialize {} {
    Verbose "[info level 1]"
    # close the session
    return 0
}

proc GetProjPath { } {
    Verbose "[info level 1]"
    global gProjectName gAuxProjPath gViewTag gLocalPath ct cteff cti tmp mkdir gUser

    set gAuxProjPath $gLocalPath

    cd "$gAuxProjPath"

    # The $gAuxProjPath directory should be a ClearCase view
    catch { exec $ct pwv -s } gViewTag
    catch { exec $ct pwv -wdv } wdv
    set nFind [string first  $gViewTag $wdv]
    set bFind [expr 0 < $nFind]

    Verbose "gAuxProjPath=$gAuxProjPath"
    Verbose "gViewTag=$gViewTag"
    Verbose "bFind=$bFind"
    
    if { $bFind } {
         return 0
    }

    return -1
}

proc OpenProject { Flag } {
    Verbose "[info level 1]"
    global gAuxProjPath gLocalPath gProjectName ct gUser
    set gAuxProjPath $gLocalPath
    # open the Configuration Manager project

    # verify if $gProjectName located in $gAuxProjPath
    cd "$gAuxProjPath"
    Verbose "gProjectName=$gProjectName"
    set bFind 0
    catch { exec $ct ls } projectsList
    foreach project [split $projectsList '\n'] {
        if { [regexp -- "$gProjectName\.ttp" $project ] } {
            set bFind 1
            break
        }
    }
    Verbose "bFind=$bFind"
    if { $bFind } {
        return 0
    }
   
    # Project is not located in ClearCase view
    return -1
}

proc CloseProject {} {
    Verbose "[info level 1]"
    return 0
}

proc Get { listFilesNames nFiles } {
    Verbose "[info level 1]"
    return 0
}

proc Checkout { listFilesNames comment } {
    Verbose "[info level 1]"
    global ct

    foreach fichier $listFilesNames {
        # call the CHECK OUT function of the provider
        catch {exec $ct co -res -c "$comment" $fichier} coMessage
        OptStudioOutput "$coMessage\n"
    }

    return 0
}

proc Uncheckout { listFilesNames } {
    Verbose "[info level 1]"
    global ct

    foreach fichier $listFilesNames {
        # call the UNDO CHECK OUT function of the provider
        catch {exec $ct unco -keep $fichier} uncoMessage
        OptStudioOutput "$uncoMessage\n"
    }

    return 0
}

proc Checkin { listFilesNames comment } {
    Verbose "[info level 1]"
    global ct

    foreach fichier $listFilesNames {
        # call the CHECK IN function of the provider
        catch {exec $ct ci -c "$comment" $fichier} ciMessage
        OptStudioOutput "$ciMessage\n"
    }

    return 0
}

proc Add { listFilesNames boolCO comment } {
    Verbose "[info level 1]"
    global gAuxProjPath mkdir tmp ct glistStatus mkelemHasmkpath
    set retVal 0
    if $mkelemHasmkpath {
        if { ![expr $boolCO] } {
            set execStat [catch { eval exec $ct mkelem -mkpath -ci -nc [join $listFilesNames] } addMessage]
        } else {
            set execStat [catch { eval exec $ct mkelem -mkpath -nc [join $listFilesNames] } addMessage]
        }
        OptStudioOutput "$addMessage\n"
        if { $execStat} {
            set retVal -1
        }
    } else {
        foreach name $listFilesNames {
            set basename [file tail $name]
            set dirname [file dirname $name]
            QueryInfo "$dirname"
            set status [lindex $glistStatus 0]
            
            if { [expr 1 > $status] } {
                if { [Add  $dirname $boolCO "$comment"] } {
                    set retVal -1
                    continue
                }
                QueryInfo "$dirname"
                set status [lindex $glistStatus 0]
            }
            cd $dirname
            if { ![expr $status & 0x2] } {
                set execStat [catch { exec $ct co -res -nc . } addMessage]
                OptStudioOutput "$addMessage\n"
                if { $execStat} {
                    set retVal -1
                    continue
                }
            }
            set execStat [catch {file isdirectory $basename} isdir]
            if { $execStat} {
                OptStudioOutput "$isdir\n"
                set retVal -1
                continue
            }
            if {$isdir} {
                set execStat [catch { file rename $basename .$basename} addMessage]
                OptStudioOutput "$addMessage\n"
                if { $execStat} {
                    set retVal -1
                    continue
                }
                set execStat [catch { exec $ct mkdir -nc $basename } addMessage]
                OptStudioOutput "$addMessage\n"
                if { $execStat} {
                    set retVal -1
                    continue
                }
                set execStat [catch { glob .$basename/*} addMessage]
                if { $execStat} {
                    OptStudioOutput "$addMessage\n"
                    set retVal -1
                    continue
                }
                foreach addName $addMessage {
                    set execStat [catch {file rename $addName $basename} addMessage]
                    OptStudioOutput "$addMessage\n"
                    if { $execStat} {
                        set retVal -1
                        continue
                    }
                }
                set execStat [catch { file delete .$basename} addMessage]
                OptStudioOutput "$addMessage\n"
                if { $execStat} {
                    set retVal -1
                    continue
                }
            } else {
                if { ![expr $boolCO] } {
                    set execStat [catch { exec $ct mkelem -ci -nc $basename } addMessage]
                } else {
                    set execStat [catch { exec $ct mkelem -nc $basename } addMessage]
                }
                OptStudioOutput "$addMessage\n"
                if { $execStat} {
                    set retVal -1
                    continue
                }
            }
            if { ![expr $boolCO] } {
                if { ![expr $status & 0x2] } {
                    set execStat [catch { exec $ct ci -nc . } addMessage]
                    OptStudioOutput "$addMessage\n"
                    if { $execStat} {
                        set retVal -1
                        continue
                    }
                }
            }
        }
    }
    return $retVal
}

proc tempname {base} {
    set filename $base
    set count 1
    while {[file exists $filename]} {
        set filename "${base}-$count"
        incr count
    }
    set filename
}

proc Remove { listFilesNames comment } {
    Verbose "[info level 1]"
    global ct gAuxProjPath gLocalPath glistStatus
    set gAuxProjPath $gLocalPath

    cd $gAuxProjPath
    foreach name $listFilesNames {
        set dirname [file dirname $name]
        QueryInfo "$dirname"
        set status [lindex $glistStatus 0]
        if { [expr 1 > $status] } {
            OptStudioOutput "The directory $dirname should be added to ClearCase first.\n"
        } else {
            cd $dirname
            Verbose "cd $dirname"
            if { ![expr $status & 0x2] } {
                catch { exec $ct co -res -nc . } addMessage
                OptStudioOutput "$addMessage\n"
                Verbose $addMessage
            }

            # call the REMOVE function of the provider
            set cpfilename [tempname "$::tmp/Remove"]

            set cpret [catch {file copy -force $name $cpfilename} cpMessage]
            OptStudioOutput "$cpMessage\n"
            Verbose "copy -force $name $cpfilename"
            set ret [catch {exec $ct rmname -c "$comment" -force $name} ctMessage]
            OptStudioOutput "$ctMessage\n"
            Verbose $ctMessage
            set cpret [catch {file copy -force $cpfilename $name } cpMessage]
            OptStudioOutput "$cpMessage\n"
            Verbose "copy -force $cpfilename $name"
            set rmret [catch {file delete -force $cpfilename } rmMessage]
            OptStudioOutput "$rmMessage\n"
            Verbose "delete -force $cpfilename "
            if { ![expr $status & 0x2] } {
                catch { exec $ct ci -nc . } addMessage
                OptStudioOutput "$addMessage\n"
                Verbose $addMessage
            }
        }
    }
    return 0
}




proc ShowDifferences { FileName } {
    Verbose "[info level 1]"
    global ct

    # display the differences of the file with the last version
    set ret [catch {exec cleartool diff -ser -pre $FileName} ctMessage]
    if {$ret} {
        regsub "child process exited abnormally$" $ctMessage "" ctMessage
    }
    OptStudioOutput "$ctMessage\n"

    return 0
}

proc ShowHistory { FileName } {
    Verbose "[info level 1]"
    global ct

    # display the history of the file
    catch {exec $ct lshistory $FileName} ctMessage
    OptStudioOutput "$ctMessage\n"

    return 0
}

proc Properties { FileName } {
    Verbose "[info level 1]"
    global ct

    # display a window or a message box showing the state of the file
    # or use StudioOutput to display in Studio SCC output tab
    catch { exec $ct describe $FileName } ctMessage
    StudioOutput "$ctMessage\n"

    return 0
}

proc QueryInfo { listFilesNames } {
    Verbose "[info level 1]"
    global glistStatus ct

#    INVALID          = -1L,		// Status could not be obtained, don't rely on it
#    NOTCONTROLLED    = 0x0000L,	// File is not under source control
#    CONTROLLED       = 0x0001L,	// File is under source code control
#    CHECKEDOUT       = 0x0002L,	// Checked out to current user at local path
#    OUTOTHER         = 0x0004L,	// File is checked out to another user
#    OUTEXCLUSIVE     = 0x0008L,	// File is exclusively check out
#    OUTMULTIPLE      = 0x0010L,	// File is checked out to multiple people
#    OUTOFDATE        = 0x0020L,	// The file is not the most recent
#    DELETED          = 0x0040L,	// File has been deleted from the project
#    LOCKED           = 0x0080L,	// No more versions allowed
#    MERGED           = 0x0100L,	// File has been merged but not yet fixed/verified
#    SHARED           = 0x0200L,	// File is shared between projects
#    PINNED           = 0x0400L,	// File is shared to an explicit version
#    MODIFIED         = 0x0800L,	// File has been modified/broken/violated
#    OUTBYUSER	      = 0x1000L,	// File is checked out by current user someplace

    set glistStatus ""
    foreach fichier $listFilesNames {
        set nStatus 0

        if {[catch { exec $ct describe $fichier } dsStatus]} {
            OptStudioOutput "$dsStatus\n"
        }
        set nFind [string first "version" $dsStatus]
        set bFind [expr 0 <= $nFind]

        if { $bFind } {
            # CONTROLLED
            set nStatus 1
            set nFind [string first "CHECKEDOUT" $dsStatus]
            set bFind [expr 0 <= $nFind]
            if { $bFind } {
                # CONTROLLED and CHECKEDOUT and OUTBYUSER
                set nStatus 4099
            }
        } else {
            # NOTCONTROLLED
            set nStatus 0
        }

        # add the calculated status to the list
        lappend glistStatus $nStatus
    }
    lappend glistStatus FIN

    return 0
}

proc Run { } {
    Verbose "[info level 1]"
    global gExePath
    eval exec "/bin/sh -c $gExePath >&/dev/tty &"
    return 0
}

proc AddFromScc {} {
    Verbose "[info level 1]"
    # return SCC_E_OPNOTSUPPORTED
    return -14
}

proc GetEvents {} {
    Verbose "[info level 1]"
    # return SCC_E_OPNOTSUPPORTED
    return -14
}

proc Rename { OldName NewName } {
    Verbose "[info level 1]"
    # not used in Studio
    # return SCC_E_OPNOTSUPPORTED
    return -14
}

proc PopulateList {} {
    Verbose "[info level 1]"
    # not used in Studio
    # return SCC_E_OPNOTSUPPORTED
    return -14
}

proc GetCommandOptions {} {
    Verbose "[info level 1]"
    # return SCC_E_OPNOTSUPPORTED
    return -14
}

proc SetOption {} {
    Verbose "[info level 1]"
    # return SCC_E_OPNOTSUPPORTED
    return -14
}

#####################################

proc randomInit {seed} {
    Verbose "[info level 1]"
    global rand
    set rand(ia) 9301;
    set rand(ic) [clock clicks];
    set rand(im) 233280;
    set rand(seed) $seed;
}

proc random {} {
    Verbose "[info level 1]"
    global rand
    set rand(seed)  [expr ($rand(seed)*$rand(ia) + $rand(ic)) % $rand(im)]
    return [expr $rand(seed)/double($rand(im))]
}

proc GetRandomName {} {
    Verbose "[info level 1]"
    randomInit [pid]
    set val [random]
    return _tmp$val
}
