<!-- ***************************************** -->
<!-- XSLT transformation for converting from   -->
<!-- U2 XML storage format 2.1 to 2.2          -->
<!-- ***************************************** -->

<xsl:transform version="1.1"
   xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<!-- Change file format version number to 2.2 -->

<xsl:template match="u2">
  <xsl:copy>
    <xsl:for-each select="@*">
      <xsl:choose>
        <xsl:when test="local-name()='OMVersion'">
          <xsl:attribute name="OMVersion">2.2</xsl:attribute>
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<!-- remove Class::Interface -->
<xsl:template match = "Class/rRealized">
  <cGeneralization>
    <Generalization>
	  <rParent>
		<xsl:attribute name = "R"><xsl:value-of select="@R"/></xsl:attribute>
	  </rParent>
	</Generalization>
  </cGeneralization>
</xsl:template>

<!-- transform MessageSend::isLost and MessageReceive::isFound into Message::isLostOrFound -->
<xsl:template match = "MessageSend | InstanceCreateSend | OperationCallSend | OperationReplySend">
  <xsl:copy>
    <xsl:for-each select="@*">
      <xsl:choose>
        <xsl:when test="local-name()='IsLost'">
		<!-- do not copy this attribute -->
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<xsl:template match = "MessageReceive | InstanceCreateReceive | OperationCallReceive | OperationReplyReceive">
  <xsl:copy>
    <xsl:for-each select="@*">
      <xsl:choose>
        <xsl:when test="local-name()='IsFound'">
		<!-- do not copy this attribute -->
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<xsl:template match = "Message">
  <xsl:copy>
    <xsl:variable name = "uid" select = "concat('uid:',./@Guid)"/>
	<!-- Check for MessageSend -->
    <xsl:if test = "count(//MessageSend/rMessage[@R=$uid]/../@IsLost) &gt; 0">
	  <xsl:attribute name = "IsLostOrFound"><xsl:value-of select = "//MessageSend/rMessage[@R=$uid]/../@IsLost"/></xsl:attribute>
	</xsl:if>
	<!-- Check for InstanceCreateSend -->
    <xsl:if test = "count(//InstanceCreateSend/rMessage[@R=$uid]/../@IsLost) &gt; 0">
	  <xsl:attribute name = "IsLostOrFound"><xsl:value-of select = "//InstanceCreateSend/rMessage[@R=$uid]/../@IsLost"/></xsl:attribute>
	</xsl:if>
	<!-- Check for InstanceCreateSend -->
    <xsl:if test = "count(//OperationCallSend/rMessage[@R=$uid]/../@IsLost) &gt; 0">
	  <xsl:attribute name = "IsLostOrFound"><xsl:value-of select = "//OperationCallSend/rMessage[@R=$uid]/../@IsLost"/></xsl:attribute>
	</xsl:if>
	<!-- Check for OperationReplySend -->
    <xsl:if test = "count(//OperationReplySend/rMessage[@R=$uid]/../@IsLost) &gt; 0">
	  <xsl:attribute name = "IsLostOrFound"><xsl:value-of select = "//OperationReplySend/rMessage[@R=$uid]/../@IsLost"/></xsl:attribute>
	</xsl:if>
	<!-- Check for MessageReceive -->
    <xsl:if test = "count(//MessageReceive/rMessage[@R=$uid]/../@IsFound) &gt; 0">
	  <xsl:attribute name = "IsLostOrFound"><xsl:value-of select = "//MessageReceive/rMessage[@R=$uid]/../@IsFound"/></xsl:attribute>
	</xsl:if>
	<!-- Check for InstanceCreateReceive -->
    <xsl:if test = "count(//InstanceCreateReceive/rMessage[@R=$uid]/../@IsFound) &gt; 0">
	  <xsl:attribute name = "IsLostOrFound"><xsl:value-of select = "//InstanceCreateReceive/rMessage[@R=$uid]/../@IsFound"/></xsl:attribute>
	</xsl:if>
	<!-- Check for OperationCallReceive -->
    <xsl:if test = "count(//OperationCallReceive/rMessage[@R=$uid]/../@IsFound) &gt; 0">
	  <xsl:attribute name = "IsLostOrFound"><xsl:value-of select = "//OperationCallReceive/rMessage[@R=$uid]/../@IsFound"/></xsl:attribute>
	</xsl:if>
	<!-- Check for OperationReplyReceive -->
    <xsl:if test = "count(//OperationReplyReceive/rMessage[@R=$uid]/../@IsFound) &gt; 0">
	  <xsl:attribute name = "IsLostOrFound"><xsl:value-of select = "//OperationReplyReceive/rMessage[@R=$uid]/../@IsFound"/></xsl:attribute>
	</xsl:if>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<!-- remove composition SubjectSymbol::UseCaseSymbol -->
<xsl:template match = "SubjectSymbol/cUseCaseSymbol">
</xsl:template>

<!-- remove Gate::FormalGate -->
<xsl:template match = "Gate/rFormalGate">
</xsl:template>

<!-- remove CombinedFragment::rParticipant - it is derived now -->
<xsl:template match = "CombinedFragment/rParticipant">
</xsl:template>

<!-- remove InteractionOccurrence::rParticipant - it is derived now -->
<xsl:template match = "InteractionOccurrence/rParticipant">
</xsl:template>

<!-- remove Action::txOnException -->
<xsl:template match = "CompoundAction | DecisionAction | DefAction | DeleteAction | ExceptionAction |
					   ExpressionAction | IfAction | LoopAction | OutputAction | BreakAction |
					   ContinueAction | JoinAction | NextStateAction | RaiseAction | ReturnAction |
					   StopAction | TimerSetAction | TimerResetAction">
  <xsl:copy>
    <xsl:for-each select="@*">
	  <xsl:choose>
		<xsl:when test="local-name()='txOnException'">
        <!-- do not copy this attribute -->
		</xsl:when>
		<xsl:otherwise>
		  <xsl:copy/>
		</xsl:otherwise>
	  </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<!-- remove ExceptionHandler::Exception -->
<xsl:template match = "ExceptionHandler/lException">
</xsl:template>

<!-- remove Exception::ExceptionHandler -->
<xsl:template match = "Exception/lExceptionHandler">
</xsl:template>

<!-- rename Port::isBehaviour to isBehavior -->
<xsl:template match = "Port">
  <xsl:copy>
    <xsl:for-each select="@*">
	  <xsl:choose>
		<xsl:when test="local-name()='IsBehaviourPort'">
		  <xsl:attribute name = "IsBehaviorPort"><xsl:value-of select = "../@IsBehaviourPort"/></xsl:attribute>
		</xsl:when>
		<xsl:otherwise>
		  <xsl:copy/>
		</xsl:otherwise>
	  </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<!-- cut GUID annotations from texts in presentation model -->
<xsl:template match = "AssociationLine | CommentSymbol | RemarkSybol |
					   CommunicatorLabel | CompoundActionOccurrenceSymbol | 
					   ConnectorEndLabel | ConnectorLine | 
					   DecisionAnswerSymbol | DecisionSymbol |
					   DependencyLine | ExtensionLine | GeneralizationLine |
					   GuardSymbol | InputSymbol | InteractionOccurrenceSymbol |
					   InteractionSymbol | JunctionSymbol | LifeLineSymbol |
					   MessageLine | InstanceCreateLine | MethodCallLine |
					   MethodReplyLine | MultiplicityLabel | 
					   NextStateActionOccurrenceSymbol | OutputSymbol |
					   PackageUseSymbol | RealizedInterfaceSymbol |
					   ActorSymbol | AttributeLabel | LiteralLabel | OperationLabel |
					   ParameterLabel | InstanceSymbol | PackageSymbol |
					   PartSymbol | PortSymbol | DataTypeSymbol | 
					   ClassSymbol | InterfaceSymbol | StereotypeSymbol |
					   ExceptionSymbol | OperationSymbol | StateMachineSymbol |
					   SignalSymbol | TimerSymbol | SubjectSymbol | UseCaseSymbol |
					   RequiredInterfaceSymbol | ReturnSymbol | SaveSymbol |
					   StartSymbol | StateCompartmentLabel | StateSymbol |
					   TaskSymbol | TextDiagram | TextSymbol | TimerResetSymbol |
					   TimerSetSymbol | TimerTimeoutSymbol | TransitionLine">
  <xsl:copy>
    <xsl:for-each select="@*">
      <xsl:choose>
        <xsl:when test="local-name()='Text'">
		  <xsl:call-template name = "CutOneGUID">
		    <xsl:with-param name = "str"><xsl:value-of select = "."/></xsl:with-param>
			<xsl:with-param name = "pos">1</xsl:with-param>
		  </xsl:call-template>
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<xsl:template name = "CutOneGUID">
  <xsl:param name = "str"/>
  <xsl:param name = "pos"/>
  <xsl:variable name = "curstr" select = 'substring($str,$pos)'/>
  <xsl:choose>
    <xsl:when test = 'contains($curstr, "#")'>
	  <xsl:choose>
	    <!-- We have # symbol (doubled to avoid clash with GUID) -->
	    <xsl:when test = 'starts-with(substring-after($curstr, "#"), "#")'>
		  <xsl:variable name = "reclen" select = '$pos + string-length(substring-before($curstr, "#"))'/>
		  <xsl:call-template name = "CutOneGUID">
		    <xsl:with-param name = "str">
			  <xsl:value-of select = 'concat(substring($str,1,$reclen - 1), substring-after(substring($str,$reclen), "#"))'/>
			</xsl:with-param>
			<xsl:with-param name = "pos">
			  <xsl:value-of select = "$reclen + 1"/>
			</xsl:with-param>
		  </xsl:call-template>
		</xsl:when>
		<!-- GUID case -->
		<xsl:otherwise>
		  <xsl:variable name = "tagname" select = 'substring-before(substring-after(substring-after($curstr, "#"), "#"), "#")'/>
		  <xsl:variable name = "tail" select = 'substring-after(substring-after(substring-after($curstr, "#"), "#"), "#")'/>
		  <xsl:call-template name = "CutOneGUID">
		    <xsl:with-param name = "str">
			  <xsl:value-of select = 'concat(concat(concat(substring($str,1,$pos - 1), substring-before($curstr, "#")),$tagname), $tail)'/>
			</xsl:with-param>
			<xsl:with-param name = "pos">
			  <xsl:value-of select = "$pos + string-length($tagname) + string-length(substring-before($curstr, '#')) - 1"/>
			</xsl:with-param>
		  </xsl:call-template>
		</xsl:otherwise>
	  </xsl:choose>
	</xsl:when>
	<!-- All GUIDs are cut -->
	<xsl:otherwise>
	  <xsl:attribute name = "Text"><xsl:value-of select = "$str"/></xsl:attribute>
	</xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!-- Default rule -->
<xsl:template match="*">
  <xsl:copy>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<xsl:template name="copy-all-attributes">
  <xsl:for-each select="@*">
    <xsl:copy/>
  </xsl:for-each>
</xsl:template>

<!-- template which renames attributes of link to attributes of reference -->
<xsl:template name = "link-to-reference">
  <xsl:for-each select="@*">
	<xsl:choose>
	  <xsl:when test="local-name()='L'">
	    <xsl:attribute name="R"><xsl:value-of select="../@L"/></xsl:attribute>
	  </xsl:when>
	  <xsl:otherwise>
	    <xsl:copy/>
	  </xsl:otherwise>
    </xsl:choose>
  </xsl:for-each>
  <xsl:apply-templates/>
</xsl:template>

</xsl:transform>