<!-- ***************************************** -->
<!-- XSLT transformation for converting from   -->
<!-- U2 XML storage format 2.6 to 2.7          -->
<!-- ***************************************** -->

<xsl:transform version="1.1"
   xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<!-- Change file format version number to 2.6 -->
<xsl:template match="u2">
  <xsl:copy>
    <xsl:for-each select="@*">
      <xsl:choose>
        <xsl:when test="local-name()='OMVersion'">
          <xsl:attribute name="OMVersion">2.7</xsl:attribute>
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<!-- remove MultiState::TxOnException-->
<xsl:template match = "MultiState | SimpleStateMachine">
  <xsl:copy>
    <xsl:for-each select="@*">
      <xsl:choose>
        <xsl:when test="local-name()='TxOnException'">
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<!-- remove Expression::instanceCreate -->
<xsl:template match = "BinaryExpr/rInstanceCreate | CallExpr/rInstanceCreate | 
					   ConditionalExpr/rInstanceCreate | CreateExpr/rInstanceCreate |
					   EmptyExpr/rInstanceCreate | FieldExpr/rInstanceCreate | Ident/rInstanceCreate | 
					   GlobalScopeIdent/rInstanceCreate | ParameterizedIdent/rInstanceCreate |
					   TemplateInstantiation/rInstanceCreate | AnyExpr/rInstanceCreate | 
					   IsPresentExpr/rInstanceCreate | NowExpr/rInstanceCreate | 
					   OffspringExpr/rInstanceCreate | ParentExpr/rInstanceCreate | 
					   SelfExpr/rInstanceCreate | SenderExpr/rInstanceCreate | StateExpr/rInstanceCreate |
					   ThisExpr/rInstanceCreate | TimerActiveExpr/rInstanceCreate |
					   IndexExpr/rInstanceCreate | InformalExpr/rInstanceCreate | 
					   InstanceExpr/rInstanceCreate | ListExpr/rInstanceCreate | 
					   ParenthesisExpr/rInstanceCreate | RangeCheckExpr/rInstanceCreate | 
					   UnaryExpr/rInstanceCreate | CharacterValue/rInstanceCreate | 
					   IntegerValue/rInstanceCreate | RealValue/rInstanceCreate |
					   BitstringValue/rInstanceCreate | CharstringValue/rInstanceCreate | 
					   HexstringValue/rInstanceCreate">
</xsl:template>

<!-- remove Connector::delayed -->
<xsl:template match = "Connector">
  <xsl:copy>
    <xsl:for-each select="@*">
      <xsl:choose>
        <xsl:when test="local-name()='Delayed'">
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<!-- remove Generalization::discriminator -->
<xsl:template match = "Generalization">
  <xsl:copy>
    <xsl:for-each select="@*">
      <xsl:choose>
        <xsl:when test="local-name()='Discriminator'">
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<!-- Move Method::class and Interaction::structuredClassifier to more general Implementation::implementedSignature -->
<xsl:template match = "Interaction/rClass | OperationBody/rClass | SimpleStateMachine/rClass |
					   Interaction/rStructuredClassifier">
  <rImplementedSignature>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </rImplementedSignature>
</xsl:template>

<!-- Remove EventClass::declaration -->
<xsl:template match = "Operation/rDeclaration | StateMachine/rDeclaration | Signal/rDeclaration |
					   Timer/rDeclaration">
</xsl:template>

<!-- OutputItem/OutputAction changes -->
<xsl:template match = "OutputAction/rVia | OutputAction/cTo">
</xsl:template>

<xsl:template match = "OutputItem">
  <CallExpr>
    <xsl:for-each select="@*">
      <xsl:choose>
        <xsl:when test="local-name()='Guid'">
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
	<xsl:for-each select = "../..//rVia">
	  <rVia>
        <xsl:call-template name="copy-all-attributes"/>
        <xsl:apply-templates/>
	  </rVia>
	</xsl:for-each>
	<xsl:for-each select = "../..//cTo">
	  <cTo>
        <xsl:call-template name="copy-all-attributes"/>
        <xsl:apply-templates/>
	  </cTo>
	</xsl:for-each>
  </CallExpr>
</xsl:template>

<xsl:template match = "OutputItem/rSignal | CallExpr/rOperation">
  <rCalled>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </rCalled>
</xsl:template>

<!-- renamed the reference Extension::metaClass to 'extended' -->
<xsl:template match = "Extension/rMetaClass">
  <rExtended>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </rExtended>
</xsl:template>

<!-- CreateExpr now inherits Typed... -->
<xsl:template match = "CreateExpr/cExpression">
</xsl:template>

<xsl:template match = "CreateExpr">
  <xsl:copy>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
	<xsl:for-each select = "cExpression/Ident | cExpression/ParameterizedIdent |
							cExpression/TemplateInstantiation">
	  <rType>
		<xsl:attribute name = "R">ref:<xsl:value-of select = "@Name"/></xsl:attribute>
	  </rType>
	</xsl:for-each>
	<xsl:for-each select = "cExpression/ThisExpr">
	  <rType>
		<xsl:attribute name = "R">ref:this</xsl:attribute>
	  </rType>
	</xsl:for-each>
  </xsl:copy>
</xsl:template>

<!-- AnyExpr now inherits Typed...-->
<xsl:template match = "AnyExpr/rSort">
  <rType>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </rType>
</xsl:template>

<!-- Remove StructuralFeature::ContainerType - replaced by informal multiplicity -->
<xsl:template match = "Attribute/rContainerType | Parameter/rContainerType">
</xsl:template>

<!-- Move all definitions into one feature -->
<xsl:template match = "Package/cSubPackage | Package/cSignalList | Package/cAttribute |
					   Internals/cAssociation | Interaction/cAssociation | OperationBody/cAssociation | 
					   SimpleStateMachine/cAssociation | Package/cAssociation | Artifact/cAssociation | 
					   Choice/cAssociation | Class/cAssociation | Collaboration/cAssociation | 
					   Interface/cAssociation | Stereotype/cAssociation |
					   Internals/cInformalDefinition | Interaction/cInformalDefinition | 
					   OperationBody/cInformalDefinition | SimpleStateMachine/cInformalDefinition |
					   Package/cInformalDefinition | Artifact/cInformalDefinition | Choice/cInformalDefinition |
					   Class/cInformalDefinition | Collaboration/cInformalDefinition | 
					   Interface/cInformalDefinition | Stereotype/cInformalDefinition |
					   Internals/cSignature | Interaction/cSignature | OperationBody/cSignature | 
					   SimpleStateMachine/cSignature | Package/cSignature | Artifact/cSignature | 
					   Choice/cSignature | Class/cSignature | Collaboration/cSignature | 
					   Interface/cSignature | Stereotype/cSignature |
					   Artifact/cSignalList | Choice/cSignalList | Class/cSignalList | 
					   Collaboration/cSignalList | Interface/cSignalList | Stereotype/cSignalList |
					   Artifact/cAttribute | Choice/cAttribute | Class/cAttribute | 
					   Collaboration/cAttribute | Interface/cAttribute | Stereotype/cAttribute |
					   Class/cPort | Class/cConnector | Collaboration/cConnector | DataType/cOperation |
					   StateMachine/cConnectionPoint | Internals/cSignalList | Interaction/cSignalList |
					   OperationBody/cSignalList | SimpleStateMachine/cSignalList | Internals/cVariable | 
					   Interaction/cVariable | OperationBody/cVariable | SimpleStateMachine/cVariable |
					   Internals/cPort | Internals/cConnector">
  <cOwnedMember>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </cOwnedMember>
</xsl:template>

<!-- Removed Incrementable -->
<xsl:template match = "Package/rIncremented">
</xsl:template>

<!-- Remove attribute 'threadingModel' and its values from InstanceExprs... -->
<xsl:template match = "InstanceExpr/cExpression/AssignmentExpr">
  <xsl:if test = "count(cLeftOperand/Ident[@Name='threadingModel']) &lt; 1">
    <BinaryExpr>
      <xsl:call-template name="copy-all-attributes"/>
      <xsl:apply-templates/>
	  <rOperation>
	    <xsl:attribute name = "R">ref:'\='</xsl:attribute>
	  </rOperation>
    </BinaryExpr>
  </xsl:if>
</xsl:template>

<!-- remove references to threadingModel attribute, supplied directly (not via assignment syntax) -->
<xsl:template match = "InstanceExpr/cExpression/Ident">
  <xsl:if test = "@Name != singleThreaded and @Name != multiThreaded">
    <xsl:copy>
      <xsl:call-template name="copy-all-attributes"/>
      <xsl:apply-templates/>
	</xsl:copy>
  </xsl:if>
</xsl:template>

<!-- Removed AssignmentExpr - use BinaryExpr instead -->
<xsl:template match = "AssignmentExpr">
  <BinaryExpr>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
	<rOperation>
	  <xsl:attribute name = "R">ref:'\='</xsl:attribute>
	</rOperation>
  </BinaryExpr>
</xsl:template>

<!-- GeneralizationLine is no longer a TextContainer -->
<xsl:template match = "GeneralizationLine">
  <xsl:copy>
    <xsl:for-each select="@*">
      <xsl:choose>
        <xsl:when test="local-name()='Text'">
        </xsl:when>
        <xsl:when test="local-name()='IsParseError'">
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<!-- attempt to convert 2.0 bad transition model to 2.1 format -->
<xsl:template match = "DecisionAnswerTransition/cAction/CompoundAction/cAction/CompoundAction | 
					   LabelTransition/cAction/CompoundAction/cAction/CompoundAction |
					   StartTransition/cAction/CompoundAction/cAction/CompoundAction | 
					   TriggeredTransition/cAction/CompoundAction/cAction/CompoundAction">
	<!-- Compute reference for which we are going to scan TaskSymbols -->
  <xsl:variable name = "RefFromTask" select = "concat('uid:',@Guid)"/>
  <xsl:choose>
    <!-- This is CompoundAction, which corresponds to TaskSymbol -->
    <xsl:when test = "(count(cAction/*) != 1 or count(cAction/CompoundAction) = 0) and 
						(count(//TaskSymbol/rCompoundAction[@R=$RefFromTask]) &gt; 0)">
      <xsl:copy>
	    <xsl:for-each select="cLabel">
		  <xsl:copy>
            <xsl:apply-templates/>
		  </xsl:copy>
		</xsl:for-each>
        <cAction>
          <CompoundAction>
			<!-- Enumerated all possible children of CompoundAction because of strange errors issued when using for-each & choose -->
		    <xsl:for-each select="cHiddenStereotypeInstance | cInformalConstraint | 
					cStereotypeInstance | cComment | cAction">
		      <xsl:copy>
			    <xsl:apply-templates/>
			  </xsl:copy>
			</xsl:for-each>
          </CompoundAction>
	    </cAction>
	    <xsl:call-template name="copy-all-attributes"/>
	  </xsl:copy>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:copy>
        <xsl:apply-templates/>
        <xsl:call-template name="copy-all-attributes"/>
	  </xsl:copy>
	</xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!-- remove old (from TT/D 1.0) references to other files -->
<xsl:template match = "A">
</xsl:template>

<!-- Default rule -->
<xsl:template match="*">
  <xsl:copy>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<xsl:template name="copy-all-attributes">
  <xsl:for-each select="@*">
    <xsl:copy/>
  </xsl:for-each>
</xsl:template>

</xsl:transform>