<!-- ***************************************** -->
<!-- XSLT transformation for converting from   -->
<!-- U2 XML storage format 2.7 (3) to 2.8 (4)  -->
<!-- ***************************************** -->

<xsl:transform version="1.1"
   xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
   xmlns:exsl="http://exslt.org/common"
   xmlns:u2="http://www.telelogic.com/tau"
                extension-element-prefixes="exsl">

<!-- Change file format version number to 2.8 -->
<xsl:template match="u2 | u2:u2">
  <xsl:copy>
    <xsl:for-each select="@*">
      <xsl:choose>
        <xsl:when test="local-name()='OMVersion'">
          <xsl:attribute name="OMVersion">2.8</xsl:attribute>
        </xsl:when>
        <xsl:when test="local-name()='StorageVersion'">
          <xsl:attribute name="StorageVersion">4</xsl:attribute>
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<!-- Informal Entity model changes-->
<xsl:template match = "InformalDefinition | InformalExpr | InformalConstraint | ContinuationFragment | 
						u2:InformalDefinition | u2:InformalExpr | u2:InformalConstraint | u2:ContinuationFragment">
  <xsl:copy>
    <xsl:for-each select="@*">
      <xsl:choose>
        <xsl:when test="local-name()='InformalText'">
		  <cInformalEntityFragment>
		    <InformalEntityFragment>
              <xsl:attribute name = "TextFragment">
			    <xsl:value-of select = "."/>
			  </xsl:attribute>
			</InformalEntityFragment>
		  </cInformalEntityFragment>
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<!-- Text labels no longer inherit from Symbol/ReferenceSymbol -->
<xsl:template match = "CommunicatorLabel | ConnectorEndLabel | AttributeLabel | LiteralLabel | 
						OperationLabel | ParameterLabel | MultiplicityLabel | StateCompartmentLabel |
						u2:CommunicatorLabel | u2:ConnectorEndLabel | u2:AttributeLabel | u2:LiteralLabel | 
						u2:OperationLabel | u2:ParameterLabel | u2:MultiplicityLabel | u2:StateCompartmentLabel">
  <xsl:copy>
    <xsl:for-each select="@*">
      <xsl:choose>
        <xsl:when test="local-name()='Autosize'">
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<xsl:template match = "CommunicatorLabel/rCommentLine | ConnectorEndLabel/rCommentLine |
						AttributeLabel/rCommentLine | LiteralLabel/rCommentLine | 
						OperationLabel/rCommentLine | ParameterLabel/rCommentLine |
						MultiplicityLabel/rCommentLine | StateCompartmentLabel/rCommentLine |
						u2:CommunicatorLabel/u2:rCommentLine | u2:ConnectorEndLabel/u2:rCommentLine |
						u2:AttributeLabel/u2:rCommentLine | u2:LiteralLabel/u2:rCommentLine | 
						u2:OperationLabel/u2:rCommentLine | u2:ParameterLabel/u2:rCommentLine |
						u2:MultiplicityLabel/u2:rCommentLine | u2:StateCompartmentLabel/u2:rCommentLine">
</xsl:template>

<!-- Comment symbol no longer inherits Flippable symbol -->
<xsl:template match = "CommentSymbol | u2:CommentSymbol">
  <xsl:copy>
    <xsl:for-each select="@*">
      <xsl:choose>
        <xsl:when test="local-name()='Flipped'">
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<!-- Changes in application builder stereotypes -->
<xsl:template match = "Artifact/cStereotypeInstance/InstanceExpr | u2:Artifact/u2:cStereotypeInstance/u2:InstanceExpr">
  <xsl:choose>
	<!-- this is application builder stereotype, lets try to find details of build type... -->
    <xsl:when test="(count(rInstanceOf[@R='uid:@TTDApplicationBuilder@general']) &gt; 0) or 
		(count(u2:rInstanceOf[@R='uid:@TTDApplicationBuilder@general']) &gt; 0)">
	  <xsl:copy>
	    <xsl:choose>
		  <!-- C Code Generator -->
	      <xsl:when test="(count(cExpression/BinaryExpr/cRightOperand/CharstringValue[@ValueString='C Code Generator']) &gt; 0) or 
				(count(u2:cExpression/u2:BinaryExpr/u2:cRightOperand/u2:CharstringValue[@ValueString='C Code Generator']) &gt; 0)">
		    <xsl:for-each select = "../InstanceExpr[rInstanceOf[@R='uid:@TTDCCodeGenerator@c']][1] | 
					../u2:InstanceExpr[u2:rInstanceOf[@R='uid:@TTDCCodeGenerator@c']][1]">
		      <xsl:call-template name="copy-all-attributes"/>
		      <xsl:apply-templates/>
		    </xsl:for-each>
			<!-- check, if there were actual instances of stereotype with build details - if none, add one... -->
		    <xsl:if test="(count(../InstanceExpr[rInstanceOf[@R='uid:@TTDCCodeGenerator@c']]) &lt; 1) and
				(count(../u2:InstanceExpr[u2:rInstanceOf[@R='uid:@TTDCCodeGenerator@c']]) &lt; 1)">
			  <rInstanceOf>
			    <xsl:attribute name = "R">uid:@TTDCCodeGenerator@c</xsl:attribute>
			  </rInstanceOf>
			</xsl:if>
		  </xsl:when>
		  <!-- C++ Code Generator -->
	      <xsl:when test="(count(cExpression/BinaryExpr/cRightOperand/CharstringValue[@ValueString='C++ Code Generator']) &gt; 0) or 
				(count(u2:cExpression/u2:BinaryExpr/u2:cRightOperand/u2:CharstringValue[@ValueString='C++ Code Generator']) &gt; 0)">
		    <xsl:for-each select = "../InstanceExpr[rInstanceOf[@R='uid:@TTDApplicationBuilder@cpp']][1] |
					../u2:InstanceExpr[u2:rInstanceOf[@R='uid:@TTDApplicationBuilder@cpp']][1]">
		      <xsl:call-template name="copy-all-attributes"/>
		      <xsl:apply-templates/>
		    </xsl:for-each>
			<!-- check, if there were actual instances of stereotype with build details - if none, add one... -->
		    <xsl:if test="(count(../InstanceExpr[rInstanceOf[@R='uid:@TTDApplicationBuilder@cpp']]) &lt; 1) and
				(count(../u2:InstanceExpr[u2:rInstanceOf[@R='uid:@TTDApplicationBuilder@cpp']]) &lt; 1)">
			  <rInstanceOf>
			    <xsl:attribute name = "R">uid:@TTDApplicationBuilder@cpp</xsl:attribute>
			  </rInstanceOf>
			</xsl:if>
		  </xsl:when>
		  <!-- Agile C Code Generator -->
	      <xsl:when test="(count(cExpression/BinaryExpr/cRightOperand/CharstringValue[@ValueString='AgileC Code Generator']) &gt; 0) or
				(count(u2:cExpression/u2:BinaryExpr/u2:cRightOperand/u2:CharstringValue[@ValueString='AgileC Code Generator']) &gt; 0)">
		    <xsl:for-each select = "../InstanceExpr[rInstanceOf[@R='uid:@TTDAgileCCodeGenerator@AgileC']][1] | 
				../u2:InstanceExpr[u2:rInstanceOf[@R='uid:@TTDAgileCCodeGenerator@AgileC']][1]">
		      <xsl:call-template name="copy-all-attributes"/>
		      <xsl:apply-templates/>
		    </xsl:for-each>
			<!-- check, if there were actual instances of stereotype with build details - if none, add one... -->
		    <xsl:if test="(count(../InstanceExpr[rInstanceOf[@R='uid:@TTDAgileCCodeGenerator@AgileC']]) &lt; 1) and
				(count(../u2:InstanceExpr[u2:rInstanceOf[@R='uid:@TTDAgileCCodeGenerator@AgileC']]) &lt; 1)">
			  <rInstanceOf>
			    <xsl:attribute name = "R">uid:@TTDAgileCCodeGenerator@AgileC</xsl:attribute>
			  </rInstanceOf>
			</xsl:if>
		  </xsl:when>
		  <!-- Model Verifier -->
	      <xsl:when test="(count(cExpression/BinaryExpr/cRightOperand/CharstringValue[@ValueString='Model Verifier']) &gt; 0) or
				(count(u2:cExpression/u2:BinaryExpr/u2:cRightOperand/u2:CharstringValue[@ValueString='Model Verifier']) &gt; 0)">
		    <xsl:for-each select = "../InstanceExpr[rInstanceOf[@R='uid:@TTDModelVerifier@modelVerifier']][1] |
				../u2:InstanceExpr[u2:rInstanceOf[@R='uid:@TTDModelVerifier@modelVerifier']][1]">
		      <xsl:call-template name="copy-all-attributes"/>
		      <xsl:apply-templates/>
		    </xsl:for-each>
			<!-- check, if there were actual instances of stereotype with build details - if none, add one... -->
		    <xsl:if test="(count(../InstanceExpr[rInstanceOf[@R='uid:@TTDModelVerifier@modelVerifier']]) &lt; 1) and
				(count(../u2:InstanceExpr[u2:rInstanceOf[@R='uid:@TTDModelVerifier@modelVerifier']]) &lt; 1)">
			  <rInstanceOf>
			    <xsl:attribute name = "R">uid:@TTDModelVerifier@modelVerifier</xsl:attribute>
			  </rInstanceOf>
			</xsl:if>
		  </xsl:when>
            <!-- Assuming Model Verifier, if not found anything else -->
		  <xsl:otherwise>
		    <rInstanceOf>
			  <xsl:attribute name = "R">uid:@TTDModelVerifier@modelVerifier</xsl:attribute>
			</rInstanceOf>
		  </xsl:otherwise>
	    </xsl:choose>
        <!-- copy targetDirectory and verboseMode attributes from general build stereotype -->
		<cExpression>
          <xsl:for-each select="cExpression/* | u2:cExpression/*">
            <xsl:choose>
              <xsl:when test="(count(cLeftOperand/Ident[@Name='buildType']) &gt; 0) or
				(count(u2:cLeftOperand/u2:Ident[@Name='buildType']) &gt; 0)">
              </xsl:when>
              <xsl:otherwise>
		        <xsl:copy>
                  <xsl:call-template name="copy-all-attributes"/>
                  <xsl:apply-templates/>
                </xsl:copy>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:for-each>
		</cExpression>
	  </xsl:copy>
	</xsl:when>
		<!-- Instances of the streotypes handled from generic case, so, should not be copied -->
    <xsl:when test="((count(rInstanceOf[@R='uid:@TTDCCodeGenerator@c']) &gt; 0) and 
			(count(../InstanceExpr/rInstanceOf[@R='uid:@TTDApplicationBuilder@general']) &gt; 0)) or
			((count(u2:rInstanceOf[@R='uid:@TTDCCodeGenerator@c']) &gt; 0) and 
			(count(../u2:InstanceExpr/u2:rInstanceOf[@R='uid:@TTDApplicationBuilder@general']) &gt; 0))">
	</xsl:when>
    <xsl:when test="((count(rInstanceOf[@R='uid:@TTDApplicationBuilder@cpp']) &gt; 0) and
			(count(../InstanceExpr/rInstanceOf[@R='uid:@TTDApplicationBuilder@general']) &gt; 0)) or
			((count(u2:rInstanceOf[@R='uid:@TTDApplicationBuilder@cpp']) &gt; 0) and
			(count(../u2:InstanceExpr/u2:rInstanceOf[@R='uid:@TTDApplicationBuilder@general']) &gt; 0))">
	</xsl:when>
    <xsl:when test="((count(rInstanceOf[@R='uid:@TTDModelVerifier@modelVerifier']) &gt; 0) and
			(count(../InstanceExpr/rInstanceOf[@R='uid:@TTDApplicationBuilder@general']) &gt; 0)) or
			((count(u2:rInstanceOf[@R='uid:@TTDModelVerifier@modelVerifier']) &gt; 0) and
			(count(../u2:InstanceExpr/u2:rInstanceOf[@R='uid:@TTDApplicationBuilder@general']) &gt; 0))">
	</xsl:when>
    <xsl:when test="((count(rInstanceOf[@R='uid:@TTDAgileCCodeGenerator@AgileC']) &gt; 0) and
			(count(../InstanceExpr/rInstanceOf[@R='uid:@TTDApplicationBuilder@general']) &gt; 0)) or
			((count(u2:rInstanceOf[@R='uid:@TTDAgileCCodeGenerator@AgileC']) &gt; 0) and
			(count(../u2:InstanceExpr/u2:rInstanceOf[@R='uid:@TTDApplicationBuilder@general']) &gt; 0))">
	</xsl:when>
    <xsl:otherwise>
      <xsl:copy>
        <xsl:call-template name="copy-all-attributes"/>
        <xsl:apply-templates/>
	  </xsl:copy>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!-- Remove compileAndLink attribute from 'C Code Generator' and 'Agile C Code Generator' stereotypes -->
<xsl:template match="Artifact/cStereotypeInstance/InstanceExpr/cExpression/BinaryExpr | 
			u2:Artifact/u2:cStereotypeInstance/u2:InstanceExpr/u2:cExpression/u2:BinaryExpr">
  <xsl:choose>
	<xsl:when test="((count(cLeftOperand/Ident[@Name='compileAndLink']) &gt; 0) or 
					 (count(u2:cLeftOperand/u2:Ident[@Name='compileAndLink']) &gt; 0)) and
					((count(./../../rInstanceOf[@R='uid:@TTDCCodeGenerator@c']) &gt; 0) or
					 (count(./../../u2:rInstanceOf[@R='uid:@TTDCCodeGenerator@c']) &gt; 0) or 
					 (count(./../../rInstanceOf[@R='uid:@TTDAgileCCodeGenerator@AgileC']) &gt; 0) or
					 (count(./../../u2:rInstanceOf[@R='uid:@TTDAgileCCodeGenerator@AgileC']) &gt; 0))">
	</xsl:when>
	<xsl:otherwise>
	  <xsl:copy>
        <xsl:call-template name="copy-all-attributes"/>
        <xsl:apply-templates/>
	  </xsl:copy>
	</xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!-- Default rule -->
<xsl:template match="*">
  <xsl:copy>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<xsl:template name="copy-all-attributes">
  <xsl:for-each select="@*">
    <xsl:copy/>
  </xsl:for-each>
</xsl:template>

</xsl:transform>