<!-- ***************************************** -->
<!-- XSLT transformation for converting from   -->
<!-- U2 XML storage format 3.4 (4) to 3.5 (4)  -->
<!-- ***************************************** -->

<xsl:transform version="1.1"
   xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
   xmlns:u2="http://www.telelogic.com/tau"
   exclude-result-prefixes='u2'>

<!-- Change file format version number to 3.5 -->
<xsl:template match="u2:u2|u2">
  <xsl:copy>
    <xsl:for-each select="@*">
      <xsl:choose>
        <xsl:when test="local-name()='OMVersion'">
          <xsl:attribute name="OMVersion">3.5</xsl:attribute>
        </xsl:when>
        <xsl:when test="local-name()='StorageVersion'">
          <xsl:attribute name="StorageVersion">4</xsl:attribute>
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>


<!-- Default rule -->
<xsl:template match="*">
  <xsl:copy>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<xsl:template name="copy-all-attributes">
  <xsl:for-each select="@*">
    <xsl:copy/>
  </xsl:for-each>
</xsl:template>

<!-- template for u2 Ident -->
<xsl:template name="Ident">
  <xsl:param name="name"/>
  <xsl:param name="guid"/>
  <u2:Ident>
    <xsl:attribute name="Name"><xsl:value-of select="$name"/></xsl:attribute>
    <u2:rDefinition>
      <xsl:attribute name="R">uid:<xsl:value-of select="$guid"/></xsl:attribute>
    </u2:rDefinition>
  </u2:Ident>
</xsl:template>

<!-- template for cppImportSpecification with non-empty cExpression -->
<xsl:template match="u2:InstanceExpr[u2:rInstanceOf/u2:Ident/@Name = 'cppImportSpecification' 
              and u2:rInstanceOf/u2:Ident/u2:rDefinition/@R = 'uid:@TTDCppImport@cppImportSpecification'
	      or u2:rInstanceOf/@R = 'uid:@TTDCppImport@cppImportSpecification']">
  <xsl:copy>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates mode="iac"/>
    <xsl:if test="count(u2:cExpression) = 0">
      <xsl:call-template name="iac_cExpression"/>
    </xsl:if>
  </xsl:copy>
</xsl:template>

<!-- template for cExpression with BinaryExpr `importActionCode = ...'-->
<xsl:template
  match="u2:cExpression[    u2:BinaryExpr/u2:cLeftOperand/u2:Ident/@Name = 'importActionCode'
                        and u2:BinaryExpr/u2:cLeftOperand/u2:Ident/u2:rDefinition/@R = 'uid:lUydXIS1zgALHFn93LipDKcI']"
  mode="iac">
  <xsl:copy>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates mode="iac"/>
  </xsl:copy>
</xsl:template>

<!-- template for cExpression without BinaryExpr `importActionCode = ...'-->
<xsl:template match="u2:cExpression" mode="iac">
  <xsl:copy>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
    <xsl:call-template name="iac_BinaryExpr"/>
  </xsl:copy>
</xsl:template>

<!-- template for BinaryExpr `importActionCode = ...'-->
<xsl:template 
  match="u2:BinaryExpr[    u2:cLeftOperand/u2:Ident/@Name = 'importActionCode'
                       and u2:cLeftOperand/u2:Ident/u2:rDefinition/@R = 'uid:lUydXIS1zgALHFn93LipDKcI']"
  mode="iac">
  <xsl:copy>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:call-template name="iac_opRef" select="rOperation"/>
    <xsl:apply-templates select="u2:cRightOperand" mode="iac"/>
    <xsl:apply-templates select="u2:cLeftOperand" mode="iac"/>
  </xsl:copy>
</xsl:template>

<xsl:template name="iac_opRef">
  <u2:rOperation>
    <xsl:call-template name="Ident">
      <xsl:with-param name="name">=</xsl:with-param>
      <xsl:with-param name="guid">@Predefined@Integer@=@Integer@Integer@Integer</xsl:with-param>
    </xsl:call-template>
  </u2:rOperation>
</xsl:template>

<xsl:template match="u2:cRightOperand" mode="iac">
  <xsl:copy>
    <xsl:choose>
      <xsl:when test="u2:Ident/@Name = 'true'">
        <xsl:call-template name="iac_ImportAC"/>
      </xsl:when>
      <xsl:when test="u2:Ident/@Name = 'false'">
        <xsl:call-template name="iac_DoNotImportAc"/>
      </xsl:when>
      <xsl:when test="u2:Ident/@Name = 'ImportAC'">
        <xsl:call-template name="iac_ImportAC"/>
      </xsl:when>
      <xsl:when test="u2:Ident/@Name = 'DoNotImportAc'">
        <xsl:call-template name="iac_DoNotImportAc"/>
      </xsl:when>
      <xsl:when test="u2:Ident/@Name = 'ImportACAsInformal'">
        <xsl:call-template name="iac_ImportACAsInformal"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="iac_DoNotImportAc"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:copy>
</xsl:template>

<xsl:template name="iac_ImportAC">
  <xsl:call-template name="Ident">
    <xsl:with-param name="name">ImportAC</xsl:with-param>
    <xsl:with-param name="guid">Ce9-6V3hUoCLMKjkEV402*9I</xsl:with-param>
  </xsl:call-template>
</xsl:template>

<xsl:template name="iac_ImportACAsInformal">
  <xsl:call-template name="Ident">
    <xsl:with-param name="name">ImportACAsInformal</xsl:with-param>
    <xsl:with-param name="guid">DzgrmLiQrvSLf7Db0IyLe0II</xsl:with-param>
  </xsl:call-template>
</xsl:template>

<xsl:template name="iac_DoNotImportAc">
  <xsl:call-template name="Ident">
    <xsl:with-param name="name">DoNotImportAc</xsl:with-param>
    <xsl:with-param name="guid">hm7IiE*vk*vL3DSWKLEBgAUE</xsl:with-param>
  </xsl:call-template>
</xsl:template>

<xsl:template name="iac_leftOp">
  <xsl:call-template name="Ident">
    <xsl:with-param name="name">importActionCode</xsl:with-param>
    <xsl:with-param name="guid">lUydXIS1zgALHFn93LipDKcI</xsl:with-param>
  </xsl:call-template>
</xsl:template>

<xsl:template name="iac_cExpression">
  <u2:cExpression>
    <xsl:call-template name="iac_BinaryExpr"/>
  </u2:cExpression>
</xsl:template>

<xsl:template name="iac_BinaryExpr">
  <u2:BinaryExpr>
    <xsl:call-template name="iac_opRef"/>
    <u2:cRightOperand>
      <xsl:call-template name="iac_DoNotImportAc"/>
    </u2:cRightOperand>
    <u2:cLeftOperand>
      <xsl:call-template name="iac_leftOp"/>
    </u2:cLeftOperand>
  </u2:BinaryExpr>
</xsl:template>

<xsl:template match="*" mode="iac">
  <xsl:copy>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

</xsl:transform>
