<!-- ***************************************** -->
<!-- XSLT transformation for converting from   -->
<!-- U2 XML storage format 3.5 (4) to 3.6 (4)  -->
<!-- ***************************************** -->

<xsl:transform version="1.1"
   xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
   xmlns:u2="http://www.telelogic.com/tau"
   xmlns="http://www.telelogic.com/tau"
   exclude-result-prefixes='u2'>

<!-- Change file format version number to 3.6 -->
<xsl:template match="u2:u2|u2">
  <xsl:copy>
    <xsl:for-each select="@*">
      <xsl:choose>
        <xsl:when test="local-name()='OMVersion'">
          <xsl:attribute name="OMVersion">3.6</xsl:attribute>
        </xsl:when>
        <xsl:when test="local-name()='StorageVersion'">
          <xsl:attribute name="StorageVersion">4</xsl:attribute>
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>


<!-- Default rule -->
<xsl:template match="*">
  <xsl:copy>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<xsl:template name="copy-all-attributes">
  <xsl:for-each select="@*">
    <xsl:copy/>
  </xsl:for-each>
</xsl:template>

<!-- Copying old compartment attributes -->
<xsl:template name="copy-compartment-attributes">
  <xsl:for-each select="@*">
    <!-- Skip size/position/autosize -->
    <xsl:if test="not(local-name() = 'Position' or local-name() = 'Size' or local-name() = 'Autosize')">
        <xsl:copy/>
    </xsl:if>
  </xsl:for-each>
</xsl:template>


<!-- ***************************************** -->
<!-- Applying compartmentLabelType stereotype  -->
<!-- ***************************************** -->
<xsl:template name="Apply_compartmentLabelType">
  <xsl:param name="Value"/>
  <xsl:element name="cStereotypeInstance">
  <xsl:element name="InstanceExpr">
    <xsl:element name="cExpression">
      <xsl:element name="BinaryExpr">
        <!-- Operation -->
        <xsl:element name="rOperation">
          <xsl:element name="Ident">
            <xsl:attribute name="Name">=</xsl:attribute>
            <xsl:element name="rDefinition">
              <xsl:attribute name="R">uid:@Predefined@Collection@=@Collection@Collection@Collection</xsl:attribute>
            </xsl:element>
          </xsl:element>
        </xsl:element>
        <!-- Right operation -->
        <xsl:element name="cRightOperand">
          <xsl:element name="CharstringValue">
            <xsl:attribute name="ValueString"><xsl:value-of select="$Value"/></xsl:attribute>
          </xsl:element>
        </xsl:element>
        <!-- Left operand -->
        <xsl:element name="cLeftOperand">
          <xsl:element name="Ident">
            <xsl:attribute name="Name">metaClass</xsl:attribute>
              <xsl:element name="rDefinition">
                <xsl:attribute name="R">uid:@Predefined@compartmentLabelType@metaClass</xsl:attribute>
              </xsl:element>
            </xsl:element>
          </xsl:element>
        </xsl:element>
      </xsl:element>
      <!-- InstanceOf-->
      <xsl:element name="rInstanceOf">
        <xsl:attribute name="R">uid:@Predefined@compartmentLabelType</xsl:attribute>
      </xsl:element>
    </xsl:element>
  </xsl:element>
</xsl:template>

<!-- *************************************** -->
<!-- Applying compartmentFilter stereotype   -->
<!-- *************************************** -->
<xsl:template name="Apply_compartmentFilter">
  <xsl:param name="Value"/>
  <xsl:element name="cStereotypeInstance">
    <xsl:element name="InstanceExpr">
      <xsl:element name="cExpression">
        <xsl:element name="BinaryExpr">
          <!-- Operation -->
          <xsl:element name="rOperation">
            <xsl:element name="Ident">
              <xsl:attribute name="Name">=</xsl:attribute>
              <xsl:element name="rDefinition">
                <xsl:attribute name="R">uid:@Predefined@Collection@=@Collection@Collection@Collection</xsl:attribute>
              </xsl:element>
            </xsl:element>
          </xsl:element>
          <!-- Right operand -->
          <xsl:element name="cRightOperand">
          <xsl:element name="ListExpr">
            <xsl:element name="cExpression">
              <xsl:element name="InstanceExpr">
                <xsl:element name="cExpression">
                  <xsl:element name="BinaryExpr">
                    <!-- Operation -->
                    <xsl:element name="rOperation">
                      <xsl:element name="Ident">
                        <xsl:attribute name="Name">=</xsl:attribute>
                        <xsl:element name="rDefinition">
                          <xsl:attribute name="R">uid:@Predefined@Collection@=@Collection@Collection@Collection</xsl:attribute>
                        </xsl:element>
                      </xsl:element>
                    </xsl:element>
                    <!-- Right operand -->
                    <xsl:element name="cRightOperand">
                      <xsl:element name="CharstringValue">
                        <xsl:attribute name="ValueString"><xsl:value-of select="$Value"/></xsl:attribute>
                      </xsl:element>
                    </xsl:element>
                    <!-- Left Operand -->
                    <xsl:element name="cLeftOperand">
                      <xsl:element name="Ident">
                        <xsl:attribute name="Name">metaClass</xsl:attribute>
                          <xsl:element name="rDefinition">
                            <xsl:attribute name="R">uid:@Predefined@compartmentFilter@MetaclassFilter@metaClass</xsl:attribute>
                          </xsl:element>
                        </xsl:element>
                      </xsl:element>
                    </xsl:element>
                  </xsl:element>
                  <!-- InstanceOf -->
                  <xsl:element name="rInstanceOf">
                    <xsl:attribute name="R">uid:@Predefined@compartmentFilter@MetaclassFilter</xsl:attribute>
                  </xsl:element>
                </xsl:element>
              </xsl:element>
            </xsl:element>
          </xsl:element>
          <!-- Left operand -->
          <xsl:element name="cLeftOperand">
            <xsl:element name="Ident">
              <xsl:attribute name="Name">filters</xsl:attribute>
              <xsl:element name="rDefinition">
                <xsl:attribute name="R">uid:@Predefined@compartmentFilter@filters</xsl:attribute>
              </xsl:element>
            </xsl:element>
          </xsl:element>
        </xsl:element>
      </xsl:element>
      <!-- Instance of -->
      <xsl:element name="rInstanceOf">
        <xsl:attribute name="R">uid:@Predefined@compartmentFilter</xsl:attribute>
      </xsl:element>
    </xsl:element>
  </xsl:element>
</xsl:template>

<!-- Converting structured classifier symbol -->
<xsl:template match="u2:ClassSymbol|u2:StereotypeSymbol|u2:InterfaceSymbol|u2:ArtifactSymbol|u2:CollaborationSymbol">
  <xsl:copy>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:element name="cCompartment">
    <!-- adding attribute label compartment -->
    <xsl:element name="Compartment">
      <xsl:call-template name="Apply_compartmentLabelType">
        <xsl:with-param name="Value">AttributeLabel</xsl:with-param>
      </xsl:call-template>
      <xsl:call-template name="Apply_compartmentFilter">
        <xsl:with-param name="Value">Attribute</xsl:with-param>
      </xsl:call-template>
      <xsl:element name="cLabel">
        <xsl:for-each select="u2:cAttributeLabel">
          <xsl:apply-templates select="*"/>
        </xsl:for-each>
      </xsl:element>
    </xsl:element>
    <!-- adding operation label compartment -->
    <xsl:element name="Compartment">
      <xsl:call-template name="Apply_compartmentLabelType">
        <xsl:with-param name="Value">OperationLabel</xsl:with-param>
      </xsl:call-template>
      <xsl:call-template name="Apply_compartmentFilter">
        <xsl:with-param name="Value">Operation</xsl:with-param>
      </xsl:call-template>
      <xsl:element name="cLabel">
      <xsl:for-each select="u2:cOperationLabel">
        <xsl:apply-templates select="*"/>
      </xsl:for-each>
    </xsl:element>
      </xsl:element>
      <!-- Moving existing compartments -->
      <xsl:for-each select="u2:cCompartment">
        <xsl:for-each select="u2:Compartment">
          <xsl:copy>
            <xsl:call-template name="copy-compartment-attributes"/>
            <xsl:choose>
              <xsl:when test="count(u2:cLabel/u2:ConstraintLabel)>0">
                <xsl:call-template name="Apply_compartmentLabelType">
                  <xsl:with-param name="Value">ConstraintLabel</xsl:with-param>
                </xsl:call-template>
                <xsl:call-template name="Apply_compartmentFilter">
                  <xsl:with-param name="Value">InformalConstraint</xsl:with-param>
                </xsl:call-template>
              </xsl:when>
              <xsl:otherwise>
                <xsl:call-template name="Apply_compartmentLabelType">
                  <xsl:with-param name="Value">StereotypeInstanceLabel</xsl:with-param>
                </xsl:call-template>
                <xsl:call-template name="Apply_compartmentFilter">
                  <xsl:with-param name="Value">InstanceExpr</xsl:with-param>
                </xsl:call-template>
              </xsl:otherwise>
            </xsl:choose>
            <xsl:apply-templates select="*"/>
          </xsl:copy>
        </xsl:for-each>
      </xsl:for-each>
    </xsl:element>
    <!-- Copying other elements -->
    <xsl:for-each select="*">
      <xsl:if test="not(self::u2:cAttributeLabel or self::u2:cOperationLabel or self::u2:cCompartment)">
        <xsl:copy>
          <xsl:call-template name="copy-all-attributes"/>
          <xsl:apply-templates/>
        </xsl:copy>
      </xsl:if>
    </xsl:for-each>
  </xsl:copy>   
</xsl:template>

<!-- Converting datatype symbol -->
<xsl:template match="u2:DataTypeSymbol">
  <xsl:copy>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:element name="cCompartment">
    <!-- adding literal label compartment -->
    <xsl:element name="Compartment">
      <xsl:call-template name="Apply_compartmentLabelType">
        <xsl:with-param name="Value">LiteralLabel</xsl:with-param>
      </xsl:call-template>
      <xsl:call-template name="Apply_compartmentFilter">
        <xsl:with-param name="Value">Literal</xsl:with-param>
      </xsl:call-template>
      <xsl:element name="cLabel">
        <xsl:for-each select="u2:cLiteralLabel">
          <xsl:apply-templates select="*"/>
        </xsl:for-each>
      </xsl:element>
    </xsl:element>
    <!-- adding operation label compartment -->
    <xsl:element name="Compartment">
      <xsl:call-template name="Apply_compartmentLabelType">
        <xsl:with-param name="Value">OperationLabel</xsl:with-param>
      </xsl:call-template>
      <xsl:call-template name="Apply_compartmentFilter">
        <xsl:with-param name="Value">Operation</xsl:with-param>
      </xsl:call-template>
      <xsl:element name="cLabel">
      <xsl:for-each select="u2:cOperationLabel">
        <xsl:apply-templates select="*"/>
      </xsl:for-each>
    </xsl:element>
      </xsl:element>
      <!-- Moving existing compartments -->
      <xsl:for-each select="u2:cCompartment">
        <xsl:for-each select="u2:Compartment">
          <xsl:copy>
            <xsl:call-template name="copy-compartment-attributes"/>
            <xsl:choose>
              <xsl:when test="count(u2:cLabel/u2:ConstraintLabel)>0">
                <xsl:call-template name="Apply_compartmentLabelType">
                  <xsl:with-param name="Value">ConstraintLabel</xsl:with-param>
                </xsl:call-template>
                <xsl:call-template name="Apply_compartmentFilter">
                  <xsl:with-param name="Value">InformalConstraint</xsl:with-param>
                </xsl:call-template>
              </xsl:when>
              <xsl:otherwise>
                <xsl:call-template name="Apply_compartmentLabelType">
                  <xsl:with-param name="Value">StereotypeInstanceLabel</xsl:with-param>
                </xsl:call-template>
                <xsl:call-template name="Apply_compartmentFilter">
                  <xsl:with-param name="Value">InstanceExpr</xsl:with-param>
                </xsl:call-template>
              </xsl:otherwise>
            </xsl:choose>
            <xsl:apply-templates select="*"/>
          </xsl:copy>
        </xsl:for-each>
      </xsl:for-each>
    </xsl:element>
    <!-- Copying other elements -->
    <xsl:for-each select="*">
      <xsl:if test="not(self::u2:cLiteralLabel or self::u2:cOperationLabel or self::u2:cCompartment)">
        <xsl:copy>
          <xsl:call-template name="copy-all-attributes"/>
          <xsl:apply-templates/>
        </xsl:copy>
      </xsl:if>
    </xsl:for-each>
  </xsl:copy>   
</xsl:template>

<!-- Converting event class symbol symbol -->
<xsl:template match="u2:TimerSymbol|u2:OperationSymbol|u2:StateMachineSymbol|u2:SignalSymbol">
  <xsl:copy>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:element name="cCompartment">
    <!-- adding parameter label compartment -->
    <xsl:element name="Compartment">
      <xsl:call-template name="Apply_compartmentLabelType">
        <xsl:with-param name="Value">ParameterLabel</xsl:with-param>
      </xsl:call-template>
      <xsl:call-template name="Apply_compartmentFilter">
        <xsl:with-param name="Value">Parameter</xsl:with-param>
      </xsl:call-template>
      <xsl:element name="cLabel">
        <xsl:for-each select="u2:cParameterLabel">
          <xsl:apply-templates select="*"/>
        </xsl:for-each>
      </xsl:element>
    </xsl:element>
      <!-- Moving existing compartments -->
      <xsl:for-each select="u2:cCompartment">
        <xsl:for-each select="u2:Compartment">
          <xsl:copy>
            <xsl:call-template name="copy-compartment-attributes"/>
            <xsl:choose>
              <xsl:when test="count(u2:cLabel/u2:ConstraintLabel)>0">
                <xsl:call-template name="Apply_compartmentLabelType">
                  <xsl:with-param name="Value">ConstraintLabel</xsl:with-param>
                </xsl:call-template>
                <xsl:call-template name="Apply_compartmentFilter">
                  <xsl:with-param name="Value">InformalConstraint</xsl:with-param>
                </xsl:call-template>
              </xsl:when>
              <xsl:otherwise>
                <xsl:call-template name="Apply_compartmentLabelType">
                  <xsl:with-param name="Value">StereotypeInstanceLabel</xsl:with-param>
                </xsl:call-template>
                <xsl:call-template name="Apply_compartmentFilter">
                  <xsl:with-param name="Value">InstanceExpr</xsl:with-param>
                </xsl:call-template>
              </xsl:otherwise>
            </xsl:choose>
            <xsl:apply-templates select="*"/>
          </xsl:copy>
        </xsl:for-each>
      </xsl:for-each>
    </xsl:element>
    <!-- Copying other elements -->
    <xsl:for-each select="*">
      <xsl:if test="not(self::u2:cParameterLabel or self::u2:cCompartment)">
        <xsl:copy>
          <xsl:call-template name="copy-all-attributes"/>
          <xsl:apply-templates/>
        </xsl:copy>
      </xsl:if>
    </xsl:for-each>
  </xsl:copy>   
</xsl:template>

<!-- Updating xmlNamespace stereotype  -->
<xsl:template match="u2:cStereotypeInstance">    
    <xsl:copy>
        <xsl:call-template name="copy-all-attributes"/>
        <xsl:for-each select="*">
                <xsl:choose>
                    <xsl:when test="self::u2:InstanceExpr[u2:rInstanceOf/@R = 'uid:@TTDXMLFramework@xmlNamespace']">
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:copy>
                            <xsl:call-template name="copy-all-attributes"/>
                            <xsl:apply-templates select="*"/>
                        </xsl:copy>
                    </xsl:otherwise>
                </xsl:choose>        
        </xsl:for-each>        
        <xsl:if test="count(u2:InstanceExpr[u2:rInstanceOf/@R = 'uid:@TTDXMLFramework@xmlNamespace']/u2:cExpression/*)>0">
            <xsl:element name="InstanceExpr">
                <xsl:element name="rInstanceOf">
                <xsl:attribute name="R">uid:@TTDXMLFramework@xmlNamespace</xsl:attribute>
                </xsl:element>
                <xsl:element name="cExpression">
                <xsl:element name="BinaryExpr">
                    <xsl:element name="rOperation">
                    <xsl:attribute name="R">uid:@Predefined@Collection@=@Collection@Collection@Collection</xsl:attribute>
                    </xsl:element>
                    <xsl:element name="cLeftOperand">
                        <xsl:element name="Ident">
                            <xsl:attribute name="Name">declarations</xsl:attribute>
                        </xsl:element>
                    </xsl:element>
                    <xsl:element name="cRightOperand">
                    <xsl:element name="ListExpr">
                    <xsl:element name="cExpression">
                    <xsl:for-each select="u2:InstanceExpr[u2:rInstanceOf/@R = 'uid:@TTDXMLFramework@xmlNamespace']/u2:cExpression">
                        <xsl:element name="InstanceExpr">
                        <xsl:element name="rInstanceOf">                        
                        <xsl:attribute name="R">uid:@TTDXMLFramework@xmlNamespaceDecl</xsl:attribute>
                        </xsl:element>                        
                        <xsl:copy>
                            <xsl:call-template name="copy-all-attributes"/>
                            <xsl:apply-templates select="*"/>
                        </xsl:copy>
                        </xsl:element>                        
                    </xsl:for-each>
                    </xsl:element>                    
                    </xsl:element>
                    </xsl:element>
                </xsl:element>
                </xsl:element>                
            </xsl:element>
        </xsl:if>
    </xsl:copy>    
 </xsl:template>
 
</xsl:transform>
