<?xml version="1.0" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

  <xsl:template name="cm_Description">
    <xsl:param name="guid" select="guid"/>
    <xsl:param name="description" select="description"/>
    <xsl:param name="fontColor" select="'blue'"/>
    <xsl:element name="a">
      <xsl:attribute name="href">
        <xsl:value-of select="'tlog:u2:#'"/>
        <xsl:value-of select="$guid"/>
      </xsl:attribute>
      <xsl:element name="font">
        <xsl:attribute name="color">
          <xsl:value-of select="$fontColor"/>
        </xsl:attribute>
        <xsl:value-of select="$description"/>
      </xsl:element>
    </xsl:element>
  </xsl:template>

  <xsl:template name="cm_ImageOrText">
    <xsl:param name="version"/>

    <xsl:for-each select="image">
      <xsl:choose>
        <xsl:when test=".=$version">
          <center><img scale="100%"><xsl:attribute name="src"><xsl:value-of select="@src"/></xsl:attribute></img></center>
        </xsl:when>
      </xsl:choose>
    </xsl:for-each>
    <xsl:for-each select="unparsed-text">
      <xsl:choose>
        <xsl:when test="@version=$version">
          <center><xsl:value-of select="."/></center>
        </xsl:when>
      </xsl:choose>
    </xsl:for-each>
  </xsl:template>

  <xsl:template name="cm_DifferenceNode">
    <xsl:param name="hasAncestor1"/>
    <xsl:param name="hasAncestor2"/>

    <xsl:if test="version='2'">
      <td>
        <xsl:call-template name="cm_ImageOrText">
          <xsl:with-param name="version" select="'version1'"/>
        </xsl:call-template>
      </td>
    </xsl:if>
    <td>
      <xsl:choose>
        <xsl:when test="version='1'">
          <xsl:call-template name="cm_ImageOrText">
            <xsl:with-param name="version" select="'version1'"/>
          </xsl:call-template>
        </xsl:when>
        <xsl:otherwise>
          <xsl:call-template name="cm_ImageOrText">
            <xsl:with-param name="version" select="'version2'"/>
          </xsl:call-template>
        </xsl:otherwise>
      </xsl:choose>
      <xsl:call-template name="cm_Description"/>
    </td>
    <xsl:if test="version='1'">
      <td>
        <xsl:call-template name="cm_ImageOrText">
          <xsl:with-param name="version" select="'version2'"/>
        </xsl:call-template>
      </td>
    </xsl:if>
    <xsl:if test="$hasAncestor1">
      <td>
        <xsl:call-template name="cm_ImageOrText">
          <xsl:with-param name="version" select="'ancestor'"/>
        </xsl:call-template>
      </td>
    </xsl:if>
    <xsl:if test="$hasAncestor2">
      <td>
        <xsl:call-template name="cm_ImageOrText">
          <xsl:with-param name="version" select="'ancestor2'"/>
        </xsl:call-template>
      </td>
    </xsl:if>
  </xsl:template>

  <xsl:template name="cm_ConflictNode">
    <xsl:param name="hasAncestor1"/>
    <xsl:param name="hasAncestor2"/>

    <xsl:for-each select="created[version=1]|modified[version=1]|deleted[version=1]|movedFrom[version=1]|movedTo[version=1]|inserted[version=1]|removed[version=1]">
      <td>
        <xsl:call-template name="cm_ImageOrText">
          <xsl:with-param name="version" select="'version1'"/>
        </xsl:call-template>
        <xsl:call-template name="cm_Description">
          <xsl:with-param name="fontColor" select="'maroon'"/>
        </xsl:call-template>
      </td>
    </xsl:for-each>
    <xsl:for-each select="created[version=2]|modified[version=2]|deleted[version=2]|movedFrom[version=2]|movedTo[version=2]|inserted[version=2]|removed[version=2]">
      <td>
        <xsl:call-template name="cm_ImageOrText">
          <xsl:with-param name="version" select="'version2'"/>
        </xsl:call-template>
        <xsl:call-template name="cm_Description">
          <xsl:with-param name="fontColor" select="'maroon'"/>
        </xsl:call-template>
      </td>
    </xsl:for-each>
    <xsl:if test="$hasAncestor1">
      <td>
        <xsl:for-each select="created[version=1]|modified[version=1]|deleted[version=1]|movedFrom[version=1]|movedTo[version=1]|inserted[version=1]|removed[version=1]">
          <xsl:call-template name="cm_ImageOrText">
            <xsl:with-param name="version" select="'ancestor'"/>
          </xsl:call-template>
        </xsl:for-each>
      </td>
    </xsl:if>
    <xsl:if test="$hasAncestor2">
      <td>
        <xsl:for-each select="created[version=2]|modified[version=2]|deleted[version=2]|movedFrom[version=2]|movedTo[version=2]|inserted[version=2]|removed[version=2]">
          <xsl:call-template name="cm_ImageOrText">
            <xsl:with-param name="version" select="'ancestor2'"/>
          </xsl:call-template>
        </xsl:for-each>
      </td>
    </xsl:if>
  </xsl:template>

  <xsl:template name="cm_ConsolidationNode">
    <xsl:param name="hasAncestor1"/>
    <xsl:param name="hasAncestor2"/>

    <xsl:for-each select="created[version=1]|modified[version=1]|deleted[version=1]|movedFrom[version=1]|movedTo[version=1]|inserted[version=1]|removed[version=1]">
      <td>
        <xsl:call-template name="cm_ImageOrText">
          <xsl:with-param name="version" select="'version1'"/>
        </xsl:call-template>
        <xsl:call-template name="cm_Description"/>
      </td>
    </xsl:for-each>
    <xsl:for-each select="created[version=2]|modified[version=2]|deleted[version=2]|movedFrom[version=2]|movedTo[version=2]|inserted[version=2]|removed[version=2]">
      <td>
        <xsl:call-template name="cm_ImageOrText">
          <xsl:with-param name="version" select="'version2'"/>
        </xsl:call-template>
        <xsl:call-template name="cm_Description"/>
      </td>
    </xsl:for-each>
    <xsl:if test="$hasAncestor1">
      <td>
        <xsl:for-each select="created[version=1]|modified[version=1]|deleted[version=1]|movedFrom[version=1]|movedTo[version=1]|inserted[version=1]|removed[version=1]">
          <xsl:call-template name="cm_ImageOrText">
            <xsl:with-param name="version" select="'ancestor'"/>
          </xsl:call-template>
        </xsl:for-each>
      </td>
    </xsl:if>
    <xsl:if test="$hasAncestor2">
      <td>
        <xsl:for-each select="created[version=2]|modified[version=2]|deleted[version=2]|movedFrom[version=2]|movedTo[version=2]|inserted[version=2]|removed[version=2]">
          <xsl:call-template name="cm_ImageOrText">
            <xsl:with-param name="version" select="'ancestor2'"/>
          </xsl:call-template>
        </xsl:for-each>
      </td>
    </xsl:if>
  </xsl:template>

  <xsl:template name="cm_CompositeConflictNode">
    <xsl:param name="hasAncestor1"/>
    <xsl:param name="hasAncestor2"/>

    <td>
      <table cellpadding="0" cellspacing="0">
        <xsl:for-each select="created[version=1]|modified[version=1]|deleted[version=1]|movedFrom[version=1]|movedTo[version=1]|inserted[version=1]|removed[version=1]">
          <tr>
            <td>
              <xsl:call-template name="cm_ImageOrText">
                <xsl:with-param name="version"  select="'version1'"/>
              </xsl:call-template>
              <xsl:call-template name="cm_Description">
                <xsl:with-param name="fontColor" select="'maroon'"/>
              </xsl:call-template>
            </td>
          </tr>
        </xsl:for-each>
      </table>
    </td>
    <td>
      <table cellpadding="0" cellspacing="0">
        <xsl:for-each select="created[version=2]|modified[version=2]|deleted[version=2]|movedFrom[version=2]|movedTo[version=2]|inserted[version=2]|removed[version=2]">
          <tr>
            <td>
              <xsl:call-template name="cm_ImageOrText">
                <xsl:with-param name="version" select="'version2'"/>
              </xsl:call-template>
              <xsl:call-template name="cm_Description">
                <xsl:with-param name="fontColor" select="'maroon'"/>
              </xsl:call-template>
            </td>
          </tr>
        </xsl:for-each>
      </table>
    </td>
    <xsl:if test="$hasAncestor1">
      <td>
      </td>
    </xsl:if>
    <xsl:if test="$hasAncestor2">
      <td>
      </td>
    </xsl:if>
  </xsl:template>

  <xsl:template name="cm_CompositeNode">
    <xsl:param name="hasAncestor1"/>
    <xsl:param name="hasAncestor2"/>

    <xsl:variable name="nSubNodes" select="count(created|modified|deleted|movedFrom|movedTo|inserted|removed|conflict|consolidation|composite-conflict)+1"/>
    
    <tr>
      <xsl:element name="td">
        <xsl:attribute name="rowspan">
          <xsl:value-of select="$nSubNodes"/>
        </xsl:attribute>
        <xsl:value-of select="@id"/>
      </xsl:element>

      <xsl:element name="td">
        <xsl:attribute name="rowspan">
          <xsl:value-of select="$nSubNodes"/>
        </xsl:attribute>
        <xsl:value-of select="description"/>
      </xsl:element>
    </tr>
    
    <xsl:for-each select="created|modified|deleted|movedFrom|movedTo|inserted|removed|conflict|consolidation|composite-conflict">
      <tr>
        <xsl:choose>
          <xsl:when test="local-name()='composite-conflict'">
            <xsl:call-template name="cm_CompositeConflictNode">
              <xsl:with-param name="hasAncestor1" select="$hasAncestor1"/>
              <xsl:with-param name="hasAncestor2" select="$hasAncestor2"/>
            </xsl:call-template>
          </xsl:when>
          <xsl:when test="local-name()='conflict'">
            <xsl:call-template name="cm_ConflictNode">
              <xsl:with-param name="hasAncestor1" select="$hasAncestor1"/>
              <xsl:with-param name="hasAncestor2" select="$hasAncestor2"/>
            </xsl:call-template>
          </xsl:when>
          <xsl:when test="local-name()='consolidation'">
            <xsl:call-template name="cm_ConsolidationNode">
              <xsl:with-param name="hasAncestor1" select="$hasAncestor1"/>
              <xsl:with-param name="hasAncestor2" select="$hasAncestor2"/>
            </xsl:call-template>
          </xsl:when>
          <xsl:otherwise>
            <xsl:call-template name="cm_DifferenceNode">
              <xsl:with-param name="hasAncestor1" select="$hasAncestor1"/>
              <xsl:with-param name="hasAncestor2" select="$hasAncestor2"/>
            </xsl:call-template>
          </xsl:otherwise>
        </xsl:choose>
      </tr>
    </xsl:for-each>

  </xsl:template>
  
  <xsl:template match="/">

    <xsl:variable name="hasAncestor1" select="/report/gui-compare[@mode!='2-way']"/>
    <xsl:variable name="hasAncestor2" select="/report/gui-compare[@mode='4-way']"/>

    <html>
      <head>
        <title>Compare review - difference-centric example stylesheet</title>
      </head>
      <body>
        <h1>
          <xsl:call-template name="cm_Description">
            <xsl:with-param name="guid" select="/report/gui-compare/project"/>
            <xsl:with-param name="description" select="'Difference table'"/>
          </xsl:call-template>
        </h1>
        <table border="1" cellpadding="2" cellspacing="0" frame = "box">
          <tr>
            <th>Id</th>
            <th>Diff</th>
            <th>Version 1</th>
            <th>Version 2</th>
            <xsl:if test="$hasAncestor1">
              <th>Ancestor</th>
            </xsl:if>
            <xsl:if test="$hasAncestor2">
              <th>Alternative Ancestor</th>
            </xsl:if>
          </tr>

          <xsl:for-each select="/report/gui-compare/composite-difference">          
            <xsl:call-template name="cm_CompositeNode">
              <xsl:with-param name="hasAncestor1" select="$hasAncestor1"/>
              <xsl:with-param name="hasAncestor2" select="$hasAncestor2"/>
            </xsl:call-template>
          </xsl:for-each>
        </table>
      </body>
    </html>
  </xsl:template>
</xsl:stylesheet>
