//////////////////////////////////////////////////////////////////////////
// This is a simple example of a console application that uses the 
// Tau C++ API in order to load a .u2 file and print the names of all 
// definitions it contains.
//////////////////////////////////////////////////////////////////////////

#include <iostream>
using namespace std;

#ifdef WIN32
#   define TCOUT wcout
#   define TCERR wcerr
#   include "Windows.h" // For conversions to/from multibyte strings
#   include "TCHAR.h"
#else
#   define TCOUT cout
#   define TCERR cerr
#endif


#include "U2ModelAccess.h"

// Class for handling callbacks from u2::MetaVisit used below.
class DefinitionFinder : public u2::ITtdMetaVisitCallback {
public:
    virtual bool OnVisitedEntity(u2::ITtdEntity* pEntity) {
	if (pEntity->IsKindOf(_T("Definition"))) {
	    tstring name;
	    pEntity->GetValue(_T("Name"), name);
	    TCOUT << name << endl;
	}
	return true;
    }

    virtual void OnAfterVisitedEntity(u2::ITtdEntity* pEntity) {
    }
};

int main(int argc, char* argv[])
{
    if (argc != 2) {
	TCERR << _T("Usage: apiPrintDefinitions <.u2 file>") << endl;
	return -1;
    }

    try {

	// First initialize the API
	u2::InitializeModelAccess();

	// Obtain ITtdModelAccess interface to start using the API
	u2::ITtdModelAccess* pModelAccess = u2::GetModelAccess();

#ifdef WIN32
	// On Windows the API uses multi-byte strings!	
	const char* s = argv[1];
        size_t nMbs = strlen(s) + 1;
	size_t nWs = MultiByteToWideChar(CP_UTF8, NULL, s, nMbs, NULL, 0);
	wchar_t* pszTmp = new wchar_t[nWs + 1];
	MultiByteToWideChar(CP_UTF8, NULL, s, nMbs, pszTmp, nWs);
	wstring inputFile(pszTmp);
	delete [] pszTmp;
#else
	// On Unix the API uses single-byte strings!
	string inputFile(argv[1]);
#endif

	u2::ITtdModel* pModel = pModelAccess->LoadFile(inputFile);
	
	u2::ITtdEntity* pEntity = u2::cast<u2::ITtdEntity>(pModel); // This cast is guaranteed to always work!

	DefinitionFinder finder;
	pEntity->MetaVisit(&finder);

	// Finally, finalize the API
	u2::FinalizeModelAccess();
    }
    catch (u2::APIError& e) {
	// An error occurred when using the API!
	TCERR << _T("API Error: ") << u2::GetAPIErrorMessage(e) << endl;
	return -1;
    }

    return 0;
}
