using System;
using System.Windows.Forms;
using U2ModelAccessTypeLib;

namespace PropEdCustomizationAgents
{
	/// <summary>
	/// Implementation of agent GetUserName - executes when the "Get username" button is pressed in the
	/// properties editor.
	/// </summary>
	/// 
	public class GetUserName : ITtdAgent
	{

	#region ITtdAgent Members

		public void Execute(ITtdEntity triggeredBy, object beforeProcessing, ITtdEntity modelContext, object server, ref Array eventProperties)
		{
			string s = "\"";
			s += Environment.UserName;
			s += "\"";
			if (modelContext != null) 
			{
				// Set the username as tagged value for 'Reviewer'
				modelContext.SetTaggedValue("ReviewComment (. Reviewer .)", s, true /* overwrite */);
			}
		}

		#endregion
	}

	/// <summary>
	/// Implementation of agent OnEnableCommentField - executes when the "Comment" field is about to be updated.
	/// </summary>
	/// 
	public class OnEnableCommentField : ITtdAgent
	{
		#region ITtdAgent Members

		public void Execute(ITtdEntity triggeredBy, object beforeProcessing, ITtdEntity modelContext, object server, ref Array eventProperties)
		{
			// Disable the comment field if the Reviewer field is empty
			if (modelContext != null) 
			{
				ITtdEntity reviewer = modelContext.GetTaggedValue("ReviewComment (. Reviewer .)", false /* guid */);
                if (reviewer == null || reviewer.GetValue("Value", 0) == "")
			    {
				    eventProperties.SetValue(false, 0);
			    }
			}
		}

		#endregion

	}

	/// <summary>
	/// Implementation of agent OnChangedComment - executes when the "Comment" field is changed.
	/// </summary>
	/// 
	public class OnChangedComment : ITtdAgent
	{
		#region ITtdAgent Members

		public void Execute(ITtdEntity triggeredBy, object beforeProcessing, ITtdEntity modelContext, object server, ref Array eventProperties)
		{
			// Count the number of characters in the new comment, and update the 'commentSize' tagged value.
			if (modelContext != null) 
			{
				int commentSize = 0;
				ITtdEntity newValue = (ITtdEntity) eventProperties.GetValue(1);
				if (newValue != null) {
					string commentString = newValue.GetValue("ValueString", 0);
					commentSize = commentString.Length;
				}

				string strCommentSize = commentSize.ToString();
				modelContext.SetTaggedValue("ReviewComment (. CommentSize .)", strCommentSize, true /* overwrite */);
			}
		}

		#endregion

	}
}
