#include "U2ModelAccess.h"
#include "U2Agent.h"
#include <time.h>

#ifndef UNICODE
#define tasctime asctime
#else
#define tasctime _wasctime
#endif

AGENT_IMPL(GetTimeStamp) {
    u2::ITtdModelAccess* pMA = u2::GetModelAccess();
    if (!pMA || !pContext)
        return;

    try {
        u2::ITtdEntity* pCreationTime = pContext->GetTaggedValue(_T("TimeStamp (. creationTime .)"));
        if (!pCreationTime) {
            pMA->WriteMessage(_T("No timestamp available!\n"));
            return;
        }

        tstring str;
        pCreationTime->GetValue(_T("ValueString"), str);
        tstring message(_T("This entity was created on "));
        message += str;
        pMA->WriteMessage(message);
        pMA->WriteMessage(_T("\n"));
    }
    catch (u2::APIError e) {
        pMA->WriteMessage(_T("API Error in GetTimeStamp agent:\n"));
        pMA->WriteMessage(u2::GetAPIErrorMessage(e));
        return;
    }
}

AGENT_IMPL(SetTimeStamp) {
    u2::ITtdModelAccess* pMA = u2::GetModelAccess();
    if (!pMA || !pContext)
        return;
    
    try {
        // Navigate to model from context
        u2::ITtdModel* pModel = pContext->GetModel();
        if (!pModel)
            return;

        // Find TimeStamp stereotype
        u2::ITtdEntity* pStereotype = pModel->FindByGuid(_T("9xToZVlAid5LemwWYEhuQcqI")); // This is the GUID of the TimeStamp stereotype
        if (!pStereotype)
            return;

        // Create an instance of the stereotype and apply it to the context entity
        u2::ITtdEntity* pInstance = pStereotype->CreateInstance();
        pContext->SetEntity(_T("StereotypeInstance"), pInstance);

        // Find out current date and time
        struct tm* localDateTime;
	    time_t currentDateTime;

	    time(&currentDateTime);   // Get time in seconds
	    localDateTime = localtime(&currentDateTime);   // Convert to struct tm form 
        tstring strDateTime(tasctime(localDateTime));
        // Remove the '\n' in strDateTime.
        size_t i = strDateTime.find_first_of( '\n' );
        strDateTime.erase( i, 1 );

        // Set the creationTime tagged value        
        strDateTime.insert(0, _T("\""));
        strDateTime.append(_T("\"")); // Quotes needed to make it a proper Charstring value
        pContext->SetTaggedValue(_T("TimeStamp (. creationTime .)"), strDateTime);
    }
    catch (u2::APIError e) {
        pMA->WriteMessage(_T("API Error in GetTimeStamp agent:\n"));
        pMA->WriteMessage(u2::GetAPIErrorMessage(e));
        return;
    }

}
